<?php
declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: fec7c6e6cc9f $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2020-02-28 10:06 +0100 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
abstract class we_backup_export{

	public static function export(): bool{
		static $db = 0;
		$db = $db ?: new DB_WE();
		$settings = we_backup_settings::i();
		$class = $settings->class;
		'@phan-var we_backup_exportBase $class';
		$log = $settings->backup_log;
		$export_binarys = $settings->option(we_backup_settings::OPTION_BACKUP_BINARY);
		$export_version_binarys = $settings->isHandled(we_backup_settings::OPTION_BACKUP_VERSION_BIN);
		$lines = $settings->backup_steps;

		if($settings->offset == 0){
			$table = we_backup_util::getNextTable();
			if(!$table){
				return false;
			}

			// export table
			if($log){
				we_backup_util::addLog(sprintf('Exporting table %s', $table));
			}
			$class->flush(true);

			$object = new we_backup_dbTable($table);

			$class->writeBackupData($object->getBackup(), $object->getDefaultTableName());
		}

		$table = (string) $settings->current_table;
		if(!$table){
			return false;
		}

		//sppedup for some tables
		switch($table){
			case LANGLINK_TABLE:
			case RECIPIENTS_TABLE:
			case HISTORY_TABLE:
			case CONTENT_TABLE:
			case PREFS_TABLE:
			case FILE_TABLE://since binary files can be large
			case VERSIONS_TABLE:
				$lines = $lines * 5;
				break;
		}

		// export table item
		$keys = we_backup_tableItem::getTableKey($table, false);
		//$generated = we_backup_tableItem::getGeneratedColumns($table, false);
		$keys_str = $db->escape('`' . implode('`,`', $keys) . '`');

		$data = $db->getAllq('SELECT * FROM ' . $db->escape($table) . ' ORDER BY ' . $keys_str . ' LIMIT ' . $settings->offset . ' ,' . $lines);

		foreach($data as $record){
			$object = new we_backup_tableItem($table, (array) $record);

			if($log){
				we_backup_util::addLog($object->getLogString('Exporting item '));
			}

			$class->writeBackupData($object->getBackup());

			switch($table){
				case FILE_TABLE:
					if($export_binarys && we_base_ContentTypes::isBinaryFile($object->getContentType())){
						$succ = $class->writeBinWEFile(($id = $object->getID()));
						if($log){
							we_backup_util::addLog('Exporting binary ' . $table . '(ID):' . $id . ' ' . ($succ ? 'ok' : 'fail')/* , we_base_file::getHumanFileSize($bin->getFilesize()) */, true);
						}

						//break 2; //return this fkt, since we may have taken to much time
					}
					break;

				case VERSIONS_TABLE:
					if($export_version_binarys){
						$succ = $class->writeBinWEFile(($id = $object->getID()), we_base_ContentTypes::VERSION);
						if($log){
							we_backup_util::addLog('Exporting version ' . $table . '(ID):' . $id . ' ' . ($succ ? 'ok' : 'fail'), true);
						}
						//break 2; //return this fkt, since we may have taken to much time
					}
					break;
			}

			$settings->offset++;
			$settings->row_count++;
		}

		$table_end = fI('SELECT COUNT(1) FROM ' . $db->escape($table));
		if($settings->offset >= $table_end){
			$settings->offset = 0;
		}
		$class->flush();
		return true;
	}

}
