<?php
declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: f389fd111aab $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2020-02-17 01:17 +0100 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
abstract class we_backup_tableMap{

	/**
	 *
	 * @param string[] $data
	 * @return string[]
	 */
	private static function replaceKeys(array $data): array{
		$ret = [];
		foreach($data as $val){
			$ret[strtolower(stripTblPrefix($val))] = $val;
		}
		return $ret;
	}

	//FIXME: use tables from modInfo!!
	/**
	 *
	 * @return array<string,array<string,string>>
	 */
	public static function getMap(): array{
		return [
			we_modules_info::CORE => array_merge([
				strtolower(stripTblPrefix(FILE_TABLE)) => FILE_TABLE,
				strtolower(stripTblPrefix(LANGLINK_TABLE)) => LANGLINK_TABLE,
				strtolower(stripTblPrefix(TEMPLATES_TABLE)) => TEMPLATES_TABLE,
				strtolower(stripTblPrefix(CONTENT_TABLE)) => CONTENT_TABLE,
				strtolower(stripTblPrefix(METADATA_TABLE)) => METADATA_TABLE,
				strtolower(stripTblPrefix(FILELINK_TABLE)) => FILELINK_TABLE,
				], self::replaceKeys(array_merge(we_modules_info::getModuleField(we_modules_info::CATEGORY, 'tables'), we_modules_info::getModuleField(we_modules_info::THUMB, 'tables'), we_modules_info::getModuleField(we_modules_info::NAVIGATION, 'tables'), we_modules_info::getModuleField(we_modules_info::COLLECTION, 'tables'), we_modules_info::getModuleField(we_modules_info::DOCTYPE, 'tables')
					)
			)),
			we_backup_settings::OPTION_BACKUP_VERSIONS => [
				strtolower(stripTblPrefix(VERSIONS_TABLE)) => VERSIONS_TABLE,
				strtolower(stripTblPrefix(VERSIONSLOG_TABLE)) => VERSIONSLOG_TABLE
			],
			we_backup_settings::OPTION_BACKUP_SETTINGS => [
				strtolower(stripTblPrefix(PREFS_TABLE)) => PREFS_TABLE,
				strtolower(stripTblPrefix(RECIPIENTS_TABLE)) => RECIPIENTS_TABLE,
				strtolower(stripTblPrefix(VALIDATION_SERVICES_TABLE)) => VALIDATION_SERVICES_TABLE,
				strtolower(stripTblPrefix(SETTINGS_TABLE)) => SETTINGS_TABLE,
			],
			we_modules_info::USERS => self::replaceKeys(we_modules_info::getModuleField(we_modules_info::USERS, 'tables')),
			we_backup_settings::OPTION_BACKUP_HISTORY => [strtolower(stripTblPrefix(HISTORY_TABLE)) => HISTORY_TABLE],
			we_modules_info::OBJECT => (we_modules_info::isActive(we_modules_info::OBJECT) ?
			array_merge(self::replaceKeys(we_modules_info::getModuleField(we_modules_info::OBJECT, 'tables')), [strtolower(stripTblPrefix(OBJECT_X_TABLE)) => OBJECT_X_TABLE]) :
			[]
			),
			we_modules_info::CUSTOMER => (we_modules_info::isActive(we_modules_info::CUSTOMER) ? self::replaceKeys(we_modules_info::getModuleField(we_modules_info::CUSTOMER, 'tables')) : [
			] ),
			we_modules_info::SHOP => (we_modules_info::isActive(we_modules_info::SHOP) ? self::replaceKeys(we_modules_info::getModuleField(we_modules_info::SHOP, 'tables')) : [
			] ),
			we_modules_info::WORKFLOW => (we_modules_info::isActive(we_modules_info::WORKFLOW) ? self::replaceKeys(we_modules_info::getModuleField(we_modules_info::WORKFLOW, 'tables')) : [
			] ),
			we_modules_info::NEWSLETTER => (we_modules_info::isActive(we_modules_info::NEWSLETTER) ? self::replaceKeys(we_modules_info::getModuleField(we_modules_info::NEWSLETTER, 'tables')) : [
			] ),
			we_modules_info::SCHEDULER => (we_modules_info::isActive(we_modules_info::SCHEDULER) ? self::replaceKeys(we_modules_info::getModuleField(we_modules_info::SCHEDULER, 'tables')) : [
			] ),
			we_modules_info::EXPORT => (we_modules_info::isActive(we_modules_info::EXPORT) ? self::replaceKeys(we_modules_info::getModuleField(we_modules_info::EXPORT, 'tables')) : [
			] ),
			we_modules_info::VOTING => (we_modules_info::isActive(we_modules_info::VOTING) ? self::replaceKeys(we_modules_info::getModuleField(we_modules_info::VOTING, 'tables')) : [
			] ),
			we_modules_info::GLOSSARY => (we_modules_info::isActive(we_modules_info::GLOSSARY) ? self::replaceKeys(we_modules_info::getModuleField(we_modules_info::GLOSSARY, 'tables')) : [
			] ),
			we_modules_info::SEARCH => self::replaceKeys(we_modules_info::getModuleField(we_modules_info::SEARCH, 'tables'))
		];
	}

}
