<?php

declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: 1079eab09220 $
 * $Author: Marc Kraemer <marc.kraemer@webedition.org> $
 * $Date: 2020-02-27 16:43 +0100 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */

/**
 * Class weVersion
 *
 * Provides functions for exporting and importing backups.
 */
class we_backup_version extends we_backup_contentBase{

	public const BACKUP_XML_TAG = 'we:version';

	/** @var string */
	var $ContentType = we_base_ContentTypes::VERSION;

	/** @var array<string,string> */
	var $persistent_slots = [
		'ID' => we_base_request::INT,
		'ContentType' => we_base_request::STRING,
		'Path' => we_base_request::STRING,
		'Data' => we_base_request::STRING,
	];

	/** @var int */
	var $ID = 0;

	/** @var string */
	var $Path = '';

	/** @var string */
	var $Data = '';

	public function __construct(int $id = 0){
		foreach(array_keys($this->persistent_slots) as $slot){
			$this->{$slot} = '';
		}
		if($id){
			$this->load($id, false);
		}
	}

	function load(int $id = 0, bool $loadData = true): bool{
		$this->ID = $id;
		$this->Path = fS('SELECT binaryPath FROM ' . VERSIONS_TABLE . ' WHERE ID=' . $id);
		return ($this->Path && $loadData ? $this->loadFile(WE_VERSION_PATH . $this->Path) : false);
	}

	function loadFile(string $file): bool{
		$this->Path = (string) str_ireplace([WE_DOCUMENT_ROOT, WE_VERSION_DIR], '', $file);
		try{
			return !empty($this->Data = we_base_file::load($file, 'rb', 8192, we_base_file::isCompressed($file)));
		}catch(we_exception_file $e){
			t_e($e);
			return false;
		}
	}

	/**
	 *
	 * @return array{id:int,path:string}
	 */
	protected function getBackupData(): array{
		return ['id' => $this->ID, 'path' => $this->Path];
	}

	public function setBackupData(array $data): bool{
		$this->ID = (int) $data['id'];
		$this->Path = (string) $data['path'];

		$path = WE_VERSION_PATH . $this->Path;
		if(!is_dir(dirname($path))){
			we_base_file::createLocalFolderByPath(dirname($path));
		}
		$class = we_backup_settings::i()->class;
		assert($class instanceof we_backup_importBase);
		$class->extractBinFile(we_backup_exportBase::VERSIONS_DIR, $this->Path);
		return true;
	}

	protected function getTable(): string{
		return VERSIONS_TABLE;
	}

}
