<?php
declare(strict_types=1);

/* webEdition CMS
 *
 * $Rev: 950b8bb84d05 $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2020-06-15 19:51 +0200 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */

abstract class we_base_showDocument{
	/** @var array<string,string> */
	static $urlReplace = [];

	/** @var bool */
	static $useGlossary = false;

	/** @var we_editor_base */
	static $include = null;

	private static function checkCustomerFilter(we_contents_root $we_doc): bool{
		if(!($we_doc instanceof we_webEditionDocument) || //@phan-suppress-current-line PhanImpossibleCondition
			!($we_doc instanceof we_contents_objectFile) || //@phan-suppress-current-line PhanImpossibleCondition
			!($we_doc instanceof we_document_binary) || //@phan-suppress-current-line PhanImpossibleCondition
			!$we_doc->documentCustomerFilter ||
			we_base_WE::$versioningActive){
			return true;
		}

		// call session_start to init session, otherwise NO customer can exist
		if(!isset($_SESSION)){
			we_base_WE::$session = new we_base_sessionHandler();
		}

		if(($visitorHasAccess = $we_doc->documentCustomerFilter->accessForVisitor($we_doc->ID))){
			$code = 0;
			switch($visitorHasAccess){
				case we_customer_documentFilter::ACCESS:
				case we_customer_documentFilter::CONTROLONTEMPLATE:
					return true;
				case we_customer_documentFilter::NO_LOGIN:
					$code = 401;
				//no break
				default:
					// user has NO ACCESS => show errordocument
					we_html_tools::setHttpCode($code ?: 403);
					$errorDocId = $we_doc->documentCustomerFilter->getErrorDoc($visitorHasAccess);
					if($errorDocId && ($errorDocPath = id2path($errorDocId, FILE_TABLE)) && file_exists(WE_DOCUMENT_ROOT . $errorDocPath)){ // use given document instead !
						unset($errorDocId, $_REQUEST);
						include(WE_DOCUMENT_ROOT . $errorDocPath);
						unset($errorDocPath);
						return false;
					}
					echo 'This document is not accessible';
					return false;
				//exit('Customer has no access to this document');
			}
		}
		return true;
	}

	public static function getWEDoc(int $we_ID, int $tmplID): we_contents_root{
//these come from external!
		$editmode = we_base_request::bool('we_cmd', false, 6);
		if($we_ID){
			$we_doc = we_document_base::initDoc('', $we_ID, FILE_TABLE);
		} else {
			$we_doc = we_contents_root::getSessionDocument(we_base_WE::$transaction);
		}
// init document

		$cmd = we_base_request::string('cmd', '');
		switch($cmd){
			case '':
			case 'ResetVersion':
			case 'PublishDocs':
				break;
			default:
				if(($we_doc instanceof we_document_webEdition && !$we_doc->IsDynamic) && (!$tmplID)){ // if the document is not a dynamic php-doc and is published we make a redirect to the static page
					header('Location: ' . ($we_doc->getDocumentState() === we_contents_root::STATE_PUBLISHED ? $we_doc->Path : '/this_file_does_not_exist_on_this_server'));
					exit();
				}
		}
		//check if customer has access
		if(!self::checkCustomerFilter($we_doc)){
			throw new we_exception_base('not allowed');
		}
//FIXME: is this relevant at this point?!
		$we_doc->EditPageNr = $editmode ? we_base_constants::WE_EDITPAGE_CONTENT : we_base_constants::WE_EDITPAGE_PREVIEW;

		if($tmplID && ($we_doc instanceof we_document_webEdition)){ // if the document should displayed with an other template
			$we_doc->setTemplateID($tmplID);
		}
		return $we_doc;
	}

	/**
	 * @return int Template-ID
	 */
	public static function getWEDocObj(): int{
		//FIXME: is it possible $we_doc is already set?!
		$userID = (we_base_WE::$we_doc && we_base_WE::$we_doc->InWebEdition ? we_base_WE::$we_doc->isLockedByUser() : 0);

		if(($userID && $userID != we_users_user::getID()) || (we_base_request::string('we_cmd', '', 0) === 'switch_edit_page' || (($_SESSION['weS']['EditPageNr'] ?? -1) == we_base_constants::WE_EDITPAGE_PREVIEW))){ //	Preview-Mode of Tabs
			//	We must choose the right template to show the object.
			//	Therefore we must look, if $_SESSION['weS']['SEEM']['LASTPATH'] exists to check the workspace.
			//	First check the workspaces for the document.
			//	choose the template matching to the actual workspace.
			//	If wrong workspace or no template can be found, just show the name/value pairs.
			// init document
			$we_transaction = we_base_request::_(we_base_request::TRANSACTION, 'we_transaction');
			if(!$we_transaction){
				exit();
			}
			$tid = 0;
			$obj = we_contents_root::getSessionDocument($we_transaction);
			assert($obj instanceof we_contents_objectFile);
			// FIXME: to use the following (legacy) code from WE < 8 following fix is needed (#11583)
			// FIXME: set Templates as object property on init
			we_base_WE::$we_obj = we_base_WE::$we_doc;
			$tids = $obj->getClassTemplates();
			'@phan-var int[] $tids';

			if($tids){
				$workspaces = $obj->getWorkspaces();
				'@phan-var int[] $workspaces';

				$tmpDB = new DB_WE();

				//	determine Path from last opened wE-Document
				$lastDoc = $_SESSION['weS']['last_webEdition_document'] ?? [];
				if(!empty($lastDoc['Path']) && $workspaces){// get the correct template
					$paths = id2pathA($workspaces, FILE_TABLE);
					//	Select a matching workspace.
					foreach($paths as $workspace){
						if(strpos($lastDoc['Path'], (string) $workspace) === 0 && $tids){
							//	init document
							$tid = (int) $tids[0];
							break;
						}
					}
				}

				if(!$tid && ($paths = id2pathA($tids, TEMPLATES_TABLE))){ //@phan-suppress-current-line PhanPartialTypeMismatchArgument
					$tid = (int) key($paths);
				}
				unset($tmpDB);
			}

			if(!$tid){
				self::showContent(we_base_WE::getWeObj());
				exit;
			}
			//init document
			//FIXME: wtf do we do here - hope the new way is correct
			//we_base_WE::$we_doc->we_initSessDat($we_dt);
			$doc = we_contents_root::getSessionDocument($we_transaction);
			assert($doc instanceof we_document_webEdition);
			we_base_WE::setWeDoc($doc);
			$_REQUEST[WEOBJ_PARAM] = $obj->ID;
		} else if(($we_transaction = we_base_request::_(we_base_request::TRANSACTION, 'we_cmd', '', 3))){
			$tid = we_base_request::int('we_cmd', 0, 2);
			// init document
			$doc = we_contents_root::getSessionDocument($we_transaction);
			assert($doc instanceof we_document_webEdition);
			we_base_WE::setWeDoc($doc);
		} else { //	view with template
			$tid = 0;
			$oid = we_base_request::int(WEOBJ_PARAM);
			we_base_WE::setWeObj(($obj = ($oid ? new we_contents_objectFile($oid) : null)));

			if(!$obj || !$obj->Publish_Date){
				we_html_tools::setHttpCode(404);

				if(ERROR_DOCUMENT_NO_OBJECTFILE && ($path = id2path(ERROR_DOCUMENT_NO_OBJECTFILE, FILE_TABLE))){
					//if set, we show object again!
					unset($_REQUEST);
					include(WE_DOCUMENT_ROOT . $path);
					exit();
				}
				echo 'Sorry, we are unable to locate your requested Page.';
				exit();
			}

			$obj->setTitleAndDescription();
			$doc = new we_document_triggerDocument(we_base_request::int('we_cmd', 0, 1));
			we_base_WE::setWeDoc($doc);
			$doc->initByObj($obj);
		}
		//check if customer has access
		try{
			return self::checkCustomerFilter(we_base_WE::getWeObj()) ? $tid : 0;
		} catch (we_exception_base $ex){
			return 0;
		}
	}

	private static function setBuffer(we_base_mainmodel $we_doc): void{
		self::$urlReplace = self::$urlReplace ?: we_folder_base::getUrlReplacements(DB_WE::$inst);
// --> Glossary Replacement
		self::$useGlossary = self::$useGlossary ?: ((we_modules_info::isActive(we_modules_info::GLOSSARY) && (!we_base_WE::$mainDoc['id'] || we_base_WE::$mainDoc['id'] == we_base_WE::$we_doc->ID)) && ($we_doc instanceof we_document_base && !$we_doc->InGlossar) && we_glossary_replace::useAutomatic());
		$useBuffer = !empty(self::$urlReplace) || self::$useGlossary;

		if($useBuffer){
			ob_start();
		}
	}

	public static function prePareInc(we_document_base $we_doc): we_editor_base{
		$we_include = $we_doc->editor();
		if($we_include instanceof we_editor_showTempDocumentData){
			self::setBuffer($we_doc);
		}
		return $we_include;
	}

	//FIXME: first should not be nullable. check why we can't handle it equally
	public static function prePareIncObj(we_contents_objectFile $we_obj, int $tid): we_editor_base{
		//if we don't have a tid yet, we get it from the workspace - we_cmd[1] is set in the dynamic document
		$tid = $tid ?: $we_obj->getTemplateID(fI('SELECT ParentID FROM ' . FILE_TABLE . ' WHERE ID=' . we_base_request::int('we_cmd', 0, 1)));

		$tmplPath = $tid ? preg_replace('/.tmpl$/i', '.php', fS('SELECT Path FROM ' . TEMPLATES_TABLE . ' WHERE ID=' . $tid)) : '';
		if(!$tid || !$tmplPath || !is_readable(TEMPLATES_PATH . $tmplPath)){
			we_html_tools::setHttpCode(SUPPRESS404CODE ? 200 : 404);

			if(ERROR_DOCUMENT_NO_OBJECTFILE && ($path = id2path(ERROR_DOCUMENT_NO_OBJECTFILE, FILE_TABLE))){
				//if set, we show object again!
				unset($_REQUEST);
				return (new we_editor_showTempDocumentData($we_obj))->setFile(WE_DOCUMENT_ROOT . $path);
			}
			throw new we_exception_file('no error file set');
		}
		$doc = we_base_WE::getWeDoc();
		if(!$doc->InWebEdition){
			self::setBuffer($doc);
		}
		return (new we_editor_showTempDocumentData($we_obj))->setFile(TEMPLATES_PATH . $tmplPath);
	}

	public static function postInc(): void{
		$useBuffer = !empty(self::$urlReplace) || self::$useGlossary;
		if(!$useBuffer){
			return;
		}
		$content = ob_get_clean() ?: '';

		if(self::$useGlossary){
			$content = we_glossary_replace::replace($content, we_base_WE::$we_doc->Language);
		}
		if(self::$urlReplace){
			$content = preg_replace(self::$urlReplace, array_keys(self::$urlReplace), $content);
		}
		echo $content;
	}

	private static function showContent(we_contents_objectFile $we_obj): void{
		$editor = new we_editor_contentobjectFile($we_obj);
		$editor->setPreview();
		echo $editor->show();
		exit;
	}

}
