<?php
declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: aacab26fe68a $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2020-05-05 10:25 +0200 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */

/**
 * Class we_category
 *
 * Provides functions for handling webEdition category.
 */
class we_category_category extends we_base_model implements we_backup_restoreIF, we_exim_xmlIF{
	public const BACKUP_XML_TAG = 'we:category';

	/** @var string */
	var $ContentType = we_base_ContentTypes::CATEGORY;
	//add all DB fields here, so we have default values in case it is a new object

	/** @var int */
	public $ID = 0;

	/** @var int */
	public $ParentID = 0;

	/** @var string */
	public $Text = '';

	/** @var string */
	public $Path = '';

	/** @var string */
	public $Title = '';

	/** @var string */
	public $Description = '';

	/** @var string */
	public $Language = '';

	function __construct(int $id = 0){
		parent::__construct(CATEGORY_TABLE);
		$this->persistent_slots = [
			'ID' => we_base_request::INT,
			'ParentID' => we_base_request::INT,
			'Text' => we_base_request::STRING,
			'Path' => we_base_request::STRING,
			'Title' => we_base_request::STRING,
			'Description' => we_base_request::WYSIWYG,
			'Language' => we_base_request::STRING
		];
		if($id){
			$this->load($id);
		}
	}

	function we_save(bool $resave = false, bool $skipHook = false): bool{
		$ret = parent::save();
		//FIXME:improve clean of nav Cache each time a category is saved!
		we_navigation_cache::clean(true);
		return $ret;
	}

	function delete(): bool{
		$ret = parent::delete();
		//FIXME:improve clean of nav Cache each time a category is deleted!
		we_navigation_cache::clean(true);
		return $ret;
	}

	/**
	 *
	 * @param string[] $catCSV
	 * @param string $alias
	 * @param bool $catOr
	 * @param string $fieldName
	 * @param int[] $categoryids
	 * @return string
	 */
	static function getCatSQLTail(array $catCSV, string $alias, bool $catOr = false, string $fieldName = 'Category', array $categoryids = [
		]): string{
		//FIXME: this will give us all cats, including subcats, check what we really want:
		//SELECT c2.ID FROM `tblCategorys` c1 LEFT JOIN tblCategorys c2 ON (c2.Path LIKE CONCAT(c1.Path,"/%") OR c1.ID=c2.ID) WHERE c1.ID IN (XX,YY,ZZ)

		$pre = ' FIND_IN_SET("';
		$post = '",' . $alias . '.' . $fieldName . ') ';

		$idarray = [];
		$folders = [];
		if($categoryids){
			$data = DB_WE::$inst->getAllq('SELECT ID,1 AS IsFolder,Path FROM ' . CATEGORY_TABLE . ' WHERE ID IN(' . implode(',', $categoryids) . ')');
			foreach($data as $cur){
				if($cur['IsFolder']){
					//all folders need to be searched in deep
					$catCSV[] = $cur['Path'];
				} else {
					$idarray[] = $cur['ID'];
				}
			}
		}

		if($catCSV){
			$idarray1 = array_unique($catCSV);
			foreach($idarray1 as $cat){
				$cat = '/' . trim($cat, '/ ');
				$isFolder = false;
				$tmp = [];
				$data = DB_WE::$inst->getAllq('SELECT ID,EXISTS (SELECT 1 FROM ' . CATEGORY_TABLE . ' cc WHERE c.ID=cc.ParentID) AS IsFolder FROM ' . CATEGORY_TABLE . ' c WHERE c.Path LIKE "' . DB_WE::$inst->escape($cat) . '/%" OR c.Path="' . DB_WE::$inst->escape($cat) . '"', false);
				foreach($data as $cur){
					$tmp[] = $cur['ID'];
					$isFolder = $isFolder || ((bool) $cur['IsFolder']); //@phan-suppress-current-line PhanImpossibleConditionInLoop
				}
				if($isFolder){
					$folders[] = $tmp;
				} else {
					$idarray = array_merge($idarray, $tmp);
				}
			}
		}
		if(empty($idarray) && empty($folders)){
			return '';
		}

		$where = [];
		if(!empty($idarray)){
			$where[] = $pre . implode($post . ($catOr ? ' OR ' : ' AND ') . $pre, array_unique($idarray)) . $post;
		}
		if(!empty($folders)){
			foreach($folders as &$cur){
				$where[] = '(CONCAT(",",' . $alias . '.' . $fieldName . ',",") REGEXP ",(' . implode('|', $cur) . '),")';
			}
			unset($cur);
		}

		return '(' . implode(($catOr ? ' OR ' : ' AND '), $where) . ' )';
	}

	public function registerMediaLinks(): bool{
		if($this->Description){
			$this->MediaLinks = array_values(we_wysiwyg_editor::getAllMediaLinks($this->Description));
		}

		$this->unregisterMediaLinks();
		return parent::registerMediaLinks();
	}

	/**
	 *
	 * @param int[] $catIDs
	 * @param string $rootdir
	 * @param string $parentDir
	 * @param string $order
	 * @return array<int,we_category_category|we_shop_category>
	 */
	static function we_getCatsFromIDs(array $catIDs, string $rootdir = '/', string $parentDir = '', string $order = ''): array{
		return ($catIDs ? self::we_getCategories($catIDs, $rootdir, $parentDir, false, $order) : []);
	}

	/**
	 *
	 * @param int[] $catIDs
	 * @param string $rootdir
	 * @param string $parentDir
	 * @param bool $includeDir
	 * @param string $order
	 * @return array<int,static>
	 */
	static function we_getCategories(array $catIDs, string $rootdir = '/', string $parentDir = '', bool $includeDir = false, string $order = ''): array{
		$db = DB_WE::$inst;
		$cats = [];
		$whereIDs = $catIDs ? ' ID IN(' . implode(',', $catIDs) . ')' : 1;
		$whereDir = ' AND Path LIKE "' . $db->escape($parentDir) . '/%"';
		$whereIncludeDir = $parentDir && $includeDir !== false ? ' OR (Path="' . $db->escape($parentDir) . '")' : '';
		$orderBy = $order ? ' ORDER BY ' . $db->escape($order) : '';

		$all = $db->getAllFirstq('SELECT * FROM ' . CATEGORY_TABLE . ' WHERE (' . $whereIDs . $whereDir . ')' . $whereIncludeDir . $orderBy, true, MYSQLI_ASSOC);
		foreach($all as $id => $data){
			$cat = new static();
			$cat->ID = (int) $id;
			$cat->Text = $data['Text'];
			$cat->Path = $rootdir && strpos($data['Path'], $rootdir) === 0 ? substr($data['Path'], strlen($rootdir)) : $data['Path'];
			$cat->ParentID = (int) $data['ParentID'];
			$cat->Title = $data['Title'];
			$cat->Description = we_document_base::parseInternalLinks($data['Description'], 0);
			$cat->Language = $data['Language'];
			$cats[$cat->ID] = $cat;
		}

		return $cats;
	}

	public static function getJSLangConsts(): string{
		return we_base_util::setLangString([
				'new_cat_name' => g_l('fileselector', '[new_cat_name]'),
				'we_filename_notValid' => (g_l('weEditor', '[category][we_filename_notValid]')),
				'delete_cat' => g_l('alert', '[delete][cat]'),
		]);
	}

	public function toXML(array $attribs = [], array $extraSlots = array(), string $content = ''): string{
		return parent::toXML($attribs, ['ContentType' => we_base_request::STRING]);
	}

	public function getPersistent(): array{
		return $this->persistent_slots;
	}

}
