<?php
declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: e5bf7f91c9c3 $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2019-12-02 13:06 +0100 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
abstract class we_customer_copyWeDocumentFilter{
	public const USE_CT = [
		we_base_ContentTypes::FOLDER_DOCUMENT,
		we_base_ContentTypes::FOLDER_OBJECTFILE,
		we_base_ContentTypes::WEDOCUMENT,
		we_base_ContentTypes::OBJECT_FILE,
		we_base_ContentTypes::APPLICATION,
		we_base_ContentTypes::PDF,
		we_base_ContentTypes::AUDIO,
		we_base_ContentTypes::VIDEO,
		we_base_ContentTypes::FLASH,
	];

	public static function getDialog(): void{
		// if any childs of the folder are open - bring message to close them
		// REQUEST[we_cmd][1] = id of folder
		// REQUEST[we_cmd][2] = table
		$db = DB_WE::$inst;
		$tableID = we_base_request::int('we_cmd', 0, 1);
		$table = we_base_request::_(we_base_request::TABLE, 'we_cmd', FILE_TABLE, 2);
		$idForFilter = we_base_request::int('we_cmd', 0, 3);

		if(we_base_request::bool('startCopy')){ // start the fragment
			if(!$idForFilter || !f('SELECT 1 FROM ' . CUSTOMER_FILTER_TABLE . ' WHERE modelTable="' . stripTblPrefix($table) . '" AND modelId=' . $idForFilter)){//delete all filters on documents
				$db->query('DELETE filt FROM ' . CUSTOMER_FILTER_TABLE . ' filt JOIN ' . $db->escape($table) . ' f ON (f.ID=filt.modelId AND modelTable="' . stripTblPrefix($table) . '") WHERE f.Path LIKE CONCAT(SELECT Path FROM ' . DB_WE::$inst->escape($table) . ' WHERE ID=' . $tableID . ',"/%")');
			} else {
				$db->query('
INSERT INTO ' . CUSTOMER_FILTER_TABLE . ' (modelTable,modelId,mode,specificCustomers,filter,whiteList,blackList,accessControlOnTemplate,errorDocNoLogin,errorDocNoAccess)(
SELECT
	filt.modelTable,f.ID,filt.mode,filt.specificCustomers,filt.filter,filt.whiteList,filt.blackList,filt.accessControlOnTemplate,filt.errorDocNoLogin,filt.errorDocNoAccess
FROM
	' . $db->escape($table) . ' f,
	' . CUSTOMER_FILTER_TABLE . ' filt
WHERE
	filt.modelTable="' . stripTblPrefix($table) . '" AND
	filt.modelId=' . $idForFilter . ' AND
	f.ContentType IN("' . implode('","', self::USE_CT) . '" ) AND
	f.Path LIKE CONCAT(SELECT Path FROM ' . DB_WE::$inst->escape($table) . ' WHERE ID=' . $tableID . ',"/%")
)');
			}

			$jsCmd = new we_base_jsCmd();
			$jsCmd->addMsg(g_l('modules_customerFilter', '[apply_filter_done]'), we_base_util::WE_MESSAGE_NOTICE);
			$jsCmd->addCmd('close_dialog');
			echo we_html_tools::getHtmlTop('', $jsCmd->getCmds(), we_html_element::body());
			return;
		}

// print the window
		// if we_cmd 3 is set, take filters of that folder as parent!!
		//$idForFilter = we_base_request::int('we_cmd', $id, 3);
		// now get all childs of this folder which are open by the user
		$allChildsJS = $db->getAllFirstq('SELECT
	ID,ContentType
FROM
	' . $db->escape($table) . ' f JOIN
	' . LOCK_TABLE . ' l ON (l.tbl="' . stripTblPrefix($table) . '" AND l.ID=f.ID AND l.UserID=' . $_SESSION['user']['ID'] . ')
WHERE
	f.ContentType IN("' . implode('","', self::USE_CT) . '" ) AND
	f.Path LIKE CONCAT(SELECT Path FROM ' . DB_WE::$inst->escape($table) . ' WHERE ID=' . $tableID . ',"/%")', false);

		$pb = new we_gui_progressBar(0, 300);
		$pb->addText('&nbsp;', we_gui_progressBar::TOP, 'copyWeDocumentCustomerFilterText');

		echo we_html_tools::getHtmlTop(g_l('modules_customerFilter', '[apply_filter]'), we_html_element::jsScript(WE_JS_MODULES_DIR . 'customer/copyWeDocumentFilter.js', '', [
				'id' => 'loadVarFilter',
				'data-filter' => setDynamicVar([
					'allChilds' => $allChildsJS,
					'table' => $table,
					'question' => g_l('modules_customerFilter', '[apply_filter_cofirm_close]'),
					'redirect' => WEBEDITION_DIR . 'we_cmd.php?we_cmd[0]=' . we_base_request::_(we_base_request::RAW, 'we_cmd', '', 0) . '&we_cmd[1]=' . $tableID . '&we_cmd[2]=' . $table . ($idForFilter ? '&we_cmd[3]=' . $idForFilter : '' ) . '&startCopy=1'
			])]) .
			we_gui_progressBar::getJSCode(), we_html_element::body([
				'class' => 'weDialogBody', 'onload' => "checkForOpenChilds()"
				], we_html_tools::dialogLayout($pb->getHTML(), g_l('modules_customerFilter', '[apply_filter]'), we_html_button::create_button(we_html_button::CANCEL, "javascript:WE().util.dialogTop(window).doClose();")) .
				'<div style="display: none;">
	<iframe style="position: absolute; top: 150px; height: 1px; width: 1px;" name="iframeCopyWeDocumentCustomerFilter" id="iframeCopyWeDocumentCustomerFilter" src="about:blank"></iframe>
</div>'));
	}

}
