<?php
declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: 41a611de91d8 $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2020-06-09 13:21 +0200 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
//@phan-file-suppress PhanUndeclaredClassMethod,PhanUndeclaredMethod,PhanUndeclaredExtendedClass
use Sabre\DAV;

class we_davfs_dbTemplate extends we_davfs_dbFile implements DAV\PartialUpdate\IPatchSupport{

	/**
	 * @return string
	 */
	function get(){
		$templ = new we_contents_template($this->ID);
		/* if($templ->isLockedByUser()){
		  throw new Exception\ConflictingLock();
		  } */
		return $templ->getElementS('data');
	}

	/**
	 * @param string|resource $data
	 * @return string|null
	 * @throws DAV\Exception\ConflictingLock
	 * @throws DAV\Exception\PreconditionFailed
	 */
	function put($data){
		$content = is_resource($data) ? (stream_get_contents($data) ?: '') : $data;
		$templ = new we_contents_template($this->ID);
		if($templ->isLockedByUser() || !$templ->lockDocument()){
			throw new Exception\ConflictingLock();
		}
		$templ->setElementS('data', $content);
		if(!$templ->we_publish()){
			throw new DAV\Exception\PreconditionFailed('internal error');
		}
		we_users_user::unlockDocuments([$this->ID], [$this->Table], we_base_WE::$transaction);
		return $this->getETag();
	}

	/**
	 *
	 * @param string $name
	 * @return string
	 * @throws DAV\Exception\ConflictingLock
	 * @throws DAV\Exception\PreconditionFailed
	 */
	public function setName($name){
		'@phan-var string $name';
		$templ = new we_contents_template($this->ID);
		if(!$templ->lockDocument()){
			throw new DAV\Exception\ConflictingLock();
		}
		$templ->Filename = preg_replace('|' . $templ->Extension . '$|', '', $name);
		$templ->Path = $templ->getPath();
		if(!$templ->we_save()){
			throw new DAV\Exception\PreconditionFailed('internal error');
		}
		we_users_user::unlockDocuments([$this->ID], [$this->Table], we_base_WE::$transaction);
		return $templ->getText();
	}

	/**
	 * Updates the file based on a range specification.
	 *
	 * The first argument is the data, which is either a readable stream
	 * resource or a string.
	 *
	 * The second argument is the type of update we're doing.
	 * This is either:
	 * * 1. append
	 * * 2. update based on a start byte
	 * * 3. update based on an end byte
	 * ;
	 * The third argument is the start or end byte.
	 *
	 * After a successful put operation, you may choose to return an ETag. The
	 * ETAG must always be surrounded by double-quotes. These quotes must
	 * appear in the actual string you're returning.
	 *
	 * Clients may use the ETag from a PUT request to later on make sure that
	 * when they update the file, the contents haven't changed in the mean
	 * time.
	 *
	 * @param resource|string $data
	 * @param int             $rangeType
	 * @param int             $offset
	 *
	 * @return string|null
	 */
	public function patch($data, $rangeType, $offset = null){
		$templ = new we_contents_template($this->ID);
		if($templ->isLockedByUser() || !$templ->lockDocument()){
			throw new DAV\Exception\ConflictingLock();
		}
		$content = $this->doPatch($templ->getElementS('data'), is_resource($data) ? (stream_get_contents($data) ?: '') : $data, $rangeType, $offset ?: 0);

		$templ->setElementS('data', $content);
		if(!$templ->we_publish()){
			throw new DAV\Exception\PreconditionFailed('internal error');
		}
		we_users_user::unlockDocuments([$this->ID], [$this->Table], we_base_WE::$transaction);
		return $this->getETag();
	}

}
