<?php
declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: 53a08b3d6828 $
 * $Author: Marc Krämer <marc.kraemer@aldente-it.de> $
 * $Date: 2019-09-09 17:57 +0200 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
class we_dialog_wysiwyg extends we_dialog_base{
	/** @var bool */
	protected $isFrontend = false;

	/** @var string */
	protected $name = '';

	/** @var string */
	protected $fieldName = '';

	/** @var array<string,mixed> */
	protected $dialogProperties = [];

	/** @var string */
	protected $editorType = '';

	protected function __construct(bool $noInternals = true, string $editorType = we_wysiwyg_editor::TYPE_INLINE_FALSE){
		parent::__construct($noInternals);

		$this->editorType = $editorType;
		if($this->editorType === we_wysiwyg_editor::TYPE_INLINE_FALSE){
			$this->dialogTitle = 'WYSIWYG-Editor';
		} else {
			$this->onOkJsOnly = true;
			$this->dialogTitle = g_l('wysiwyg', '[fullscreen_editor]');
			$this->bodyId = 'weFullscreenDialog';
		}

		$this->initByHttp();
		$this->isTinyDialog = $this->dialogProperties['weEditorType'] === 'fullscreen';
	}

	protected function initByHttp(): void{
		parent::initByHttp();

		$this->dialogProperties = we_unserialize((string) $this->args['dialogProperties']);

		$this->name = $this->dialogProperties['weName'];
		$this->fieldName = $this->dialogProperties['weFieldName'];
		$this->isFrontend = $this->dialogProperties['weIsFrontendEdit'];
	}

	protected function getJs(): string{
		return parent::getJs() .
			we_html_element::jsScript(JS_DIR . 'dialogs/we_dialog_wysiwyg.js', 'self.focus();') .
			we_wysiwyg_editor::getHTMLHeader($this->noInternals, true);
	}

	protected function getTinyMceDialogJS(): string{
		if($this->dialogProperties['weEditorType'] === 'fullscreen'){
			return parent::getTinyMceDialogJS();
		}

		return '';
	}

	protected function getDialogContentHTML(): string{
		$e = new we_wysiwyg_editor($this->dialogProperties, $this->dialogProperties['weEditorType']);

		return we_html_element::div(['id' => 'dialogBody_' . $this->name, 'style' => 'position:absolute;top:0;bottom:0;left:0;right:0;overflow:hidden;margin:0;'], $e->getHTML());
	}

	protected function getDialogHTML(): string{
		$dialogContent = we_html_tools::dialogLayout($this->getDialogContentHTML(), '<i class="fa fa-2x fa-spinner fa-pulse"></i>', $this->getDialogButtons());

		return $this->getFormHTML($dialogContent . we_html_element::hidden('we_what', 'cmd') . $this->getHiddenArgs());
	}

	protected function getFramesetHTML(string $extraBody = ''): string{
		return parent::getFramesetHTML();
	}

	protected function cmdFunction(array $args): string{
		switch($this->we_cmd[0] ?? ''){
			case 'writeBack_inlineFalse':
			case 'open_dialog_wysiwyg':
				// do we need this twice?
				$reqName = 'none';
				if(!$this->isFrontend){
					if(preg_match('%^(.+_te?xt)\[.+\]$%i', $this->name)){
						$reqName = preg_replace('/^(.+_te?xt)\[.+\]$/', '${1}', $this->name);
					} else if(preg_match('|^(.+_input)\[.+\]$|i', $this->name)){
						$reqName = preg_replace('/^(.+_input)\[.+\]$/', '${1}', $this->name);
					}
				} else {
					$reqName = str_replace('[' . $this->fieldName . ']', '', $this->name);
				}

				$value = preg_replace('|(</?)script([^>]*>)|i', '${1}scr"+"ipt${2}', strtr(we_base_request::_(we_base_request::RAW_CHECKED, $reqName, '', $this->fieldName), [
					//"\r" => '\r',
					//"\n" => '\n',
					"'" => '&#039;'
				]));
				$replacements = [
					//'"' => '\"',
					"\xe2\x80\xa8" => '',
					"\xe2\x80\xa9" => '',
				];
				$taValue = strtr($value, $replacements);
				$divValue = $this->isFrontend ? $taValue : strtr(we_document_base::parseInternalLinks($value, 0), $replacements);

				return we_html_tools::getHtmlTop($this->fieldName, we_html_element::jsScript(JS_DIR . 'dialogs/we_dialog_cmdFrame.js', "we_cmd('wysiwyg_writeBack')", [
							'id' => 'loadVarDialog_cmdFrame',
							'data-payload' => setDynamicVar(['textareaValue' => $taValue,
								'divValue' => $divValue,
								'name' => $this->name,
								'isFrontendEdit' => $this->isFrontend ? 1 : 0]),
							'editorTyoe' => $this->editorType
						]), we_html_element::body()
				);
		}
		return'';
	}

}
