<?php
declare(strict_types=1);
we_html_tools::protect([we_base_perm::WORK_BROWSE_SERVER]);

/**
 * webEdition CMS
 *
 * $Rev: 69557c160c33 $
 * $Author: Marc Krämer <marc.kraemer@aldente-it.de> $
 * $Date: 2019-09-06 01:06 +0200 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
class we_editor_selectorEdit extends we_editor_base{
	/** @var string */
	private $id = '';

	/** @var bool */
	private $isSave = false;

	public function __construct(?we_base_mainmodel $we_doc, ?we_base_jsCmd $jsCmd = null, string $we_transaction = ''){
		parent::__construct($we_doc, $jsCmd, $we_transaction);
		$this->id = we_base_request::_(we_base_request::FILE, 'id');
	}

	public function show(): string{
		$we_fileData = $this->getContent();
		return $this->getPage(
				we_html_tools::dialogLayout(
					'<textarea name="editFile" id="editFile" style="width:540px;height:380px;overflow: auto;">' . htmlspecialchars($we_fileData) . '</textarea>', g_l('global', '[edit_file]') . ': ' .
					we_html_element::span(['class' => "weMultiIconBoxHeadline"], str_replace(dirname($this->id) . '/', '', $this->id) . we_html_button::position_yes_no_cancel(we_html_button::create_button(we_html_button::SAVE, "javascript:document.forms[0].submit();"), '', we_html_button::create_button(we_html_button::CANCEL, "javascript:WE().util.dialogTop(window).doClose();")))) .
				we_html_element::hidden('cmd', 'save'), we_html_element::jsScript(JS_DIR . 'selectorEdit.js', ($this->isSave ? 'init();' : '')), ['onresize' => "setSize()",
				'style' => "width:100%; height:100%;text-align:center;"], [
				'onsubmit' => '']);
	}

	private function getContent(): string{
		if(we_base_request::string('cmd') === 'save'){
			$data = we_base_request::_(we_base_request::RAW_CHECKED, 'editFile');
			if($data !== false){
				we_base_file::save($this->id, $data);
				$this->isSave = true;
				return stripslashes($data);
			}
			return '';
		}
		if(!$this->id){
			return '';
		}

		try{
			if($this->id){
				return we_base_file::load($this->id);
			}
		} catch (we_exception_file $e){

			$this->jsCmd->addMsg(sprintf(g_l('alert', '[can_not_open_file]'), str_replace(str_replace('\\', '/', dirname($this->id)) . '/', '', $this->id), 1), we_base_util::WE_MESSAGE_ERROR);
			$this->jsCmd->addCmd('close');
		}
		return '';
	}

}
