<?php
declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: b26067fc7281 $
 * $Author: Marc Kraemer <marc.kraemer@webedition.org> $
 * $Date: 2020-02-04 13:57 +0100 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
abstract class we_exim_ExIm{
	/** @var string */
	const INFO_TAG = 'we:info';

	/** @var string */
	const MAP_TAG = 'we:map';
	const COLLISION_RENAME = 'rename';
	const COLLISION_OVERWRITE = 'overwrite';
	const COLLISION_REPLACE = 'replace';
	const COLLISION_SKIP = 'skip';

	/** @var array<string,bool> */
	var $destination = [];

	/** @var we_exim_refTable */
	var $RefTable;

	/** @var int */
	var $chunk_count;

	/** @var int */
	var $chunk_number;

	/** @var array<string,int[]> */
	var $analyzed = [];

	/** @var int */
	var $level = 0;

	//var $recover_mode=0; // 0 -	save all to selected folder; 1 - save with given path
	/** @var string */
	const TYPE_CSV = 'CSV';

	/** @var string */
	const TYPE_XML = 'XML';

	/** @var string */
	const TYPE_WE = 'WE';

	/** @var string */
	const TYPE_ODS = 'ODS';

	/** @var string */
	const TYPE_LOCAL_FILES = 'FileImport';

	/** @var string */
	const TYPE_SITE = 'siteImport';

	/** @var array<string,mixed> */
	var $options = [
		'handle_paths' => false,
		'handle_def_templates' => false,
		'handle_doctypes' => false,
		'handle_categorys' => false,
		'handle_def_classes' => false,
		'handle_binarys' => false,
		'update_mode' => 0,
		'handle_document_includes' => false,
		'handle_document_linked' => false,
		'handle_object_includes' => false,
		'handle_object_embeds' => false,
		'handle_class_defs' => false,
		'export_depth' => 1,
		'handle_documents' => false,
		'handle_templates' => false,
		'handle_classes' => false,
		'handle_objects' => false,
		'handle_content' => false,
		'handle_table' => false,
		'handle_tableitems' => false,
		'handle_doc_paths' => false,
		'handle_templ_paths' => false,
		'document_path' => 0,
		'template_path' => 0,
		'handle_collision' => '',
		'restore_doc_path' => true,
		'restore_tpl_path' => true,
		'handle_owners' => false,
		'owners_overwrite' => 0,
		'owners_overwrite_id' => 0,
		'handle_navigation' => false,
		'navigation_path' => '',
		'handle_thumbnails' => false,
		'change_encoding' => false,
		'xml_encoding' => '',
		'target_encoding' => '',
		'rebuild' => 1,
		'xml_cdata' => 1,
		'xml_table' => 'tblFile',
		'csv_delimiter' => 'colon',
		'csv_lineend' => 'windows',
		'csv_enclose' => 'doublequote',
		'csv_fieldnames' => 1
	];

	function __construct(string $file = ''){
		$this->RefTable = new we_exim_refTable();
		if($file){
			$this->loadPreserves();
		}

		$this->destination[strtolower(FILE_TABLE)] = false;
		$this->destination[strtolower(TEMPLATES_TABLE)] = false;
		$this->destination[strtolower(DOC_TYPES_TABLE)] = false;
		$this->destination[strtolower(OBJECT_TABLE)] = false;
		$this->destination[strtolower(OBJECT_FILES_TABLE)] = false;
	}

	function setOptions(array $options): void{
		foreach($options as $k => $v){
			if(isset($this->options[$k])){
				$this->options[$k] = $v;
			}
		}
	}

	function setBackupProfile(): void{
		$options = [
			'handle_content' => true,
			'handle_table' => true,
			'handle_tableitems' => true,
			'handle_binarys' => true,
		];

		$this->setOptions($options);
	}

	public static function getTableForCT(string $we_ContentType, string $table = ''): string{
		switch($we_ContentType){
			case we_base_ContentTypes::DOCTYPE:
				return DOC_TYPES_TABLE;
			case we_base_ContentTypes::CATEGORY:
				return CATEGORY_TABLE;
			case we_base_ContentTypes::OBJECT:
				return OBJECT_TABLE;
			case we_base_ContentTypes::TEMPLATE:
				return TEMPLATES_TABLE;
			case we_base_ContentTypes::OBJECT_FILE:
				return OBJECT_FILES_TABLE;
			case we_base_ContentTypes::BACKUP_BINARY:
				return '';
			case we_base_ContentTypes::NAVIGATION:
				return NAVIGATION_TABLE;
			case we_base_ContentTypes::NAVIGATIONRULE:
				return NAVIGATION_RULE_TABLE;
			case we_base_ContentTypes::THUMBNAIL:
				return THUMBNAILS_TABLE;
			case we_base_ContentTypes::FOLDER_COMMON:
			case we_base_ContentTypes::FOLDER_DOCUMENT:
			case we_base_ContentTypes::FOLDER_TEMPLATE:
			case we_base_ContentTypes::FOLDER_OBJECTFILE:
			case we_base_ContentTypes::FOLDER_VFILE:
				if(!empty($table)){
					return $table;
				}
			//intentionally no break
			default:
				return FILE_TABLE;
		}
	}

	public function loadPreserves(): void{
		if(isset($_SESSION['weS']['ExImRefTable'])){
			$this->RefTable = $_SESSION['weS']['ExImRefTable'];
		}
	}

	public function savePreserves(): void{
		$_SESSION['weS']['ExImRefTable'] = $this->RefTable;
	}

	public static function unsetPreserves(): void{
		if(isset($_SESSION['weS']['ExImRefTable'])){
			unset($_SESSION['weS']['ExImRefTable']);
		}
	}

	//FIXME given parameter is not used in the call stack!
	public function prepareExport(array $ids = []): void{
		$this->RefTable = new we_exim_refTable();
		$preparer = new we_export_preparer();
		$preparer->prepareExport($ids);
	}

	static function getHeader(string $type = '', bool $skipWE = false): string{
		return '<?xml version="1.0" encoding="UTF-8" standalone="yes"?>' . "\n" .
			($skipWE ? '' : we_backup_util::weXmlExImHead . ' version="' . WE_VERSION . '" type="' . $type . '" xmlns:we="we-namespace">' . "\n");
	}

	static function getFooter(): string{
		return we_backup_util::weXmlExImFooter;
	}

	public static function queryForAllowed(string $table): string{
		$db = new DB_WE();
		$wsQuery = [];
		if(($ws = we_users_util::get_ws($table))){
			$wsPathArray = id2pathA($ws, $table);
			$parents = [];
			foreach($wsPathArray as $path){
				'@phan-var string $path';
				$wsQuery[] = 'Path LIKE "' . $db->escape($path) . '/%"';
				$parents = array_merge($parents, we_tree_base::getParents($path));
			}
			$parents = array_unique($parents);
			if($parents){
				$wsQuery[] = 'Path IN(' . implode(',', $parents) . ')';
			}
		} else if($table === OBJECT_FILES_TABLE && (!we_base_permission::hasPerm([we_base_perm::ADMINISTRATOR]))){
			$classes = we_users_util::getAllowedClasses();
			if($classes){
				$wsQuery[] = 'TableID IN (' . implode(',', $classes) . ')';
			}
		}

		return ' AND (1 ' . we_users_util::makeOwnersSql() . ( $wsQuery ? ' OR (' . implode(' OR ', $wsQuery) . ')' : '') . ')';
	}

	function saveObject(we_base_mainmodel $object): bool{
		// save binary data first to stay compatible with the new binary feature in v5.1
		if($object instanceof we_document_binary){
			$object->savebinarydata();
		}

		if(constant(get_class($object) . '::BACKUP_XML_TAG') === we_doctype_docTypes::BACKUP_XML_TAG){
			assert($object instanceof we_doctype_docTypes);
			return $object->we_save_exim();
		}
		if($object instanceof we_document_webEdition){
			we_base_WE::setWeDoc($object);
		}
		if(
			$object instanceof we_backup_restoreIF && !$object->we_save() &&
			$object instanceof we_contents_base && !$object->we_publish()){
			return false;
		}

		if($object instanceof we_document_binary){
			$object->setElementS('data', '');
		}

		return true;
	}

	public static function getExtensionByType(string $type): string{
		switch($type){
			case self::TYPE_CSV:
				return '.csv';
			case self::TYPE_XML:
				return '.xml';
			case self::TYPE_WE:
				return '.zip';
			case self::TYPE_ODS:
				return '.ods';
			default:
				return '';
		}
	}

}
