<?php
declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: b26067fc7281 $
 * $Author: Marc Kraemer <marc.kraemer@webedition.org> $
 * $Date: 2020-02-04 13:57 +0100 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
class we_fragment_copyFolderFinish extends we_fragment_copyFolder{

	protected function init(): void{
		if(isset($_SESSION['weS']['WE_CREATE_TEMPLATE'])){
			$this->alldata = [];
			foreach($_SESSION['weS']['WE_CREATE_TEMPLATE'] as $id){
				$this->alldata[] = $id;
			}
			unset($_SESSION['weS']['WE_CREATE_TEMPLATE']);
		}
	}

	protected function doTask(): void{
		if(!$this->correctTemplate()){
			t_e("Error correcting Template with id: " . $this->data);
			exit("Error correcting Template with id: " . $this->data);
		}
	}

	protected function updateProgressBar(): void{
		$this->jsCmd->addCmd('setProgress', [
			'progress' => ((int) ((100 / count($this->alldata)) * (1 + $this->currentTask))),
			'name' => 'pbar1',
			'text' => sprintf(g_l('copyFolder', '[correctTemplate]'), basename(id2path($this->data, TEMPLATES_TABLE)))
		]);
	}

	private function correctTemplate(): bool{
		$templ = new we_contents_template($this->data);
		$content = $templ->getElementS('data');
		$regs = [];
		if(preg_match_all('/' . we_base_constants::PLACEH_PATH . '([^ ]+) ' . we_base_constants::PLACEH_PATH . '/i', $content, $regs, PREG_SET_ORDER)){
			foreach($regs as $cur){
				$path = $cur[1];
				$id = (string) path_to_id($path, FILE_TABLE);
				$content = str_replace(we_base_constants::PLACEH_PATH . $path . ' ' . we_base_constants::PLACEH_PATH, $id, $content);
			}
		}
		$templ->setElementS('data', $content);
		return $templ->we_save();
	}

	protected function finish(): void{
		if(isset($_SESSION['weS']['WE_CREATE_TEMPLATE'])){
			unset($_SESSION['weS']['WE_CREATE_TEMPLATE']);
		}
		$this->jsCmd->addCmd('we_cmd', ['load', FILE_TABLE]);
		$this->jsCmd->addMsg(g_l('copyFolder', '[copy_success]'), we_base_util::WE_MESSAGE_NOTICE);
		$this->jsCmd->addCmd('close');
	}

}
