<?php
declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: 6917b9add631 $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2020-04-21 13:24 +0200 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
abstract class we_glossary_frameEditorFolder extends we_glossary_frameEditor{

	public static function Header(we_glossary_frames $weGlossaryFrames): string{
		$we_tabs = new we_gui_tabs();
		$we_tabs->addTab(g_l('modules_glossary', '[overview]'), '', true, '1');
		$frontendL = we_base_country::getWeFrontendLanguagesForBackend();

		return self::buildHeader($weGlossaryFrames, $we_tabs, g_l('modules_glossary', '[folder]'), $frontendL[substr(we_base_request::string('cmdid'), 0, 5)]);
	}

	public static function Body(we_glossary_frames $weGlossaryFrames, we_glossary_view $view): string{
		$weGlossaryFrames->jsCmd->addCmd('loadHeaderFooter', 'folder', we_base_request::string('cmdid'));
		return self::buildBody($weGlossaryFrames, $view, we_html_element::div(['id' => 'tab1', 'style' => ''],
																																				(new we_html_multiIconBox(self::getHTMLOverview($weGlossaryFrames)))->marginLeft()->get()), we_html_element::jsScript(WE_JS_MODULES_DIR . 'glossary/we_glossary_frameEditorType.js'));
	}

	private static function getHTMLOverview(we_glossary_frames $weGlossaryFrames): we_html_multiIconBoxContent{
		$cmdid = we_base_request::string('cmdid');
		$list = [
			we_glossary_glossary::TYPE_ABBREVATION,
			we_glossary_glossary::TYPE_ACRONYM,
			we_glossary_glossary::TYPE_FOREIGNWORD,
			we_glossary_glossary::TYPE_LINK,
			we_glossary_glossary::TYPE_TEXTREPLACE,
		];

		$language = DB_WE::$inst->escape(substr(we_base_request::string('cmdid'), 0, 5));

		$parts = new we_html_multiIconBoxContent();

		foreach($list as $key){
			$items = fI('SELECT COUNT(1) FROM ' . GLOSSARY_TABLE . ' WHERE Language="' . $language . '" AND Type="' . $key . '"');
//FIXME createbuttontable?
			$button = we_html_button::create_button('new_glossary_' . $key, "javascript:top.opener.top.we_cmd('new_glossary_" . $key . "', '" . $cmdid . "');", '', "", !we_base_permission::hasPerm([
						we_base_perm::GLOSSARY_NEW]));

			$parts->add((new we_html_multiIconBoxContentLine('<a href="javascript://" onclick="top.content.editor.edbody.location=\'' . $weGlossaryFrames->getFrameset() . '&pnt=edbody&cmd=we_glossary_view_type&cmdid=' . $cmdid . '_' . $key . '&tabnr=\'+top.content.activ_tab;">' . g_l('modules_glossary', '[' . $key . ']') . '</a>', '<table style="width:550px;" class="default defaultfont">
	<tr><td>' . g_l('modules_glossary', '[' . $key . '_description]') . '</td></tr>
	<tr><td>' . g_l('modules_glossary', '[number_of_entries]') . ': ' . $items . '</td></tr>
	<tr><td style="text-align:right">' . $button . '</td></tr>
</table>'))
					->space(we_html_multiIconBox::SPACE_MED));
		}

		return $parts;
	}

}
