<?php
declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: 2aeecd34c7bc $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2020-06-24 19:23 +0200 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */

/**
 * Class we_forms
 *
 * Provides functions for creating html tags used in forms.
 */
abstract class we_html_forms{

	/**
	 * @param      $value                                  string
	 * @param      $checked                                bool
	 * @param      $name                                   string
	 * @param      $text                                   string
	 * @param      $uniqid                                 bool                (optional)
	 * @param      $class                                  string              (optional)
	 * @param      $onClick                                string              (optional)
	 * @param      $disabled                               bool                (optional)
	 *
	 * @return     string
	 */
	static function checkbox(string $value, bool $checked, string $name, string $text, bool $uniqid = false, string $class = '', string $onClick = '', bool $disabled = false, string $description = '', int $type = we_html_tools::TYPE_NONE, /* mixed */ $width = 0, string $html = '', string $style = '', string $title = ''): string{
		// Check if we have to create a uniqe id
		$id = ($uniqid ? $name . '_' . md5(uniqid(__FUNCTION__, true)) : $name);

		// Create HTML tags
		return we_html_element::div(['class' => 'default checkbox', 'style' => $style],
															'<input type="checkbox" name="' . $name . '" id="' . $id . '" value="' . $value . '" ' . ($checked ? ' checked="checked"' : '') . ($onClick ? ' onclick="' . $onClick . '"' : '') . ($disabled ? ' disabled="disabled"' : "") . ' />' .
				we_html_element::div(['class' => 'elementText ' . $class], we_html_element::label([
						'id' => 'label_' . $id, 'for' => $id, 'class' => ($disabled ? 'disabled ' : '') . ($title ? ' showhelp' : ''), 'title' => $title],
																																											$text) . ($html ?: '')) .
				($description ? we_html_tools::alertAttentionBox($description, $type, $width) : '')
		);
	}

	/**
	 * @param      $checked                                bool
	 * @param      $name                                   string
	 * @param      $text                                   string
	 * @param      $uniqid                                 bool                (optional)
	 * @param      $class                                  string              (optional)
	 * @param      $onClick                                string              (optional)
	 * @param      $disabled                               bool                (optional)
	 *
	 * @return     string
	 */
	static function checkboxWithHidden(bool $checked, string $name, string $text, bool $uniqid = false, string $class = '', string $onClick = '', bool $disabled = false, string $description = '', int $type = we_html_tools::TYPE_NONE, /* mixed */ $width = 0, string $html = '', string $style = '', string $title = ''): string{
		$onClick = "this.form.elements['" . $name . "'].value=this.checked ? 1 : 0;" . $onClick;
		return
			we_html_element::hidden($name, ($checked ? 1 : 0)) .
			self::checkbox('1', $checked, 'check_' . $name, $text, $uniqid, $class, $onClick, $disabled, $description, $type, $width, $html, $style, $title);
	}

	static function radiobuttonV(string $value, string $curVal, string $name, string $text, bool $uniqid = true, string $class = '', string $onClick = '', bool $disabled = false, string $description = '', int $type = we_html_tools::TYPE_NONE, /* mixed */ $width = 0, string $onMouseUp = '', string $extra_content = '', string $title = ''): string{
		return self::radiobutton($value, $value === $curVal, $name, $text, $uniqid, $class, $onClick, $disabled, $description, $type, $width, $onMouseUp, $extra_content, $title);
	}

	/**
	 * @param      $value                                  string
	 * @param      $checked                                bool
	 * @param      $name                                   string
	 * @param      $text                                   string
	 * @param      $uniqid                                 bool                (optional)
	 * @param      $class                                  string              (optional)
	 * @param      $onClick                                string              (optional)
	 * @param      $disabled                               bool                (optional)
	 *
	 * @return     string
	 */
	static function radiobutton(string $value, bool $checked, string $name, string $text, bool $uniqid = true, string $class = '', string $onClick = '', bool $disabled = false, string $description = '', int $type = we_html_tools::TYPE_NONE, /* mixed */ $width = 0, string $onMouseUp = '', string $extra_content = '', string $title = ''): string{
		// Check if we have to create a uniqe id
		$id = $name . ($uniqid ? '_' . md5(uniqid(__FUNCTION__, true)) : '_' . $value);
		$desc = $description ? we_html_tools::alertAttentionBox($description, $type, $width) : '';
		// Create HTML tags
		return '
<div class="radiobuttonContainer">
<div class="radiobutton"><input type="radio" name="' . $name . '" id="' . $id . '" value="' . $value . '" ' . ($checked ? ' checked="checked"' : '') . ($onMouseUp ? ' onmouseup="' . $onMouseUp . '"' : '') . ($onClick ? ' onclick="' . $onClick . '"' : "") . ($disabled ? ' disabled="disabled"' : '') . ' />
<label id="label_' . $id . '" for="' . $id . '" class="elementText ' . ($disabled ? 'disabled ' : '') . $class . ($title ? ' showhelp' : '') . '" ' . ($onMouseUp ? ' onmouseup="' . str_replace('this.', "document.getElementById('" . $id . "').", $onMouseUp) . '"' : '') . ($title ? ' title="' . $title . '"' : '') . '>' . $text . ($desc && $type === we_html_tools::TYPE_HELP ? $desc : '') . '</label></div>' .
			($desc && $type !== we_html_tools::TYPE_HELP ? '<div class="extra">' . $desc . '</div>' : "") .
			($extra_content ? '<div class="extra">' . $extra_content . '</div>' : "") . '
</div>';
	}

	/**
	 * returns the HTML Code for a webEdition Textarea (we:textarea we:sessionfield ...)
	 *
	 * @return string
	 * @param string $name
	 * @param string $value
	 * @param array<string,mixed> $attribs
	 * @param boolean $hidestylemenu
	 * @param boolean $forceinwebedition
	 * @param boolean $removeFirstParagraph
	 * @param $docCss array<int,string>
	 *
	 */
	static function weTextarea(string $name, string $value, array $attribs, bool $hidestylemenu = false, bool $forceinwebedition = false, bool $removeFirstParagraph = true, bool $isFrontendEdit = false, string $origName = '', array $docCss = [
		]): string{
		$attribs = array_change_key_case($attribs);
		$wysiwyg = we_weTag_tag::getAttribute('wysiwyg', $attribs, false, we_base_request::BOOL);
		$value = self::removeBrokenInternalLinksAndImages($value);
		$class = we_weTag_tag::getAttribute('class', $attribs, '', we_base_request::STRING);
		$oldHtmlspecialchars = we_weTag_tag::getAttribute('htmlspecialchars', $attribs, false, we_base_request::BOOL);
		$width = we_weTag_tag::getAttribute('width', $attribs, '', we_base_request::UNIT);
		$height = we_weTag_tag::getAttribute('height', $attribs, '', we_base_request::UNIT);
		$style = we_weTag_tag::getAttribute('style', $attribs, '', we_base_request::STRING);
		$match = [];
		if(preg_match_all('/(width|height):([^;"]+)[;"]?/i', $style, $match)){
			$wh = array_combine($match[1], $match[2]);
			$width = $width ?: (empty($wh['width']) ? 0 : we_base_util::convertUnits($wh['width']));
			$height = $height ?: (empty($wh['height']) ? 0 : we_base_util::convertUnits($wh['height']));
		}

		if($wysiwyg){
			$commands = preg_replace('/ *, */', ',', we_weTag_tag::getAttribute('commands', $attribs, defined('COMMANDS_DEFAULT') ? COMMANDS_DEFAULT : '', we_base_request::STRING));
			$menu = preg_replace('/ *, */', ',', we_weTag_tag::getAttribute('menu', $attribs, '', we_base_request::STRING));
			$contextmenu = preg_replace('/ *, */', ',', we_weTag_tag::getAttribute('contextmenu', $attribs, '', we_base_request::STRING));
			$bgcolor = we_weTag_tag::getAttribute('bgcolor', $attribs, '', we_base_request::STRING);
			$cssClasses = we_weTag_tag::getAttribute('classes', $attribs, '', we_base_request::STRING);
			$editorcss = we_weTag_tag::getAttribute('editorcss', $attribs, [], we_base_request::INTLISTA);
			$imagestartid = we_weTag_tag::getAttribute('imagestartid', $attribs, 0, we_base_request::INT);
			$galleryTemplates = we_weTag_tag::getAttribute('gallerytemplates', $attribs, [], we_base_request::INTLISTA);
			$tinyParams = we_weTag_tag::getAttribute('tinyparams', $attribs, '', we_base_request::RAW);
			$templates = we_weTag_tag::getAttribute('templates', $attribs, '', we_base_request::STRING);
			$formats = we_weTag_tag::getAttribute('formats', $attribs, '', we_base_request::STRING);
			$fontsizes = we_weTag_tag::getAttribute('fontsizes', $attribs, '', we_base_request::STRING);
			$ignoredocumentcss = we_weTag_tag::getAttribute('ignoredocumentcss', $attribs, false, we_base_request::BOOL);
			$size = we_weTag_tag::getAttribute('size', $attribs, '', we_base_request::STRING);

			$commands = ($hidestylemenu ? str_replace('applystyle,', '', $commands ?: implode(',', we_wysiwyg_editor::getAllCmds())) : $commands);

			$contentCss = '';
			if(($contentCssIDs = array_unique(array_filter(array_merge((!$ignoredocumentcss ? array_keys($docCss) : []), $editorcss))))){
				$contentCss = implode(',', DB_WE::$inst->getAllq('SELECT CONCAT(Path,' . (DISABLE_MEDIA_MODTIME ? '""' : '"?m=",UNIX_TIMESTAMP(Publish_Date)') . ') FROM ' . FILE_TABLE . ' WHERE IsPublished=1 AND ID IN (' . implode(',', $contentCssIDs) . ')', true));
			}

			$inwebedition = ($forceinwebedition ?: isWE());
			$inlineedit = (bool) we_weTag_tag::getAttribute('inlineedit', $attribs, ($inwebedition ? INLINEEDIT_DEFAULT : true), we_base_request::BOOL);
			$frontendHeader = $isFrontendEdit ? we_wysiwyg_editor::getHTMLHeader($isFrontendEdit, false, $inlineedit) : '';
			$lang = (we_base_WE::$we_doc && isset(we_base_WE::$we_doc->Language)) ? we_base_WE::$we_doc->Language : WE_LANGUAGE;

			if($inlineedit){
				$e = new we_wysiwyg_editor([], we_wysiwyg_editor::TYPE_INLINE_TRUE, $name, $width, $height, $value, $commands, $bgcolor, $class, '', $removeFirstParagraph, $inlineedit, '', $cssClasses, $lang, false, $isFrontendEdit, '', $oldHtmlspecialchars, $contentCss, $origName, $tinyParams, $contextmenu, $templates, $formats, $imagestartid, $galleryTemplates, $fontsizes, $menu, '');

				return $frontendHeader . $e->getHTML();
			}

			$e = new we_wysiwyg_editor([], we_wysiwyg_editor::TYPE_EDITBUTTON, $name, $width, $height, '', $commands, $bgcolor, $class, '', $removeFirstParagraph, $inlineedit, '', $cssClasses, $lang, false, $isFrontendEdit, '', $oldHtmlspecialchars, $contentCss, $origName, $tinyParams, $contextmenu, $templates, $formats, $imagestartid, $galleryTemplates, $fontsizes, $menu, $size);

			if(stripos($name, 'we_ui') === false){//we are in backend
				$hiddenTextareaContent = strtr(we_wysiwyg_editor::parseInternalImageSrc($value), ['##|r##' => "\r",
					'##|n##' => "\n", '&' => '&amp;']);
				$previewDivContent = strtr((
					we_base_WE::$we_doc && !(we_base_WE::$we_doc instanceof we_contents_objectFile) && !(we_base_WE::$we_doc instanceof we_contents_object) ?
					we_wysiwyg_editor::parseInternalImageSrc((string) we_base_WE::$we_doc->getField($attribs)) :
					we_document_base::parseInternalLinks($value, 0)
					), ['##|r##' => "\r", '##|n##' => "\n"]
				);
			} else {//we are in frontend
				$previewDivContent = $hiddenTextareaContent = strtr(we_document_base::parseInternalLinks($value, 0), ['##|r##' => "\r", '##|n##' => "\n", '&' => '&amp;']);
			}

			$bgcolor = preg_match('/^[a-f0-9]{6}$/i', $bgcolor) ? '#' . $bgcolor : $bgcolor;
			$bgStyle = $bgcolor ? ' style="background-color: ' . $bgcolor . ' !important; background-image: none !important;"' : '';

			return $frontendHeader . we_html_element::textArea(['name' => $name, 'id' => $name, 'onchange' => '_EditorFrame.setEditorIsHot(true);', 'style' => 'display: none',
					'class' => 'wetextarea'], $hiddenTextareaContent) .
				'<div class="wysiwygPreview tbButtonWysiwygBorder ' . ($class ?: "") . ' wetextarea tiny-wetextarea wetextarea-' . $origName . '" ' . $bgStyle . ' id="div_wysiwyg_' . $name . '">' . $previewDivContent . '</div><div class="tbButtonWysiwygBorder" style="border-top:0;">' . $e->getHTML() . '</div>';
		}

		$style = ($style ? explode(';', trim(preg_replace('/(width|height):[^;"]+[;"]?/i', '', $style), ' ;')) : []);
		$style[] = 'width:' . $width . (is_numeric($width) ? 'px' : '');
		$style[] = 'height:' . $height . (is_numeric($height) ? 'px' : '');

		$atts = we_weTag_tag::removeAttribs($attribs, ['wysiwyg', 'commands', '_name_orig', 'pure', 'type', 'value', 'checked', 'autobr', 'name',
				'values', 'hidden', 'editable', 'format', 'property', 'size', 'maxlength', 'width', 'height',
				'fontnames', 'bgcolor', 'editorcss', 'ignoredocumentcss', 'importrtf', 'hideautobr',
				'stylesheets'
		]);

		$atts['id'] = we_weTag_tag::getAttribute('id', $attribs, we_weTag_tag::getAttribute('pass_id', $attribs, '', we_base_request::STRING), we_base_request::STRING);
		$atts['cols'] = we_weTag_tag::getAttribute('cols', $attribs, '', we_base_request::INT);
		$atts['rows'] = we_weTag_tag::getAttribute('rows', $attribs, '', we_base_request::INT);
		$atts['style'] = implode(';', $style);
		$atts['class'] = ($class ? $class . ' ' : '') . 'wetextarea wetextarea-' . $origName;

		$atts['name'] = $name;
		return we_weTag_tag::htmlTag('textarea', $atts, htmlspecialchars($value), true);
	}

	static function removeBrokenInternalLinksAndImages(string &$text): string{
		$regs = [];
		if(preg_match_all('/(href|src)="' . we_base_link::TYPE_INT_PREFIX . '(\\d+)([" \?#])/i', $text, $regs, PREG_SET_ORDER)){
			$fileIDs = DB_WE::$inst->getAllq('SELECT ID FROM ' . FILE_TABLE . ' WHERE ID IN(' . implode(',', array_column($regs, 2)) . ')', true);
			foreach($regs as $reg){
				if(in_array((int) $reg[2], $fileIDs)){
					continue;
				}
				$text = preg_replace(['|<a [^>]*href="' . we_base_link::TYPE_INT_PREFIX . $reg[2] . $reg[3] . '[^>]*>([^<]+)</a>|i',
					'|<a [^>]*href="' . we_base_link::TYPE_INT_PREFIX . $reg[2] . $reg[3] . '[^>]*>|i',
					//'|<img [^>]*src="' . we_base_link::TYPE_INT_PREFIX . $reg[2] . $reg[3] . '"[^>]*>|i',
					], ['${1}'], $text);
			}
		}
		/*
		  if(preg_match_all('/src="' . we_base_link::TYPE_THUMB_PREFIX . '(\\d+)[" ]/i', $text, $regs, PREG_SET_ORDER)){
		  foreach($regs as $reg){
		  list($imgID, $thumbID) = explode(',', $reg[1]);
		  $thumbObj = new we_thumbnail();
		  if(!$thumbObj->initByImageIDAndThumbID(intval($imgID), intval($thumbID))){
		  $text = preg_replace('|<img[^>]+src="' . we_base_link::TYPE_THUMB_PREFIX . $reg[1] . '[^>]+>|i', '', $text);
		  }
		  }
		  }
		 */

		if(we_modules_info::isActive(we_modules_info::OBJECT)){
			if(preg_match_all('/href="' . we_base_link::TYPE_OBJ_PREFIX . '(\\d+)[^" \?#]+\??/i', $text, $regs, PREG_SET_ORDER)){
				$fileIDs = DB_WE::$inst->getAllq('SELECT ID FROM ' . OBJECT_FILES_TABLE . ' WHERE ID IN(' . implode(',', array_column($regs, 1)) . ')', true);
				foreach($regs as $reg){
					$id = (int) $reg[1];
					if(in_array($id, $fileIDs)){
						continue;
					}// if object doesn't exists, remove the link
					$text = preg_replace(['|<a [^>]*href="' . we_base_link::TYPE_OBJ_PREFIX . $id . '"[^>]*>([^<]+)</a>|i',
						'|<a [^>]*href="' . we_base_link::TYPE_OBJ_PREFIX . $id . '"[^>]*>|i',
						], ['${1}'], $text);
				}
			}
		}

		return $text;
	}

	public static function formSelectFromDB(string $docName, string $elemVal, string $name, string $table, string $id, string $value, string $text, string $sqlTail = '', int $size = 1,
																				 array $selectedIndex = [], bool $multiple = false, array $attribs = [], string $precode = '', string $postcode = '', array $firstEntry = []): string{
		$vals = [];
		if($firstEntry){
			$vals[$firstEntry[0]] = $firstEntry[1];
		}
		$vals = $vals + DB_WE::$inst->getAllFirstq('SELECT ' . $id . ',' . $value . ' FROM ' . DB_WE::$inst->escape($table) . ' WHERE ' . $sqlTail, false);

		$vals = we_html_tools::groupArray($vals, false, 1);
		$myname = $docName ? 'we_' . $docName . '_' . $name : $name;

		$attribs['onchange'] = ($multiple ? ";var we_sel='';for(i=0;i<this.options.length;i++){if(this.options[i].selected){we_sel += (this.options[i].value + ',');};};if(we_sel){we_sel=we_sel.substring(0,we_sel.length-1)};this.form.elements['" . $myname . "'].value=we_sel;" : '') . ($attribs['onchange'] ?? '');

		$pop = we_html_tools::select($myname . ($multiple ? 'Tmp' : ''), $vals, $size, $elemVal, $multiple, $attribs, true);

		if($precode || $postcode){
			$pop = ($precode ? ('<span style="padding-right:20px;">' . $precode . '</span>') : '') . $pop . $postcode;
		}
		return ($multiple ? we_html_element::hidden($myname, implode(',', $selectedIndex)) : '') . we_html_tools::formElementTable($text, $pop);
	}

}
