<?php
declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: def1d23a4935 $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2020-04-15 13:13 +0200 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */

/**
 * class    listviewBase
 * @desc    This is the base class for all webEdition listviews.
 *
 */
abstract class we_listview_base{
	const PROPPREFIX = 'we_';
	const CATEGORY_FIELD = self::PROPPREFIX . 'category';
	const CLASS_FIELD = self::PROPPREFIX . 'class_id';
	const CONTENTTYPE_FIELD = self::PROPPREFIX . 'contenttype';
	const CUSTOMER_FIELD = self::PROPPREFIX . 'webuserid';
	const CUSTOMER_PREFIX = self::PROPPREFIX . 'customer_';
	const DATECREATION_FIELD = self::PROPPREFIX . 'creationdate';
	const DATEMOD_FIELD = self::PROPPREFIX . 'moddate';
	const DATEPUBLISHED_FIELD = self::PROPPREFIX . 'published';
	const DESC_FIELD = self::PROPPREFIX . 'description';
	const DOCTYPE_FIELD = self::PROPPREFIX . 'doctype';
	const FILENAME_FIELD = self::PROPPREFIX . 'filename';
	const HREF_FIELD = self::PROPPREFIX . 'href';
	const ID_FIELD = self::PROPPREFIX . 'id';
	const KEYWORDS_FIELD = self::PROPPREFIX . 'keywords';
	const LANG_FIELD = self::PROPPREFIX . 'language';
	const LASTPATH_FIELD = self::PROPPREFIX . 'lastpath';
	const OBJECT_FIELD_PREFIX = self::PROPPREFIX . 'object_';
	const PARENTID_FIELD = self::PROPPREFIX . 'parentid';
	const PATH_FIELD = self::PROPPREFIX . 'path';
	const RANDOM_FIELD = 'random()';
	const SHOPVARIANTS_FIELD = self::PROPPREFIX . 'shopvariants';
	const TEXT_FIELD = self::PROPPREFIX . 'text';
	const TITLE_FIELD = self::PROPPREFIX . 'title';
	const TRIGGER_FIELD = self::PROPPREFIX . 'triggerid';
	const URL_FIELD = self::PROPPREFIX . 'url';
	const VARIANT_FIELD = self::PROPPREFIX . 'variant';
	const WEEKDAYS = ['sunday', 'monday', 'tuesday', 'wednesday', 'thursday', 'friday', 'saturday'];

	/** @var we_database_base */
	protected $DB_WE; /* Main DB Object */

	/** @var string name of listview */
	var $name; /* name of listview */

	/** @var int */
	public $rows = -1; /* Number of rows */ // FIXME: make getter

	/** @var int */
	protected $cols = 0; /* Number of cols */

	/** @var int */
	protected $maxItemsPerPage = 1;

	/** @var bool */
	protected $stop_next_row = false;

	/** @var int Where to start output */
	var $start = 0; /* Where to start output */

	/** @var string search words */
	protected $search = ''; /* search words */

	/** @var int start offset of first page */
	protected $offset = 0; /* start offset of first page */

	/** @var string Order string */
	protected $order = ''; /* Order string */

	/** @var bool set to true, if order should be descendend */
	protected $desc = false; /* set to true, if order should be descendend */

	/** @var string[] category string  */
	var $cats = [];

	/** @var bool set to true if it should be an 'OR condition' e.g. categories='value1' OR categories='value2' */
	var $catOr = false; /* set to true if it should be an 'OR condition' e.g. categories='value1' OR categories='value2' */

	/** @var int total number of matches */
	protected $anz_all = 0; /* total number of matches */

	/** @var int number of rows in page */
	var $anz = 0; /* number of rows in page */

	/** @var int[] id's of workspace */
	var $workspaceID = [];

	/** @var int internal counter */
	var $count = 0; /* internal counter */

	/** @var array<string,mixed> array to store results */
	protected $Record = []; /* array to store results */

	/** @var bool close </a> when endtag used */
	private $close_a = true; /* close </a> when endtag used */

	/** @var string */
	protected $customerFilterType = ''; // shall we control customer-filter?

	/** @var array{calendar:string,defaultDate:DateTime,date:?DateTime,calendarCount:int,datefield:string,start_date:?DateTime,end_date:?DateTime,storage:array<int,DateTime>,forceFetch:bool,count:int,weekstart:int,date_human?:string,day_human?:string,month_human?:int,year_human?:int} */
	public $calendar_struct = null;

	/** @var int[] */
	var $id = [];

	/** @var int[] array of ids with pages which are found */
	protected $IDs = []; /* array of ids with pages which are found */

	/** @var string */
	protected $condition = '';

	/** @var bool */
	public $hidedirindex = false; //since $lv->hidedirindex is accessed at output

	/** @var bool */
	protected $searchable = true;

	/** @var int[] */
	protected $customers = [];

	/** @var int */
	protected static $lvNr = 0;

	/** @var int[] */
	protected $categoryids = [];

	/** @var string */
	protected $LastDocPath = '';

	/**
	 * listviewBase()
	 * @desc    constructor of class
	 *
	 * @param  string $name          - name of listview
	 * @param   int $rows          - number of rows to display per page
	 * @param   int $offset        - start offset of first page
	 * @param   string $order         - field name(s) to order by
	 * @param   bool $desc          - set to true, if order should be descendend
	 * @param   string[] $cats          - comma separated categories
	 * @param   bool $catOr         - set to true if it should be an "OR condition"
	 * @param   array $workspaceID   - commaseperated string of id's of workspace
	 * @param   int $cols   		   - to display a table this is the number of cols
	 * @param int[] $categoryids
	 *
	 */
	function __construct(string $name = '0', int $rows = 999999999, int $offset = 0, string $order = '', bool $desc = false, array $cats = [], bool $catOr = false, array $workspaceID = [
		], int $cols = 0, string $calendar = '', string $datefield = '', string $date = '', string $weekstart = '', array $categoryids = [], string $customerFilterType = 'all', array $id = [
		]){
		$this->rows = $rows;
		$this->order = $order;
		$this->desc = $desc;
		$this->cats = $cats;
		$this->categoryids = $categoryids;
		$this->catOr = $catOr;
		$this->workspaceID = $workspaceID ?: [];
		$this->customerFilterType = $customerFilterType;
		$this->id = $id;
		$this->offset = max(0, $offset);

		self::$lvNr++;
		$this->name = $name ?: (self::$lvNr);
		//? strange setting - comes from we_tag_search
		$this->search = trim(str_replace(['"', '\\"'], '', (!($val = we_base_request::string('we_lv_search_' . $this->name, '')) && we_base_request::bool('we_from_search_' . $this->name)) ? '' : $val));
		$this->DB_WE = new DB_WE();
		$this->maxItemsPerPage = $cols ? ($rows * $cols) : $rows;
		$this->cols = (($cols == 0 && ($calendar === 'month' || $calendar === 'month_table')) ? 7 : $cols);
		$this->start = max(we_base_request::int('we_lv_start_' . $this->name, $this->offset), $this->offset);

		if($calendar){
			$this->calendar_struct = [
				'calendar' => $calendar,
				'defaultDate' => new DateTime($date ?: 'now'),
				'date' => null,
				'calendarCount' => 0,
				'datefield' => ($datefield ?: '###Published###'),
				'start_date' => null,
				'end_date' => null,
				'storage' => [],
				'forceFetch' => false,
				'count' => 0,
				'weekstart' => ($weekstart && ($match = array_search($weekstart, self::WEEKDAYS)) !== false ? $match : 0),
			];
		}
	}

	protected function getIdQuery(string $fieldname): string{
		return ($this->id ? $fieldname . ' IN(' . implode(',', $this->id) . ') ' : '');
	}

	/**
	 * next_record()
	 * @desc    selects next row and returns true. if no more row exists, it returns false
	 *
	 *
	 */
	public function next_record(): bool{
		// overwrite
		if(!$this->calendar_struct){
			return false;
		}
		$this->calendar_struct['calendarCount']++;
		$this->calendar_struct['count'] = $this->count;
		$this->calendar_struct['forceFetch'] = false;
		$calendarCount = $this->calendar_struct['calendarCount'];
		if(($calendarCount > 0 || ($this->calendar_struct['calendar'] === 'day' && $calendarCount >= 0)) && $calendarCount <= $this->calendar_struct['numofentries']){
			if(!$this->calendar_struct['date']){
				$this->calendar_struct['date'] = $this->calendar_struct['defaultDate'];
			}
			$date = $this->calendar_struct['date'];
			assert($date instanceof DateTime);
			$month = (int) $date->format('m');
			$year = (int) $date->format('Y');
			$day = (int) $date->format('j');
			switch($this->calendar_struct['calendar']){
				case 'year':
					$date->setDate($year, $calendarCount, $day)->setTime(0, 0, 0);
					$start_date = (new DateTime())->setDate($year, $calendarCount, $day)->setTime(0, 0, 0);
					$end_date = (new DateTime())->setDate($year, $calendarCount, self::getNumberOfDays($calendarCount, $year))->setTime(23, 59, 59);
					break;
				case 'day':
					$date->setDate($year, $month, $day)->setTime($calendarCount, 0);
					$start_date = (new DateTime())->setDate($year, $month, $day)->setTime($calendarCount, 0);
					$end_date = (new DateTime())->setDate($year, $month, $day)->setTime($calendarCount, 59, 59);
					break;
				default:
					$date->setDate($year, $month, $calendarCount)->setTime(0, 0, 0);
					$start_date = (new DateTime())->setDate($year, $month, $calendarCount)->setTime(0, 0, 0);
					$end_date = (new DateTime())->setDate($year, $month, $calendarCount)->setTime(23, 59, 59);
			}

			$this->calendar_struct['date'] = $date;
			$this->calendar_struct['date_human'] = $date->format('Y-m-d');
			$this->calendar_struct['day_human'] = $day;
			$this->calendar_struct['month_human'] = $month;
			$this->calendar_struct['year_human'] = $year;
			$this->calendar_struct['start_date'] = $start_date;
			$this->calendar_struct['end_date'] = $end_date;

			foreach($this->calendar_struct['storage'] as $k => $v){
				assert($v instanceof DateTime);
				if($v >= $start_date && $v <= $end_date){
					$found = array_search($k, $this->IDs, false);
					if($found !== false){
						$this->calendar_struct['forceFetch'] = true;
						$this->calendar_struct['count'] = $found;
					}
				}
			}
		}
		if($calendarCount > $this->calendar_struct['numofentries']){
			$this->calendar_struct['date'] = null;
		}
		if(!$this->calendar_struct['forceFetch']){
			$this->count++;
		}
		$this->Record = [];
		return true;
	}

	/**
	 * f()
	 * @desc    returns content of a field
	 *
	 * @param   string $key  name of field to return
	 *
	 */
	function f(string $key, bool $stripWE = true): string{
		$key = strtolower(preg_replace('/^(of|wedoc|we)_/i', ($stripWE ? '' : self::PROPPREFIX), $key));
		return strval($this->Record[$key] ?? '');
	}

	/**
	 * hasNextPage()
	 * @desc    returns true if next page exists, otherwise it returns false
	 *
	 *
	 */
	public function hasNextPage(bool $parentEnd = false): bool{
		if(!empty($this->calendar_struct['calendar'])){
			return true;
		}
		if($parentEnd && ($end = we_base_request::int('we_lv_pend_' . $this->name))){
			return (($this->start + $this->anz) < $end);
		}
		return (($this->start + $this->anz) < $this->anz_all);
	}

	/**
	 * hasPrevPage()
	 * @desc    returns true if previous page exists, otherwise it returns false
	 *
	 *
	 */
	public function hasPrevPage(bool $parentStart = false): bool{
		if(!empty($this->calendar_struct['calendar'])){
			return true;
		}
		if($parentStart && ($start = we_base_request::int('we_lv_pstart_' . $this->name, 0))){
			return (abs($this->start) != $start);
		}
		return (abs($this->start) > abs($this->offset));
	}

	/**
	 * getBackLink()
	 * @desc    gets the HTML Code for the back link
	 *
	 * @param   array<string,string> $attribs  array with HTML attributes
	 *
	 */
	public function getBackLink(array $attribs, string $content = ''): string{
		$only = we_weTag_tag::getAttribute('only', $attribs, '', we_base_request::STRING);
		$urlID = we_weTag_tag::getAttribute('id', $attribs, 0, we_base_request::INT);

		if(!$this->hasPrevPage()){
			return $content;
		}
		if(!empty($this->calendar_struct['calendar'])){

			$month = $this->calendar_struct['month_human'];
			$day = $this->calendar_struct['day_human'];
			$year = $this->calendar_struct['year_human'];
			switch($this->calendar_struct['calendar']){
				case 'month':
				case 'month_table':
					$day = 1;
					$month--;
					if($month < 1){
						$month = 12;
						$year--;
					}
					break;
				case 'year':
					$year--;
					break;
				case 'day':
					$day--;
					if($day < 1){
						$month--;
						if($month <= 1){
							$month = 12;
							$year--;
						}
						$day = self::getNumberOfDays($month, $year);
					}
					break;
			}
			$newdate = $year . '-' . $month . '-' . $day;

			$href = (string) we_weTag_util::tag('url', ['id' => ($urlID ?: 'top'), 'hidedirindex' => $this->hidedirindex]);
			$attribs['href'] = $href . (strpos($href, '?') === false ? '?' : '&');
			$attribs['rel'] = 'prev';

			$tmp_href = we_http_util::makeQueryString(['we_lv_calendar_' . $this->name => $this->calendar_struct['calendar'], 'we_lv_datefield_' . $this->name => $this->calendar_struct['datefield'],
					'we_lv_date_' . $this->name => $newdate]);
		} else {
			$foo = max($this->offset, $this->start - $this->maxItemsPerPage);
			$tmp_href = we_http_util::makeQueryString(['we_lv_start_' . $this->name => $foo]);
		}

		$href = (string) we_weTag_util::tag('url', ['id' => ($urlID ?: 'top'), 'hidedirindex' => $this->hidedirindex]);
		$attribs['href'] = $href . (strpos($href, '?') === false ? '?' : '&') . $tmp_href;
		if($only){
			$this->close_a = false;
			return ($attribs[$only] ?? '');
		}
		return we_weTag_tag::htmlTag('a', $attribs, $content, true);
	}

	/**
	 * getNextLink()
	 * @desc    gets the HTML Code for the next link
	 *
	 * @param   array<string,string> $attribs  array with HTML attributes
	 *
	 */
	public function getNextLink(array $attribs, string $content = ''): string{
		$only = we_weTag_tag::getAttribute('only', $attribs, '', we_base_request::STRING);
		$urlID = we_weTag_tag::getAttribute('id', $attribs, 0, we_base_request::INT);
		if(!empty($this->calendar_struct['calendar'])){

			$month = $this->calendar_struct['month_human'];
			$day = $this->calendar_struct['day_human'];
			$year = $this->calendar_struct['year_human'];
			switch($this->calendar_struct['calendar']){
				case 'month':
				case 'month_table':
					$day = 1;
					$month++;
					if($month > 12){
						$month = 1;
						$year++;
					}
					break;
				case 'year':
					$year++;
					break;
				case 'day':
					$day++;
					$numd = self::getNumberOfDays($month, $year);
					if($day > $numd){
						$day = 1;
						$month++;
					}
					if($month >= 12){
						$month = 1;
						$year++;
					}
					break;
			}
			$newdate = $year . '-' . $month . '-' . $day;
			$tmp_href = we_http_util::makeQueryString(['we_lv_calendar_' . $this->name => $this->calendar_struct['calendar'], 'we_lv_datefield_' . $this->name => $this->calendar_struct['datefield'],
					'we_lv_date_' . $this->name => $newdate]);
		} else if(!$this->hasNextPage()){
			return $content;
		} else {
			$foo = $this->start + $this->maxItemsPerPage;
			$tmp_href = we_http_util::makeQueryString(['we_lv_start_' . $this->name => $foo]);
		}

		$href = (string) we_weTag_util::tag('url', ['id' => ($urlID ?: 'top'), 'hidedirindex' => $this->hidedirindex]);
		$attribs['href'] = $href . (strpos($href, '?') === false ? '?' : '&') . $tmp_href;
		$attribs['rel'] = 'next';
		if($only){
			$this->close_a = false;
			return ($attribs[$only] ?? '');
		}
		return we_weTag_tag::htmlTag('a', $attribs, $content, true);
	}

	public function shouldPrintEndTR(): bool{
		return ($this->cols && ($this->count % $this->cols) === 0);
	}

	public function shouldPrintStartTR(): bool{
		return ($this->cols && (($this->count - 1) % $this->cols) === 0);
	}

	public function tdEmpty(): bool{
		return ($this->count > $this->anz);
	}

	public function getCalendarField(string $type, string $langcode): string{
		$showDate = ($this->calendar_struct['date'] ?: $this->calendar_struct['defaultDate']);
		assert($showDate instanceof DateTime);
		switch($type){
			case 'day':
				return ($this->calendar_struct['calendar'] === 'day' ?
					$this->calendar_struct['defaultDate']->format('j') :
					($this->calendar_struct['date'] ? $this->calendar_struct['date']->format('j') : '')
					);
			case 'dayname':
			case 'dayname_long':
				$days = we_base_country::getTranslationList(we_base_country::DAY, $langcode);
				$day = strtolower($showDate->format('D'));
				return $days['wide'][$day];
			case 'dayname_short':
				$days = we_base_country::getTranslationList(we_base_country::DAY, $langcode);
				$day = strtolower($showDate->format('D'));
				return $days['abbreviated'][$day];
			case 'month':
				return ($this->calendar_struct['date'] ?: $this->calendar_struct['defaultDate'])->format('m');
			case 'monthname':
			case 'monthname_long':
				$months = we_base_country::getTranslationList(we_base_country::MONTH, $langcode);
				$mon = $showDate->format('n');
				return $months['wide'][$mon];
			case 'monthname_short':
				$months = we_base_country::getTranslationList(we_base_country::MONTH, $langcode);
				$mon = $showDate->format('n');
				return $months['abbreviated'][$mon];
			case 'year':
				return $showDate->format('Y');
			case 'hour':
				return $showDate->format('H:i');
			case 'week':
				return $showDate->format('W');
			default:
				return ($this->calendar_struct['date'] ? $this->calendar_struct['date']->format('j') : '');
		}
	}

	public function getCalendarFieldValue(string $name, string $langcode): string{
		$showDate = ($this->calendar_struct['date'] ?: $this->calendar_struct['defaultDate']);
		assert($showDate instanceof DateTime);
		switch($name){
			case 'day':
				return $showDate->format('j');
			case 'month':
				return $showDate->format('m');
			case 'year':
				return $showDate->format('Y');
			case 'dayname':
			case 'dayname_long':
				$days = we_base_country::getTranslationList(we_base_country::DAY, $langcode);
				$day = strtolower($showDate->format('D'));
				return $days['wide'][$day];
			case 'dayname_short':
				$days = we_base_country::getTranslationList(we_base_country::DAY, $langcode);
				$day = strtolower($showDate->format('D'));
				return $days['abbreviated'][$day];
			case 'monthname':
			case 'monthname_long':
				$months = we_base_country::getTranslationList(we_base_country::MONTH, $langcode);
				$mon = $showDate->format('n');
				return $months['wide'][$mon];
			case 'monthname_short':
				$months = we_base_country::getTranslationList(we_base_country::MONTH, $langcode);
				$mon = $showDate->format('n');
				return $months['abbreviated'][$mon];
			case 'hour':
				return $showDate->format('H:i');
			case 'start_date':
				return $this->calendar_struct['start_date']->format('Y-m-d');
			case 'end_date':
				return $this->calendar_struct['end_date']->format('Y-m-d');
			case 'timestamp':
				return $showDate->format('U');
			default:
				return $showDate->format('j');
		}
	}

	public function isCalendarField(string $type): bool{
		return in_array($type, ['day', 'dayname', 'dayname_long', 'dayname_short', 'month', 'monthname', 'monthname_long', 'monthname_short', 'year', 'hour'], true);
	}

	protected function fetchCalendar(&$condition, &$calendar_select, &$calendar_where, array $matrix = []): void{
		if(!$this->calendar_struct['calendar']){
			return;
		}
		$calendar = $this->calendar_struct['calendar'];
		$date = $this->calendar_struct['defaultDate'];
		assert($date instanceof DateTime);
		$day = (int) $date->format('j');
		$month = (int) $date->format('m');
		$year = (int) $date->format('Y');

		switch($calendar){
			case 'year':
				$start_date = (new DateTime())->setDate($year, 1, $day)->setTime(0, 0, 0);
				$end_date = (new DateTime())->setDate($year, 12, $day)->setTime(23, 59, 59);
				$numofentries = 12;
				break;
			case 'day':
				$start_date = (new DateTime())->setDate($year, $month, $day)->setTime(0, 0, 0);
				$end_date = (new DateTime())->setDate($year, $month, $day)->setTime(23, 59, 59);
				$numofentries = 24;
				break;
			default:
				$numofentries = self::getNumberOfDays($month, $year);
				$start_date = (new DateTime())->setDate($year, $month, 1)->setTime(0, 0, 0);
				$end_date = (new DateTime())->setDate($year, $month, $numofentries)->setTime(23, 59, 59);
		}

		$this->calendar_struct['date_human'] = $date->format('Y-m-d');
		$this->calendar_struct['day_human'] = $day;
		$this->calendar_struct['month_human'] = $month;
		$this->calendar_struct['year_human'] = $year;
		$this->calendar_struct['numofentries'] = $numofentries;
		$this->calendar_struct['start_date'] = $start_date;
		$this->calendar_struct['end_date'] = $end_date;


		if(!$this->calendar_struct['datefield'] || $this->calendar_struct['datefield'] === '###Published###'){
			$this->calendar_struct['datefield'] = '###Published###';
			$calendar_select = ',f.Publish_Date AS Calendar ';
			$calendar_where = '(f.Publish_Date BETWEEN ' . $start_date->format(we_database_base::DATE_TIME_FORMAT) . ' AND ' . $end_date->format(we_database_base::DATE_TIME_FORMAT) . ') ';
		} else {
			$field = ($matrix && in_array($this->calendar_struct['datefield'], array_keys($matrix))) ?
				$matrix[$this->calendar_struct['datefield']]['alias'] . '.' . $matrix[$this->calendar_struct['datefield']]['type'] . '_' . $this->calendar_struct['datefield'] :
				CONTENT_TABLE . '.Dat';

			$calendar_select = ',' . $field . ' AS Calendar ';
			$condition = ($condition ? $condition . ' AND ' : '') . $this->calendar_struct['datefield'] . '>=' . $start_date->format(we_database_base::DATE_TIME_FORMAT) . ' AND ' . $this->calendar_struct['datefield'] . '<=' . $end_date->format(we_database_base::DATE_TIME_FORMAT);
		}
	}

	function postFetchCalendar(): void{
		if(!$this->calendar_struct['calendar'] || !$this->calendar_struct['defaultDate']){
			return;
		}
		$start = 0;
		$this->anz = $this->calendar_struct['numofentries'] + $start;
		$this->anz_all = $this->anz;

		switch($this->calendar_struct['calendar']){
			case 'day':
				$this->calendar_struct['calendarCount'] = -1;
				break;
			case 'month':
			case 'month_table':
				$start = (int) date('w', strtotime($this->calendar_struct['defaultDate']->format('Y') . '-' . $this->calendar_struct['defaultDate']->format('m') . '-1') ?: 0);
				if($this->calendar_struct['weekstart'] != ''){
					$start = $start - $this->calendar_struct['weekstart'];
					if($start < 0){
						$start = 7 + $start;
					}
				}
				$this->anz += $start;
				$this->anz_all = $this->anz;
				$this->calendar_struct['calendarCount'] = 0 - $start;
				$rest = $this->cols - ($this->anz % $this->cols);
				if($rest < $this->cols){
					$this->anz += $rest;
					$this->anz_all += $rest;
				}
				break;
			default:
				$this->calendar_struct['calendarCount'] = 0;
		}

		$this->calendar_struct['date'] = -1;
	}

	/**
	 * Returns number od days for given month
	 *
	 * @param          int                                  $month
	 * @param          int                                  $year
	 *
	 *
	 * @return         int
	 */
	static function getNumberOfDays(int $month, int $year): int{
		switch($month){
			case 1:
			case 3:
			case 5:
			case 7:
			case 8:
			case 10:
			case 12:
				return 31;
			case 2:
				return ($year % 4) == 0 ? 29 : 28;
			default:
				return 30;
		}
	}

	/**
	 * @return boolean
	 * @desc returns, if tag we:next / we:back should set an endtag automatically. As default
	 *       it should set one
	 */
	function close_a(): bool{
		$close = $this->close_a;
		$this->close_a = true;
		return $close;
	}

	public function getCustomerRestrictionQuery(int $classID): array{
		return ['', ''];
	}

	public function getRecord(): array{
		return $this->Record;
	}

	public function getFoundDocument(): we_document_base{
		static $doc = null;
		static $id = 0;
		if($id == ($docID = (int) $this->f(self::ID_FIELD))){
			assert($doc instanceof we_document_base);
			return $doc;
		}
		$id = $docID;
		$model = new we_document_webEdition($docID);
		$doc = $model;
		return $doc;
	}

	public function getListviewRequest(string $lvname): array{
		return array_merge(array_filter([
			'we_lv_order_' . $lvname => $this->order,
			'we_lv_desc_' . $lvname => $this->desc,
			'we_lv_cats_' . $lvname => implode(',', $this->cats),
			'we_lv_catOr_' . $lvname => $this->catOr,
			'we_lv_ws_' . $lvname => implode(',', $this->workspaceID),
			'we_lv_condition_' . $lvname => $this->condition,
			]), (!$this->searchable ? ['we_lv_se_' . $lvname => 0] : [])
		);
	}

	/**
	 *
	 * @return array{id:int,type:string,req:array}
	 */
	public function getShopArticleID(): array{
		return ['id' => 0, 'type' => '', 'req' => []];
	}

	public function getTableID(): int{
		return 0;
	}

	public function getHrefArray(string $name): array{
		return [];
	}

	public static function preFixArrayKeys(array $array): array{
		return array_combine(
			array_map(function($k){
				return self::PROPPREFIX . $k;
			}, array_keys($array)), $array);
	}

	public function getDID(): int{
		return $this->IDs[$this->count - 1];
	}

	public function getCalendarStruct(): ?array{
		return $this->calendar_struct;
	}

	/** get the count of pages found
	 * @return int page count
	 */
	public function getMaxPages(): int{
		return $this->rows ? (int) ceil(((float) $this->anz_all - abs($this->offset)) / ((float) $this->maxItemsPerPage )) : 1;
	}

	/** get current page number
	 * @return int page number
	 */
	public function getPageNo(): int{
		return $this->rows ? (int) (((abs($this->start) - abs($this->offset)) / $this->maxItemsPerPage) + 1) : 1;
	}

	public function getStart(): int{
		return $this->start + 1 - abs($this->offset);
	}

	public function getEnd(): int{
		return $this->rows ? min(($this->start - abs($this->offset)) + ($this->maxItemsPerPage), ($this->anz_all - abs($this->offset))) : ($this->anz_all - abs($this->offset));
	}

	/** get number of rows in the listview, a correction of a specified offset is applied
	 * @return int rows
	 */
	public function getRows(): int{
		return $this->anz_all - abs($this->offset);
	}

	/** get the set offset
	 * @return int the offset passed to lv
	 */
	public function getOffset(): int{
		return $this->offset;
	}

	/** get the set number of items  per page
	 * @return int item number
	 */
	public function getMaxItemsPerPage(): int{
		return $this->maxItemsPerPage;
	}

	public function useSeoUrls(): bool{
		return false;
	}

}
