<?php
declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: c9bfa5530311 $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2020-03-25 01:40 +0100 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */

/**
 * class    we_listview_onlinemonitor
 * @desc    class for tag
 *
 */
class we_listview_customerOnlineMonitor extends we_listview_base{
	/** @var string */
	private $Path = '';

	/** @var int */
	private $docID = 0;

	/** @var int */
	private $lastaccesslimit = 0;

	/** @var int */
	private $lastloginlimit = 0;

	/**
	 * @desc    constructor of class
	 *
	 * @param   $name          string - name of listview
	 * @param   $rows          integer - number of rows to display per page
	 * @param   $order         string - field name(s) to order by
	 * @param   $desc		   string - if desc order
	 * @param   $condition	   string - condition of listview
	 * @param   $cols		   string - number of cols (default = 1)
	 * @param   $docID	   	   string - id of a document where a we:customer tag is on
	 *
	 */
	function __construct(string $name, int $rows, int $offset, string $order, bool $desc, string $condition, int $cols, int $docID, int $lastaccesslimit, int $lastloginlimit, bool $hidedirindex){
		parent::__construct($name, $rows, $offset, $order, $desc, [], false, [], $cols);

		$this->docID = $docID;
		$this->condition = $condition;
		$this->lastaccesslimit = $lastaccesslimit;
		$this->lastloginlimit = $lastloginlimit;

		$this->Path = ($this->docID ?
			id2path($this->docID, FILE_TABLE) :
			we_base_WE::$we_doc->Path);

		$this->hidedirindex = $hidedirindex;
		// IMPORTANT for seeMode !!!! #5317
		$this->LastDocPath = '';
		if(isset($_SESSION['weS']['last_webEdition_document'])){
			$this->LastDocPath = $_SESSION['weS']['last_webEdition_document']['Path'];
		}

		if($this->desc && $this->order != '' && (!preg_match('|.+ desc$|i', $this->order))){
			$this->order .= ' DESC';
		}

		$laStr = $llStr = '';
		if($this->lastloginlimit > 0){
			$llStr = 'LastLogin_Date>(NOW() - INTERVAL ' . $this->lastloginlimit . ' SECOND) ';
		}
		if($this->lastaccesslimit > 0){
			$laStr = 'LastAccess_Date>(NOW() - INTERVAL ' . $this->lastaccesslimit . ' SECOND) ';
		}
		if($this->lastloginlimit > 0){
			$this->condition = ($this->condition ? $this->condition . ' AND ' : '') . $llStr;
		}

		if($this->lastaccesslimit > 0){
			$this->condition = ($this->condition ? $this->condition . ' AND ' : '') . $laStr;
		}
		$where = $this->condition ? (' WHERE ' . $this->condition) : '';

//FIXME use window function COUNT(*) OVER() if mariadb 10.2/mysql 8
		$this->anz_all = fI('SELECT COUNT(1) FROM ' . CUSTOMER_SESSION_TABLE . $where);

		$this->DB_WE->query('SELECT
			SessionID,
			SessionIp,
			WebUserID,
			WebUserGroup,
			Browser,
			Referrer,
			LastLogin_Date,
			UNIX_TIMESTAMP(LastLogin_Date) AS LastLogin,
			LastAccess_Date,
			UNIX_TIMESTAMP(LastAccess_Date) AS LastAccess,
			PageID,
			SessionAutologin
FROM ' .
			CUSTOMER_SESSION_TABLE .
			$where . ' ' .
			($this->order ? ' ORDER BY ' . $this->order . ' ' : '') . ' ' .
			(($this->maxItemsPerPage > 0) ? (' LIMIT ' . $this->start . ',' . $this->maxItemsPerPage) : ''));
		$this->anz = $this->DB_WE->num_rows();
	}

	public function next_record(): bool{
		if($this->DB_WE->next_record()){
			$this->Record = array_change_key_case($this->DB_WE->getRecord());
			$this->Record['sessionip'] = inet_ntop($this->Record['sessionip']);
			$this->Record[self::CUSTOMER_FIELD] = $this->Record['WebUserID'];
			$this->Record[self::PATH_FIELD] = $this->Path . '?we_omid=' . $this->Record['SessionID'];
			$this->Record[self::TEXT_FIELD] = $this->Record['SessionID'];
			$this->Record[self::ID_FIELD] = $this->Record['SessionID'];
			$this->Record[self::LASTPATH_FIELD] = $this->LastDocPath . '?we_omid=' . $this->Record['SessionID'];
			$this->count++;
			return true;
		}
		$this->stop_next_row = $this->shouldPrintEndTR();
		if($this->cols && ($this->count <= $this->maxItemsPerPage) && !$this->stop_next_row){
			$this->Record = [];
			$this->count++;
			return true;
		}

		return false;
	}

	function f(string $key, bool $stripWE = true): string{
		return parent::f($key, $stripWE);
	}

}
