<?php
declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: 7fc40f04f89e $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2020-06-24 13:49 +0200 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */

/**
 * class    we_listview_object
 * @desc    class for tag <we:listview type="object">
 *
 */
class we_listview_object extends we_listview_objectBase{

	/**
	 * @desc    constructor of class
	 *
	 * @param  string $name           name of listview
	 * @param  int $rows           number of rows to display per page
	 * @param  int $offset        start offset of first page
	 * @param  string $order         string - field name(s) to order by
	 * @param  bool $desc           set to true, if order should be descendend
	 * @param  string[] $cats          comma separated categories
	 * @param  bool $catOr         set to true if it should be an "OR condition"
	 * @param  string $condition      condition string (like SQL)
	 * @param int  $triggerID  ID of a document which to use for displaying the detail page
	 * @param   int $cols          ??
	 * @param bool  $searchable 	  if false then show also documents which are not marked as searchable
	 * @param	string  $calendar
	 * @param	string  $datefield
	 * @param	string $date
	 * @param	string $weekstart
	 * @param	int[]        $categoryids
	 * @param int[] $customers
	 * @param string[] $languages
	 *
	 */
	public function __construct(string $name, int $rows, int $offset, string $order, bool $desc, int $classID, array $cats, bool $catOr, string $condition, int $triggerID, int $cols, bool $searchable, string $calendar, string $datefield, string $date, string $weekstart, array $categoryids, array $workspaceID, string $customerFilterType, int $docID, array $customers, array $id, array $languages, bool $hidedirindex, bool $objectseourls, bool $fromWeObject = false){
		parent::__construct($name, $rows, $offset, $order, $desc, $cats, $catOr, $workspaceID, $cols, $calendar, $datefield, $date, $weekstart, $categoryids, $customerFilterType, $id, $triggerID, $searchable, $docID, $languages, $objectseourls, $hidedirindex);

		$this->classID = $classID;
		$this->customers = $customers;
		$this->condition = $condition;
		$this->customerArray = [];

		if($this->order && $this->desc && (!preg_match('|.+ desc$|i', $this->order))){
			$this->order .= ' DESC';
		}

		// IMPORTANT for seeMode !!!! #5317
		$this->LastDocPath = (isset($_SESSION['weS']['last_webEdition_document']) ? $_SESSION['weS']['last_webEdition_document']['Path'] : '');

		$this->fillMatrix($this->classID);

		$calendar_select = $calendar_where = '';

		if($calendar){
			$this->fetchCalendar($this->condition, $calendar_select, $calendar_where, $this->matrix);
		}
		$sqlParts = $this->makeSQLParts();

		//allways join the file table itself
		$sqlParts['tables'] .= ' JOIN ' . OBJECT_FILES_TABLE . ' `of` ON (of.ID=ob' . $this->classID . '.OF_ID AND ob' . $this->classID . '.OF_DocumentTable="tblObjectFiles")' . implode('', $this->joins);

		if($this->customers/* && $this->customers !== '*' */){
			$cust = implode(',', $this->customers);
			$wsql = ' of.WebUserID IN(' . $cust . ') ';
			$this->customerArray = $this->DB_WE->getAllFirstq('SELECT * FROM ' . CUSTOMER_TABLE . ' WHERE ID IN (' . $cust . ')', true, MYSQLI_ASSOC);
			$webUserID_tail = '(' . $wsql . ')';
		} else {
			$webUserID_tail = '';
		}

		$condParts = [
			'search' => ($this->searchable ? 'of.IsSearchable=1' : ''),
			'pid' => ($fromWeObject ? '' : we_contents_objectFile::makePIDTail(we_base_WE::$we_doc->ParentID, $this->classID)),
			'lang' => ($this->languages ? 'of.Language IN ("' . implode('","', array_map('escape_sql_query', $this->languages)) . '")' : ''),
			'cat' => ($this->cats || $this->categoryids ? we_category_category::getCatSQLTail($this->cats, 'of', $this->catOr, 'Category', $this->categoryids) : ''),
			'publ' => $sqlParts['publ_cond'],
			'cond' => ($sqlParts['cond'] ? '(' . $sqlParts['cond'] . ')' : ''),
			'cal' => $calendar_where,
			'webUser' => $webUserID_tail,
		];

		if(!$sqlParts['tables']){
			$this->anz_all = 0;
			$this->anz = 0;
			return;
		}
		$condParts['id'] = $this->getIdQuery('ob' . $this->classID . '.OF_ID');

		if($this->workspaceID){
			$folders = getAllFoldersRecursive($this->workspaceID, OBJECT_FILES_TABLE);
			$condParts['ws'] = empty($folders) ? '' : 'of.isFolder=0 AND of.ParentID IN(' . implode(',', $folders) . ')';
		}

		$where = implode(' AND ', array_filter($condParts)) .
			(we_modules_info::isActive(we_modules_info::CUSTOMER) ? we_customer_documentFilter::getConditionForListviewQuery($this->customerFilterType, $this, $this->classID, $id) : '') .
			$sqlParts['groupBy'];

//FIXME use window function COUNT(*) OVER() if mariadb 10.2/mysql 8
		$this->anz_all = fI('SELECT COUNT(DISTINCT of.ID) FROM ' . $sqlParts['tables'] . ' WHERE ' . $where);
		if($calendar){
			foreach($this->DB_WE->getAllq('SELECT of.ID FROM ' . $sqlParts['tables'] . ' WHERE ' . $where, true) as $wedocID){
				$this->calendar_struct['storage'][(int) $wedocID] = new DateTime($this->DB_WE->f('Calendar'));
			}
		}

		$this->DB_WE->query('SELECT ' .
			$sqlParts['fields'] .
			$calendar_select . '
FROM ' .
			$sqlParts['tables'] . '
WHERE ' .
			$where . $sqlParts['order'] .
			(($this->maxItemsPerPage > 0) ? (' LIMIT ' . $this->start . ',' . $this->maxItemsPerPage) : ''));
		$this->anz = $this->DB_WE->num_rows();

		if($calendar){
			$this->postFetchCalendar();
		}
	}

}
