<?php
declare(strict_types=1);
/**
 * webEdition SDK
 *
 * This source is part of the webEdition SDK. The webEdition SDK is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU Lesser General Public License can be found at
 * http://www.gnu.org/licenses/lgpl-3.0.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionSDK/License.txt
 *
 *

 * @license    http://www.gnu.org/licenses/lgpl-3.0.html  LGPL
 */

/**
 * PHP email transport class
 *
 */
//FIXME: this class is used in WE & uses Zend!!!
class we_mail_mail extends we_mail_znd{
	private const EMBED_IMAGES = ['gif', 'jfif', 'jpg', 'jpeg', 'jpe', 'bmp', 'png', 'tif', 'tiff', 'swf'];
	public const TYPE_SMTP = 'smtp';
	public const TYPE_PHP = 'php';

	/**
	 * Type of Message, either text/html or text/plain
	 *
	 * @var String
	 */
	protected $ContentType = we_mail_mime::TYPE_HTML;

	/**
	 * Flag for embed images
	 *
	 * @var Bool
	 */
	protected $isEmbedImages = false;

	/**
	 *
	 * @var String
	 */
	protected $basedir = '';

	/**
	 * Flag for using <base href
	 *
	 * @var Bool
	 */
	protected $isUseBaseHref = true;

	/**
	 * MessageBody (html)
	 *
	 * @var String
	 */
	protected $Body = '';

	/**
	 * MessageBody (text)
	 *
	 * @var String
	 */
	protected $AltBody = '';

	/**
	 * Flag if message is finally constructed and ready to send
	 *
	 * @var Bool
	 */
	//protected $messageBuilt = false;

	/**
	 *
	 * @var we_mail_mimePart[]
	 */
	protected $inlineAtt = [];

	/**
	 * Internal storage for the subject to survive change of charset
	 *
	 * @var string
	 */
	//protected $internal_subject = '';

	/**
	 *
	 * @param we_mail_address[] $to use Array for a list of users
	 * @param string $subject
	 * @param ?we_mail_address $sender
	 * @param ?we_mail_address $reply
	 * @param bool $isEmbedImages
	 */
	public function __construct(array $to = [], string $subject = '', we_mail_address $sender = null, we_mail_address $reply = null, bool $isEmbedImages = false){
		switch(WE_MAILER){
			case self::TYPE_SMTP:
				if(SMTP_SERVER){
					$smtp_config = [];
					if(SMTP_PORT){
						$smtp_config['port'] = SMTP_PORT;
					}
					if(SMTP_AUTH){
						$smtp_config['auth'] = SMTP_AUTH;
						if(SMTP_USERNAME){
							$smtp_config['username'] = SMTP_USERNAME;
						}
						if(SMTP_PASSWORD){
							$smtp_config['password'] = SMTP_PASSWORD;
						}
					}
					if(SMTP_ENCRYPTION){
						$smtp_config['ssl'] = SMTP_ENCRYPTION;
					}
					self::setDefaultTransport(new we_mail_TransportSmtp(SMTP_SERVER, $smtp_config));
				}
				break;

			default:
			case self::TYPE_PHP:
				//this should set return-path
				we_mail_znd::setDefaultTransport(
					($sender && !empty($sender->email) ?
						new we_mail_TransportSendmail('-f' . $sender->email) :
						new we_mail_TransportSendmail())
				);
				break;
		}

		foreach($to as $_to){
			$this->addTo($_to);
		}

		$this->setReplyTo($reply);
		if($sender){
			$this->setFrom($sender);
		} else {
			$this->setFrom(new we_mail_address(WE_DEFAULT_EMAIL, WE_DEFAULT_EMAIL));
		}

		$this->setSubject($subject);
		$this->setIsEmbedImages($isEmbedImages);
		//$this->setIsUseBaseHref(true);
	}

	/**
	 * @param string[] $toCC
	 */
	public function setCC(array $toCC): void{
		foreach($toCC as $_toCC){
			$this->addCc(we_mail_address::parse($_toCC));
		}
	}

	/**
	 * @param string[] $bcc
	 * @return void
	 */
	public function setBCC(array $bcc): void{
		foreach($bcc as $toBCC){
			$this->addBcc(we_mail_address::parse($toBCC));
		}
	}

	public function addHTMLPart(string $val): void{
		$this->ContentType = we_mail_mime::TYPE_HTML;
		$this->Body = $val;
	}

	public function addTextPart(string $val): void{
		$this->AltBody = str_replace(["\r\n", "\r", "\n"], ["\n", "\n", "\r\n"], $val);
	}

	/**
	 *
	 * @param string[] $list
	 * @return void
	 */
	public function addAddressList(array $list): void{
		foreach($list as $to){
			$this->addTo(we_mail_address::parse($to));
		}
	}

	public function buildMessage(): void{
		if($this->Body){
			if($this->isEmbedImages){
				$binParts = $images = [];
				preg_match_all('/(src|background)="(.*)"/Ui', $this->Body, $images);
				$images[2] = array_unique($images[2]); //entfernt doppelte Bildereinfügungen #3725

				foreach($images[2] as $i => $url){
					$isBinaryData = preg_match('/image\/(.*);base64,(.*)"/Ui', $url . '"', $binParts);

					$cid = '';
					if($isBinaryData){
						if(!in_array(strtolower($binParts[1]), self::EMBED_IMAGES, true)){
							continue;
						}
						$cid = 'cid:' . $this->doaddAttachmentInline($binParts[2], true, $binParts[1]);
					} elseif(preg_match('/^[A-z]+:\/\/' . $_SERVER['SERVER_NAME'] . '/', $url) || !preg_match('/^[A-z]+:\/\//', $url)){
						$filename = (explode('?', basename($url))[0]);
						$fileParts = pathinfo($filename);
						$ext = $fileParts['extension'];

						if(!in_array(strtolower($ext), self::EMBED_IMAGES, true)){
							continue;
						}
						$directory = str_replace('..', '', dirname($url) . '/');
						$directory = ($directory === '.' ? '' : $directory);
						if(($pos = stripos($directory, $_SERVER['SERVER_NAME']))){
							$directory = (string) substr($directory, (strlen($_SERVER['SERVER_NAME']) + $pos), strlen($directory));
						}
						$this->basedir = ($this->basedir ?: WE_DOCUMENT_ROOT) .
							((strlen($this->basedir) > 1 && substr($this->basedir, -1) != '/') ? '/' : '') .
							((strlen($directory) > 1 && substr($directory, -1) != '/') ? '/' : '');
						$attachmentpath = str_replace('//', '/', $this->basedir . $directory . (is_array($filename) ? $filename[0] : $filename));
						$cid = 'cid:' . $this->doaddAttachmentInline($attachmentpath);
					}
					$this->Body = preg_replace('/' . $images[1][$i] . '="' . preg_quote($url, '/') . '"/Ui', $images[1][$i] . '="' . $cid . '"', $this->Body);
				}
			}

			if($this->isUseBaseHref && //Bug #3735
				$this->ContentType === we_mail_mime::TYPE_HTML &&
				!strpos($this->Body, '<base')){
				$this->Body = str_replace('</head>', '<base href="' . we_http_util::getServerUrl() . "\" />\n</head>", $this->Body);
			}
		}

		if($this->Body){ // es gibt einen HTML-Part
			$urlReplace = we_folder_base::getUrlReplacements(DB_WE::$inst);
			$htmlPart = we_mail_mimePart::getHTMLPart($urlReplace ?
					preg_replace($urlReplace, array_keys($urlReplace), trim($this->Body)) :
					trim($this->Body), $this->charset);
			if($this->inlineAtt){// es gibt Inline-Bilder
				$htmlMulti = new we_mail_mimeMultiPartRelated($htmlPart);
				foreach($this->inlineAtt as $at){
					$htmlMulti->addPart($at);
				}
				$this->setBodyHtml($htmlMulti);
			} else {
				$this->setBodyHtml($htmlPart);
			}
		}
		$this->setBodyText(trim($this->AltBody ?: self::getTextContent($this->Body)));
	}

	public static function getTextContent(string $html): string{
		return trim(strip_tags(preg_replace([
			'-<br[^>]*>-s',
			'-<(ul|ol)[^>]*>-s',
			'-<(head|title|style|script)[^>]*>.*?</\1>-s'
					], [
			"\n",
			"\n\n",
			''
					], strtr($html, [
			"\n" => '',
			"\r" => '',
			'</tr>' => "\n",
			'</td>' => '  ',
			'</th>' => '  ',
			'</h1>' => "\n\n",
			'</h2>' => "\n\n",
			'</h3>' => "\n\n",
			'</h4>' => "\n\n",
			'</h5>' => "\n\n",
			'</h6>' => "\n\n",
			'</p>' => "\n\n",
			'</div>' => "\n",
			'</li>' => "\n",
			'&lt;' => '<',
			'&gt;' => '>',
						]
				))
		));
	}

	/**
	 * @param $attachment
	 * @param bool $isBinData
	 * @param string $ext
	 * @return bool|string
	 */
	public function doaddAttachmentInline(string $attachment, bool $isBinData = false, string $ext = ''): string{
		if(!$attachment){
			return '';
		}
		if($isBinData){
			$at = new we_mail_mimePart(base64_decode($attachment) ?: '');
			$at->id = $at->filename = str_replace('.', '', uniqid('', true));
			$at->type = self::get_mime_type($ext, '');
		} else {
			$filename = basename($attachment);
			$rep = str_replace(WE_DOCUMENT_ROOT, '', $attachment);
			$attachment = str_replace(WE_DOCUMENT_ROOT, WEBEDITION_PATH . '..', $attachment);
			$at = new we_mail_mimePart(we_base_file::load($attachment));
			$at->id = md5($filename);
			$at->filename = $filename;
			$fileParts = pathinfo($filename);
			$ext = $fileParts['extension'];
			$at->type = self::get_mime_type($ext, $filename, $attachment);
			$at->location = we_http_util::getServerUrl() . $rep;
		}
		$at->disposition = we_mail_mime::DISPOSITION_INLINE;
		$at->encoding = we_mail_mime::ENCODING_BASE64;
		$this->inlineAtt[] = $at;
		return $at->id;
	}

	/**
	 * Extends Zend Mail addAttachment to be compatible with phpMailer
	 * @access public
	 * @return boolean success
	 */
	public function doaddAttachment(string $attachmentpath): bool{
		if(!$attachmentpath){
			return false;
		}
		//$attachmentpath = str_replace(rtrim(WE_DOCUMENT_ROOT, '/'), WEBEDITION_PATH . '..', $attachmentpath);
		$filename = basename($attachmentpath);
		$ext = pathinfo($filename, PATHINFO_EXTENSION);
		$binarydata = we_base_file::load($attachmentpath);
		if(!$binarydata){
			return false;
		}
		$at = new we_mail_mimePart($binarydata);
		$at->disposition = we_mail_mime::DISPOSITION_ATTACHMENT;
		$at->encoding = we_mail_mime::ENCODING_BASE64;
		$at->id = md5($filename);
		$at->filename = $filename;
		$at->type = self::get_mime_type($ext, $filename, $attachmentpath);
		$this->addAttachment($at);
		return true;
	}

	/**
	 * Gets the mime type of attachments
	 * @access public
	 * @return string mime type of ext
	 * Replacement for mime_content_type (deprecated in PHP 5.3, and not available on some older systems
	 * Replacement for  finfo_file, available only for >= PHP 5.3
	 * Da Zend Mail keinen name="yxz" übergibt, kann man den hier einfach anhängen
	 */
	private static function get_mime_type(string $ext, string $name = '', string $filepath = ''): string{
		return (we_base_util::getMimeType($ext, $filepath, we_base_util::MIME_BY_HEAD_THEN_EXTENSION) ?: we_mail_mime::TYPE_OCTETSTREAM) . '; name="' . $name . '"';
	}

	public function setContentType(string $val = we_mail_mime::TYPE_TEXT): void{
		$this->ContentType = $val;
	}

	public function setSubject(string $val): we_mail_mimeMessage{
		$this->clearSubject();
		parent::setSubject($val);
		return $this;
	}

	public function setBaseDir(string $val): void{
		$this->basedir = $val;
	}

	public function setIsEmbedImages(bool $val = false): void{
		$this->isEmbedImages = $val;
	}

	public function setIsUseBaseHref(bool $val = true): void{
		$this->isUseBaseHref = $val;
	}

	public function setBody(string $val): void{
		$this->Body = $val;
	}

	/**
	 *
	 * @return bool
	 */
	public function send(): bool{
		try{
			parent::send();
		} catch (Exception $e){
			t_e('warning', 'Error while sending mail: ', $e);
			return false;
		}
		return true;
	}

	public function setTextPartOutOfHTML(string $html): void{
		//remove css/js code
		$html = preg_replace(['|<script.*</script>|', '|<style.*/[ ]*style>|'], '', $html);
		$this->addTextPart(trim(strip_tags(strtr($html, ['&nbsp;' => ' ', '<br/>' => "\n"]))));
	}

}
