<?php
declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: 4300e0cb4f50 $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2020-05-06 03:03 +0200 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
class we_navigation_tree extends we_tree_base{

	protected function customJSFile(): string{
		return we_html_element::jsScript(JS_DIR . 'navigation_tree.js', 'initTree();');
	}

	public static function getItems(int $ParentID, int $offset = 0, int $segment = 500, bool $sort = false, array $openFolders = []): array{
		$db = new DB_WE();

		$items = $aWsQuery = $parentpaths = [];
		if(($ws = we_users_util::get_ws(NAVIGATION_TABLE))){
			/* FIXME can we use ?!
			 * 				$folders = getAllFoldersRecursive($ws,NAVIGATION_TABLE);
			  if($folders){
			  $ws_where = 'ParentID IN(' . implode(',', $folders) . ')';
			  }
			 */
			$wsPathArray = array_map('escape_sql_query', id2pathA($ws, NAVIGATION_TABLE));
			if($wsPathArray){
				$aWsQuery[] = ' Path REGEXP "^(' . implode('|', $wsPathArray) . ')/"';
			}
			foreach($wsPathArray as $path){
				$parentpaths = array_merge($parentpaths, we_tree_base::getParents($path));
			}
			$parentpaths = array_unique($parentpaths);
			if($parentpaths){
				$aWsQuery[] = 'Path IN(' . implode(',', $parentpaths) . ')';
			}
		}

		$prevoffset = max(0, $offset - $segment);
		if($offset && $segment){
			$items[] = [
				'id' => 'prev_' . $ParentID,
				'parentid' => $ParentID,
				'text' => 'display (' . $prevoffset . '-' . $offset . ')',
				'contenttype' => self::CT_UP,
				'table' => NAVIGATION_TABLE,
				'typ' => self::TYP_DOTS,
				'open' => 0,
				'published' => 0,
				'disabled' => 0,
				'tooltip' => '',
				'offset' => $prevoffset,
			];
		}

		$where = ($aWsQuery ? '(' . implode(' OR ', $aWsQuery) . ') AND ' : '') . ' ParentID IN(' . implode(',', array_merge([$ParentID], $openFolders)) . ') ';

		foreach($db->getAllq('SELECT ID,ParentID,Path,Text,IsFolder,Ordn,Depended,abs(text) as Nr, (text REGEXP "^[0-9]") AS isNr FROM ' . NAVIGATION_TABLE . ' WHERE ' . $where . ' ORDER BY Ordn, isNr DESC,Nr,Text ' . ($segment ? 'LIMIT ' . $offset . ',' . $segment : '')) as $item){
			$typ = [
				'id' => (int) $item['ID'],
				'parentid' => $item['ParentID'],
				'typ' => ($item['IsFolder'] ? self::TYP_FOLDER : self::TYP_ITEM),
				'open' => in_array((int) $item['ID'], $openFolders) ? 1 : 0,
				'tooltip' => (int) $item['ID'],
				'offset' => $offset,
				'order' => (int) $item['Ordn'],
				'published' => $item['Depended'] ? 0 : 1,
				'disabled' => we_base_permission::hasPerm([we_base_perm::ADMINISTRATOR]) ? 0 : ($item['IsFolder'] && !$item['ParentID'] || in_array($item['Path'], $parentpaths) ? 1 : 0),
				'contenttype' => ($item['IsFolder'] ? we_base_ContentTypes::FOLDER_COMMON : we_base_ContentTypes::NAVIGATION),
			];

			$fields = array_change_key_case($db->getRecord());

			$text = strip_tags(strtr($item['Text'], ['&amp;' => '&', '<br/>' => ' ']));
			$path = str_replace('&amp;', '&', $item['Path']);
			$typ['text'] = $text;
			$typ['path'] = $path;

			$items[] = array_merge($fields, $typ);
		}

		$total = fI('SELECT COUNT(1) FROM ' . NAVIGATION_TABLE . ' WHERE ' . $where);
		$nextoffset = $offset + $segment;
		if($segment && ($total > $nextoffset)){
			$items[] = [
				'id' => 'next_' . $ParentID,
				'parentid' => $ParentID,
				'text' => 'display (' . $nextoffset . '-' . ($nextoffset + $segment) . ')',
				'contenttype' => self::CT_DOWN,
				'table' => NAVIGATION_TABLE,
				'typ' => self::TYP_DOTS,
				'open' => 0,
				'disabled' => 0,
				'tooltip' => '',
				'offset' => $nextoffset
			];
		}

		return $items;
	}

}
