<?php
declare(strict_types=1);

/* webEdition CMS
 *
 * $Rev: 44db4d7c3342 $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2020-06-24 18:25 +0200 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */

abstract class we_tag_calculate extends we_weTag_tag{
	/** @var bool */
	public static $active = false;

	public static function parse(array $attribs, string $content): string{
		$attribs['_type'] = 'stop';
		$content = strtr($content, ['<?=' => '', ';?>' => '', '; ?>' => '', '<?php' => '', 'echo' => '', ';' => '']);
		return '<?php ' . we_weTag_parser::printTag('calculate', ['_type' => 'start']) . '; echo ' . $content . ';echo ' . we_weTag_parser::printTag('calculate', $attribs) . ';?>';
	}

	/**
	 * @param array<string,mixed> $attribs parsed tag-attributes
	 * @param string $content content between openingtag and closing tag
	 * @param bool $internal if used by we internally
	 * @return string output to print
	 * @throws we_exception_parser if any attribute is missing
	 */
	public static function tag(array $attribs = [], string $content = '', bool $internal = false): string{
		//internal Attribute
		switch(self::getAttribute('_type', $attribs, '', we_base_request::STRING)){
			case 'start':
				self::$active = true;
				ob_start();
				return '';
			case 'stop':
				self::$active = false;
				$sum = self::getAttribute('sum', $attribs, '', we_base_request::STRING);
				$num_format = self::getAttribute('num_format', $attribs, '', we_base_request::STRING);
				$print = self::getAttribute('print', $attribs, true, we_base_request::BOOL);
				$result = (float) ($content ? trim($content) : 0);

				if($sum){
					we_tag_sum::addSum($sum, $result);
				}
				return ($print ? ($num_format ? we_base_util::formatNumber($result, $num_format, intval(self::getAttribute('decimals', $attribs, 2, we_base_request::INT))) : (string) $result) : '');
			default:
				self::attributeMissing($attribs, ['_type' => false], __CLASS__);
				return '';
		}
	}

}
