<?php
declare(strict_types=1);

/* webEdition CMS
 *
 * $Rev: 44db4d7c3342 $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2020-06-24 18:25 +0200 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */

abstract class we_tag_conditionAdd extends we_weTag_tag{

	/**
	 * @param array<string,mixed> $attribs parsed tag-attributes
	 * @param string $content content between openingtag and closing tag
	 * @param bool $internal if used by we internally
	 * @return string output to print
	 * @throws we_exception_parser if any attribute is missing
	 */
	public static function tag(array $attribs = [], string $content = '', bool $internal = false): string{
		self::attributeMissing($attribs, ['field' => false], __CLASS__);

		// initialize possible Attributes
		$field = self::getAttribute('field', $attribs, '', we_base_request::STRING);
		$value = strtr(self::getAttribute('value', $attribs, '', we_base_request::RAW), ['&gt;' => '>', '&lt;' => '<']);
		$compare = self::getAttribute('compare', $attribs, '=', we_base_request::RAW);
		$var = self::getAttribute('var', $attribs, '', we_base_request::STRING);
		$type = self::getAttribute('type', $attribs, '', we_base_request::STRING);
		$property = self::getAttribute('property', $attribs, false, we_base_request::BOOL);
		$exactmatch = self::getAttribute('exactmatch', $attribs, false, we_base_request::BOOL);
		$docAttr = self::getAttribute('doc', $attribs, '', we_base_request::STRING);
		// end initialize possible Attributes

		if(!we_tag_condition::$conditionName){//ignore if not in tag
			return '';
		}

		$regs = [];
		if($var && strtolower($compare) === 'like'){
			if(preg_match('/^(%)?([^%]+)(%)?$/', $var, $regs)){
				$var = $regs[2];
			}
		}
		$escape = true;
		switch(strtolower($type)){
			case 'now' :
				$value = time();
				$escape = false;
				break;
			case 'sessionfield':
				if($var && isset($_SESSION['webuser'][$var])){
					$value = $_SESSION['webuser'][$var];
				}
				break;
			case 'document':
				if($var){
					$doc = self::getDocForTag($docAttr, false);
					$value = ($property ? $doc->$var : $doc->getElementS($var));
				}
				break;
			case 'request':
				if($var && isset($_REQUEST[$var])){
					$value = we_base_request::string($var);
				}
				break;
			default:
				if($var && isset($GLOBALS[$var])){
					$value = $GLOBALS[$var];
				}
		}
		if($exactmatch){
			$compare = 'COLLATE utf8mb4_bin ' . $compare;
		}
		$value = ($regs[1] ?? '') . $value . ($regs[3] ?? '');

		if(strlen($field) && isset($GLOBALS[we_tag_condition::$conditionName])){
			$GLOBALS[we_tag_condition::$conditionName] .= '(' . $field . ' ' . $compare . ' ' . ($escape ? '"' : '') . DB_WE::$inst->escape($value) . ($escape ? '"' : '') . ') ';
		} else {
			$GLOBALS[we_tag_condition::$conditionName] .= (preg_match('/^(.*)AND ?$/', $GLOBALS[we_tag_condition::$conditionName]) ? '1 ' : '0 ');
		}
		return '';
	}

}
