<?php
declare(strict_types=1);

/* webEdition CMS
 *
 * $Rev: f389fd111aab $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2020-02-17 01:17 +0100 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */

abstract class we_tag_customer extends we_weTag_tag{

	public static function parse(array $attribs, string $content): string{
		$name = self::getParserAttribute('name', $attribs, '', we_base_request::STRING);
		if($name && strpos($name, ' ') !== false){
			return self::parseError(sprintf(g_l('parser', '[name_with_space]'), 'customer'), __CLASS__);
		}

		return '<?php ' . (strpos($content, '$lv') !== false ? 'global $lv;' : '') .
			'if(' . we_weTag_parser::printTag('customer', $attribs) . '){?>' . $content . '<?php }
		we_tag_listview::post(); ?>';
	}

	public static function tag(array $attribs = [], string $content = '', bool $internal = false): bool{
		if(!we_modules_info::isActive(we_modules_info::CUSTOMER)){
			echo self::modulMissingError('Customer', __CLASS__);
			return false;
		}

		$condition = self::getAttribute('condition', $attribs, 0, we_base_request::RAW);
		$we_cid = self::getAttribute('id', $attribs, 0, we_base_request::INT);
		$name = self::getAttribute('name', $attribs, '', we_base_request::STRING);
		$hidedirindex = self::getAttribute('hidedirindex', $attribs, TAGLINKS_DIRECTORYINDEX_HIDE, we_base_request::BOOL);

		if($name){
			$we_doc = we_base_WE::getWeDoc();
			$we_cid = intval(($we_doc->getElementI($name) ?:
				($we_doc->getElementS($name) ?:
				$we_cid)
			));

			$we_cid = $we_cid ?: we_base_request::int('we_cid', 0);

			if(we_base_WE::$editmode){
				self::editField($we_doc, $name, $we_cid, self::getAttribute('_name_orig', $attribs, '', we_base_request::STRING));
				return false;
			}
		} else {
			$we_cid = $we_cid ?: we_base_request::int('we_cid', 0);
		}

		$lv = new we_listview_customer('', 1, 0, '', false, 'ID=' . intval($we_cid) . ($condition ? ' AND ' . $condition : ''), 0, 0, $hidedirindex);
		we_tag_listview::pre($lv);

		$avail = $lv->next_record();
		/* if($avail){
		  //implement seem
		  } */
		return $avail;
	}

	protected static function editField(we_document_webEdition $doc, string $name, int $we_cid, string $showName): void{
		$textname = 'we_' . $doc->Name . '_customer[' . $name . '_path]';
		$idname = 'we_' . $doc->Name . '_customer[' . $name . '#' . we_contents_base::ETYPE_NUMBER . ']';
		$path = fS('SELECT Username FROM ' . CUSTOMER_TABLE . ' WHERE ID=' . $we_cid);
		?>
		<table class="we we_customer weEditTable">
			<tr>
				<td><?= $showName; ?></td>
				<td><?= we_html_tools::textInput($textname, $path, 0, ['readonly' => 'readonly'], 'text') . we_html_element::hidden($idname, $we_cid); ?></td>
				<td><?= we_html_button::create_button(we_html_button::SELECT, "javascript:we_cmd('we_customer_selector',{id:document.forms[0].elements." . $idname . ".value,targetID:'" . $idname . "',targetName:'" . $textname . ",jsCmd:'reload_editpage'});opener._EditorFrame.setEditorIsHot(true);','',0,'',1)"); ?></td>
				<td><?= we_html_button::create_button(we_html_button::TRASH, "javascript:document.forms[0].elements." . $idname . ".value=0;document.forms[0].elements['" . $textname . "'].value='';_EditorFrame.setEditorIsHot(false);we_cmd('reload_editpage');"); ?></td>
			</tr>
		</table><?php
	}

}
