<?php
declare(strict_types=1);

/* webEdition CMS
 *
 * $Rev: 44db4d7c3342 $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2020-06-24 18:25 +0200 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */

abstract class we_tag_ifIsDomain extends we_weTag_ifTag{

	/**
	 * @param array<string,mixed> $attribs parsed tag-attributes
	 * @param string $content content between openingtag and closing tag
	 * @param bool $internal if used by we internally
	 * @return string output to print
	 * @throws we_exception_parser if any attribute is missing
	 */
	public static function tag(array $attribs = [], string $content = '', bool $internal = false): bool{
		self::attributeMissing($attribs, ['domain' => false], __CLASS__);
		if(we_base_WE::$editmode){
			return true;
		}

		$domain = explode(',', strtolower(self::getAttribute('domain', $attribs, '', we_base_request::STRING)));
		$matchType = self::getAttribute('matchType', $attribs, 'exact', we_base_request::STRING);
		$servername = strtolower($_SERVER['SERVER_NAME']);
		switch($matchType){
			case 'exact':
				return in_array($servername, $domain, true);
			case 'contains':
				foreach($domain as $d){
					if(strpos($servername, $d) !== false){
						return true;
					}
				}
				return false;
			case 'front':
				foreach($domain as $d){
					if(strpos($servername, $d) === 0){
						return true;
					}
				}
				return false;
			case 'back':
				$len = strlen($servername);
				foreach($domain as $d){
					$pos = strpos($servername, $d);
					if($pos !== false && ($pos + strlen($d)) === $len){
						return true;
					}
				}
				return false;
			default:
				return false;
		}
	}

}
