<?php
declare(strict_types=1);

/* webEdition CMS
 *
 * $Rev: 44db4d7c3342 $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2020-06-24 18:25 +0200 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */

abstract class we_tag_input extends we_weTag_tag{

	/**
	 * @param array<string,mixed> $attribs parsed tag-attributes
	 * @param string $content content between openingtag and closing tag
	 * @param bool $internal if used by we internally
	 * @return string output to print
	 * @throws we_exception_parser if any attribute is missing
	 */
	public static function tag(array $attribs = [], string $content = '', bool $internal = false): string{
		self::attributeMissing($attribs, ['name' => false], __CLASS__);
		self::$doc = we_base_WE::getWeDoc();

		$name = self::getAttribute('name', $attribs, '', we_base_request::STRING);
		$value = self::getAttribute('value', $attribs, '', we_base_request::RAW);
		$type = self::getAttribute('type', $attribs, '', we_base_request::STRING);
		$val = htmlspecialchars(self::$doc->getElementS($name, $value));
		$attribs = self::removeAttribs($attribs, ['user']);

		if(we_base_WE::$editmode){
			//all edit-specific things
			if($type === ''){
				$seperator = self::getAttribute('seperator', $attribs, '|', we_base_request::RAW);
				$values = self::getAttribute('values', $attribs, '', we_base_request::RAW);
				$values = $values ? explode($seperator, $values) : [];
			} else {
				$values = self::getAttribute('values', $attribs, [], we_base_request::STRING_LIST);
			}
			$mode = self::getAttribute('mode', $attribs, '', we_base_request::STRING);
			$reload = self::getAttribute('reload', $attribs, false, we_base_request::BOOL);
			return we_html_element::span(['class' => 'we we_input'], self::getEdit($type, $name, $reload, $val, $values, $mode, $attribs));
		}
		//not-editmode
		switch($type){
			case 'date':
				return (string) self::$doc->getField($attribs, 'date');
			case 'checkbox':
				return (string) self::$doc->getElementI($name);
			case 'country':
				if(self::$doc->getElementS($name) === '--'){
					return '';
				}
				$langcode = self::getLangCode($attribs);
				return we_base_country::getTranslation(self::$doc->getElementS($name), we_base_country::TERRITORY, $langcode);
			case 'language':
				$langcode = self::getLangCode($attribs);
				return we_base_country::getTranslation(self::$doc->getElementS($name), we_base_country::LANGUAGE, $langcode);
			case 'choice':
				return self::$doc->getElementS($name);
			case 'select':
				return (string) self::$doc->getField($attribs, 'select');
			case 'text':
			default:
				return (string) self::$doc->getField($attribs);
		}
	}

	public static function getLangCode(array $attribs): string{
		$lang = self::getAttribute('outputlanguage', $attribs, '', we_base_request::STRING);
		if(!$lang){
			$docAttr = self::getAttribute('doc', $attribs, 'self', we_base_request::STRING);
			$doc = self::getDocForTag($docAttr);
			$lang = $doc->Language;
		}
		if(!$lang){
			$lang = explode('_', we_base_WE::$Language);
			return array_search($lang[0], we_base_country::getWELangs(), true) ?: 'de';
		}
		return substr($lang, 0, 2) ?: 'de';
	}

	protected static function editDate(string $name, bool $currentdate/* , string $format */): string{
		$d = abs(self::$doc->getElementI($name));
		return we_html_tools::getNewDateInput('we_' . self::$doc->Name . '_date[' . $name . '][%s]', new DateTime((date('Y-m-d H:i:s', $d ?: ($currentdate ? time() : 0)))), we_html_tools::FORMAT_DATE_TIME, true);
	}

	protected static function editcheckbox(string $name, bool $reload, string $val, array $attribs): string{
		$value = self::getAttribute('value', $attribs, '1', we_base_request::STRING);
		$active = self::$doc->issetElement($name) && self::$doc->getElementB($name);
		if($active){
			$attribs['checked'] = 'checked';
		}

		return we_html_element::hidden('we_' . self::$doc->Name . '_checkbox[' . $name . ']', $active ? $val : 0) .
			self::htmlTag('input', array_merge(self::removeAttribs($attribs, ['_name_orig', 'reload']), [
				'type' => 'checkbox',
				'name' => 'we_' . self::$doc->Name . '_attrib_' . $name,
				'value' => '1',
				'onclick' => '_EditorFrame.setEditorIsHot(true);this.form.elements[\'we_' . self::$doc->Name . '_checkbox[' . $name . ']\'].value=(this.checked ? ' . ((string) $value ) . ' : 0);' . ($reload ? (';setScrollTo();top.we_cmd(\'reload_editpage\');') : '')
		]));
	}

	protected static function editCountry(string $name, string $docAttr, array $attribs): string{
		$newAtts = self::removeAttribs($attribs, ['checked', 'type', 'options', 'selected', 'onchange', 'onChange', 'name', 'value', 'values', 'onclick', 'onClick', 'mode',
				'choice', 'pure', 'rows', 'cols', 'maxlength', 'wysiwyg']);
		$newAtts['name'] = 'we_' . self::$doc->Name . '_txt[' . $name . ']';
		$newAtts['onclick'] = '_EditorFrame.setEditorIsHot(true);';

		$doc = self::getDocForTag($docAttr);
		$lang = $doc->Language;
		$langcode = self::getLangCode($attribs);

		//$orgVal = self::$doc->getElement($name);
		$topCountries = array_flip(explode(',', WE_COUNTRIES_TOP));
		foreach($topCountries as $countrykey => &$countryvalue){
			$countryvalue = we_base_country::getTranslation((string) $countrykey, we_base_country::TERRITORY, $langcode);
		}
		unset($countryvalue);
		$shownCountries = array_flip(explode(',', WE_COUNTRIES_SHOWN));
		foreach($shownCountries as $countrykey => &$countryvalue){
			$countryvalue = we_base_country::getTranslation((string) $countrykey, we_base_country::TERRITORY, $langcode);
		}
		unset($countryvalue);
		$oldLocale = setlocale(LC_ALL, NULL);
		setlocale(LC_ALL, $lang . '.UTF-8');
		asort($topCountries, SORT_LOCALE_STRING);
		asort($shownCountries, SORT_LOCALE_STRING);
		if($oldLocale){
			setlocale(LC_ALL, $oldLocale);
		}
		$orgVal = self::$doc->getElementS($name);
		$content = '';
		if(WE_COUNTRIES_DEFAULT != ''){
			$content .= '<option value="--" ' . ($orgVal === '--' ? ' selected="selected">' : '>') . WE_COUNTRIES_DEFAULT . '</option>';
		}
		foreach($topCountries as $countrykey => &$countryvalue){
			$content .= '<option value="' . $countrykey . '" ' . ($orgVal == $countrykey ? ' selected="selected">' : '>') . $countryvalue . '</option>';
		}
		unset($countryvalue);
		$content .= '<option value="-" disabled="disabled">----</option>';
		foreach($shownCountries as $countrykey2 => &$countryvalue2){
			$content .= '<option value="' . $countrykey2 . '" ' . ($orgVal == $countrykey2 ? ' selected="selected">' : '>') . $countryvalue2 . '</option>';
		}
		unset($countryvalue2);

		return self::htmlTag('select', $newAtts, $content, true);
	}

	protected static function editLanguage(string $name, string $docAttr, array $attribs): string{
		$newAtts = self::removeAttribs($attribs, ['checked', 'type', 'options', 'selected', 'onchange', 'onChange', 'name', 'value', 'values', 'onclick', 'onClick', 'mode',
				'choice', 'pure', 'rows', 'cols', 'maxlength', 'wysiwyg']);
		$newAtts['name'] = 'we_' . self::$doc->Name . '_txt[' . $name . ']';
		$newAtts['onclick'] = '_EditorFrame.setEditorIsHot(true);';
		$doc = self::getDocForTag($docAttr);
		$lang = $doc->Language;
		$langcode = (string) ($lang ? substr($lang, 0, 2) : array_search(we_base_WE::$Language, we_base_country::getWELangs(), true));

		$frontendL = we_base_WE::$Langs['FrontendLanguages'];
		foreach($frontendL as &$lcvalue){
			$lccode = explode('_', $lcvalue);
			$lcvalue = $lccode[0];
		}
		$frontendLL = [];
		foreach($frontendL as &$lcvalue){
			$frontendLL[$lcvalue] = we_base_country::getTranslation($lcvalue, we_base_country::LANGUAGE, $langcode);
		}

		$oldLocale = setlocale(LC_ALL, NULL);
		setlocale(LC_ALL, $lang . '.UTF-8');
		asort($frontendLL, SORT_LOCALE_STRING);
		if($oldLocale){
			setlocale(LC_ALL, $oldLocale);
		}
		$content = '';
		$orgVal = self::$doc->getElementS($name);
		foreach($frontendLL as $langkey => &$langvalue){
			$content .= '<option value="' . $langkey . '" ' . ($orgVal == $langkey ? ' selected="selected">' : '>') . $langvalue . '</option>';
		}
		unset($langvalue);
		return self::htmlTag('select', $newAtts, $content, true);
	}

	protected static function editChoice(string $name, string $val, array $values, string $mode, bool $reload, array $attribs): string{
		if($values){
			$tagname = 'we_' . self::$doc->Name . '_txt[' . $name . ']';

			$onChange = ($mode === 'add' ?
				"this.form.elements['" . $tagname . "'].value += ((this.form.elements['" . $tagname . "'].value ? ' ' : '')+this.options[this.selectedIndex].text);" :
				"this.form.elements['" . $tagname . "'].value = this.options[this.selectedIndex].text;") .
				($reload ? 'setScrollTo();top.we_cmd(\'reload_editpage\');' : '');

			$sel = self::htmlTag('select', [
					'class' => 'defaultfont',
					'name' => 'we_choice_' . self::$doc->Name . '_txt[' . $name . ']',
					'onchange' => $onChange . ';this.selectedIndex=0;_EditorFrame.setEditorIsHot(true);'
					], '<option disabled="disabled">' . g_l('button', '[select][value]') . '</option><option>' . implode('</option><option>', $values) . '</option>', true);
		} else {
			$sel = '';
		}

		$attribs['onchange'] = '_EditorFrame.setEditorIsHot(true);';
		$attribs['type'] = 'text';
		$attribs['name'] = 'we_' . self::$doc->Name . '_txt[' . $name . ']';
		$attribs['value'] = $val;

		return self::htmlTag('input', self::removeAttribs($attribs, ['mode', 'values', '_name_orig'])) . '&nbsp;' . $sel;
	}

	protected static function editDefault(string $name, string $val, array $attribs): string{
		$attribs['onchange'] = '_EditorFrame.setEditorIsHot(true);';
		$attribs['class'] = 'wetextinput';
		$attribs['type'] = 'text';
		$attribs['name'] = 'we_' . self::$doc->Name . '_txt[' . $name . ']';
		$attribs['value'] = $val;
		return self::htmlTag('input', self::removeAttribs($attribs, ['mode', 'values', '_name_orig']));
	}

	protected static function getEdit(string $type, string $name, bool $reload, string $val, array $values, string $mode, array $attribs): string{
		switch($type){
			case 'date':
				//$format = self::getAttribute('format', $attribs, '', we_base_request::STRING);
				$currentdate = self::getAttribute('currentdate', $attribs, false, we_base_request::BOOL);
				return self::editDate($name, $currentdate/* , $format */);
			case 'checkbox':
				return self::editCheckbox($name, $reload, $val, $attribs);
			case 'country':
				$docAttr = self::getAttribute('doc', $attribs, 'self', we_base_request::STRING);
				return self::editCountry($name, $docAttr, $attribs);
			case 'language':
				$docAttr = self::getAttribute('doc', $attribs, 'self', we_base_request::STRING);
				return self::editLanguage($name, $docAttr, $attribs);
			case 'choice':
				return self::editChoice($name, $val, $values, $mode, $reload, $attribs);
			case 'select':
				//NOTE: this tag is for objects only
				return (string) self::$doc->getField($attribs, 'select');
			case 'print':
				return $val;
			case 'text':
			default:
				return self::editDefault($name, $val, $attribs);
		}
	}

}
