<?php
declare(strict_types=1);

/* webEdition CMS
 *
 * $Rev: 44db4d7c3342 $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2020-06-24 18:25 +0200 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */

abstract class we_tag_select extends we_weTag_tag{

	public static function parse(array $attribs, string $content): string{
		if(self::getParserAttribute('type', $attribs, 'html', we_base_request::STRING) === 'csv'){
			return parent::parse($attribs, '');
		}
		return '<?php if(we_base_WE::$editmode){ob_start();?>' . $content . '<?php $we_select_content=ob_get_clean()?:\'\';}else{$we_select_content=\'\';}'
			. 'echo ' . we_weTag_parser::printTag('select', $attribs, '$we_select_content', false, true) . ';?>';
	}

	/**
	 * @param array<string,mixed> $attribs parsed tag-attributes
	 * @param string $content content between openingtag and closing tag
	 * @param bool $internal if used by we internally
	 * @return string output to print
	 * @throws we_exception_parser if any attribute is missing
	 */
	public static function tag(array $attribs = [], string $content = '', bool $internal = false): string{
		self::attributeMissing($attribs, ['name' => false], __CLASS__);
		self::$doc = we_base_WE::getWeDoc();

		$name = self::getAttribute('name', $attribs, '', we_base_request::STRING);
		$multiple = self::getAttribute('multiple', $attribs, false, we_base_request::BOOL);
		$val = (string) (self::$doc->getElementS($name) ?: self::$doc->getElementI($name));
		if(!we_base_WE::$editmode){
			return $val;
		}
		$val = $val ? explode(',', $val) : [];

		$onchange = self::getAttribute('onchange', $attribs, '', we_base_request::JS);
		$reload = self::getAttribute('reload', $attribs, false, we_base_request::BOOL);
		switch(self::getAttribute('type', $attribs, 'html', we_base_request::STRING)){
			case 'csv':
				$vals = self::getAttribute('values', $attribs, array_filter(explode(',', $content)), we_base_request::STRING_LIST);
				$content = '';
				foreach($vals as $cur){
					$content .= (in_array($cur, $val, true) ?
						self::htmlTag('option', ['value' => $cur, 'selected' => 'selected'], $cur, true) :
						self::htmlTag('option', ['value' => $cur], $cur, true)
						);
				}
				break;
			case 'html':
			default:
				$content = preg_replace('|<(option[^>]*) selected( *=? *"selected")?([^>]*)>|i', '<${1}${3}>', $content);
				if(stripos($content, '<option>') !== false){
					foreach($val as $v){
						$content = preg_replace('|<option>' . preg_quote($v) . '( ?[<\n\r\t])|i', '<option selected="selected">' . $v . '${1}', $content);
					}
				}
				if(preg_match('|<option[^>]*value=[\'"]?.*[\'"]?>|i', $content)){
					foreach($val as $v){
						$content = preg_replace('|<option([^>]*)value *= *"' . preg_quote($v) . '"([^>]*)>|i', '<option value="' . $v . '" selected="selected">', $content);
					}
				}
				break;
		}
		$attribs = self::removeAttribs($attribs, ['reload', 'value', '_name_orig', 'type', 'values', 'multiple']); //	not html - valid
		if($multiple){
			$attribs['multiple'] = 'multiple';
		}
		$attribs['class'] = 'defaultfont';
		$attribs['name'] = 'we_' . self::$doc->Name . '_select[' . $name . ($multiple ? '#multiple][]' : ']');
		$attribs['onchange'] = '_EditorFrame.setEditorIsHot(true);' . ($onchange ?: '') . ';' . ($reload ? (';setScrollTo();top.we_cmd(\'reload_editpage\');') : '');
		return we_html_element::span(['class' => 'we we_select'], self::htmlTag('select', $attribs, $content, true));
	}

}
