<?php
declare(strict_types=1);

/* webEdition CMS
 *
 * $Rev: 44db4d7c3342 $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2020-06-24 18:25 +0200 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
we_modules_info::isActive(we_modules_info::SHOP);

abstract class we_tag_showShopItemNumber extends we_weTag_tag{

	/**
	 * @param array<string,mixed> $attribs parsed tag-attributes
	 * @param string $content content between openingtag and closing tag
	 * @param bool $internal if used by we internally
	 * @return string output to print
	 * @throws we_exception_parser if any attribute is missing
	 */
	public static function tag(array $attribs = [], string $content = '', bool $internal = false): string{
		self::attributeMissing($attribs, ['shopname' => false], __CLASS__);

		$shopname = self::getAttribute('shopname', $attribs, '', we_base_request::STRING);
		$type = self::getAttribute('option', $attribs, false, we_base_request::BOOL) ? 'select' : '';
		$type = $type ?: (self::getAttribute('inputfield', $attribs, false, we_base_request::BOOL) ? 'textinput' : '');
		$type = $type ?: (self::getAttribute('type', $attribs, '', we_base_request::STRING));
		$num_format = self::getAttribute('num_format', $attribs, '', we_base_request::STRING);
		$floatquantities = self::getAttribute('floatquantities', $attribs, false, we_base_request::BOOL);

		try{
			$basket = we_tag_createShop::getBasket($shopname);
		} catch (Exception $ex){
			return self::parseError(sprintf(g_l('parser', '[missing_createShop]'), 'showShopItemNumber'), __CLASS__);
		}
		$lv = we_tag_listview::$current;
		if(!$lv || !$lv instanceof we_listview_shopItems){
			return '';
		}
		$attr = self::removeAttribs($attribs, ['option', 'inputfield', 'type', 'start', 'stop', 'shopname', 'nameto', 'to', 'floatquantities', 'num_format']);

		// $type of the field
		//$articleType = ( $lv->f('OF_ID') ? we_shop_shop::OBJECT : we_shop_shop::DOCUMENT);
//		assert($lv instanceof we_listview_shopItems);
		$cartKey = $lv->getShoppingCartKey();
		$itemQuantity = $basket->getItemQuantity($cartKey);

		switch($type){
			case 'select':
				$start = self::getAttribute('start', $attribs, 0, we_base_request::INT);
				$stop = self::getAttribute('stop', $attribs, 10, we_base_request::INT);
				$step = self::getAttribute('step', $attribs, 1, we_base_request::INT);
				$stop = ($floatquantities ?
					( $stop > $itemQuantity ? $stop : $itemQuantity) :
					( intval($stop) > intval($itemQuantity) ? $stop : $itemQuantity));

				$attr['name'] = 'shop_cart_id[' . $cartKey . ']';
				$attr['size'] = 1;

				$out = '';
				while($start <= $stop){
					$out .= ($itemQuantity == $start ?
						self::htmlTag('option', ['value' => $start, 'selected' => 'selected'], $start) :
						self::htmlTag('option', ['value' => $start], $start));

					$start = $start + $step;
				}
				return self::htmlTag('select', $attr, $out, true) . self::htmlTag('input', ['type' => 'hidden', 'name' => 't', 'value' => time()]);

			case 'textinput':
				$itemQuantity = ($floatquantities ? we_base_util::formatNumber($itemQuantity, $num_format, 2) : intval($itemQuantity));
				$attr = array_merge($attr, ['type' => 'text', 'name' => 'shop_cart_id[' . $cartKey . ']', 'size' => 2, 'value' => $itemQuantity]);
				return self::htmlTag('input', $attr) . self::htmlTag('input', ['type' => 'hidden', 'name' => 't', 'value' => time()]);

			default:
				return ($floatquantities ? we_base_util::formatNumber($itemQuantity, $num_format, 2) : strval(intval($itemQuantity)));
		}
	}

}
