<?php
declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: 3f0d3cd72b0f $
 * $Author: Lukas Imhof $
 * $Date: 2019-10-14 13:19 +0200 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */

/**
 * class representing the model data of the search
 */
class we_versions_model extends we_search_modelBase{
	/** @var string */
	public $whichSearch;

	/** @var int */
	public $height = 0;

	/** @var string */
	public $transaction = '';

	/** @var int position to start the search	 */
	protected $searchstartVersionSearch = 0;

	/** @var array includes the text to search for	 */
	protected $searchVersionSearch = [];

	/** @var array includes the operators	 */
	protected $locationVersionSearch = [];

	/** @var int folder-ids of the docsearch and the tmplsearch	 */
	protected $folderIDVersion = 0;

	/** @var string	 */
	protected $setViewVersionSearch = 'list';

	/** @var int gives the number of entries in each search for one page	 */
	protected $anzahlVersionSearch = 10;

	/** @var string gives the order	 */
	protected $OrderVersionSearch = 'ID DESC';
	protected $searchTablesVersionSearch = [];

	/** @var array includes the searchfiels which you are searching in	 */
	protected $searchFieldsVersionSearch = [];

	/**
	 * Default Constructor
	 * Can load or create new searchtool object depends of parameter
	 */
	function __construct(string $transaction, string $setView = 'list'){
		//as we do actually not save this model to db nor session we do call parent
		//parent::__construct(SEARCH_TABLE);

		$this->transaction = $transaction;
		$this->searchTablesVersionSearch = [VERSIONS_TABLE];
		$this->whichSearch = we_search_view::SEARCH_VERSION;
	}

	public function initByHttp(string $whichSearch = '', bool $isWeCmd = true): void{
		// IMPORTANT: this is the ONLY place where model vars are set!
		if(isset($_REQUEST['searchstart'])){
			$this->mode = we_base_request::int('mode', $this->mode);
			$this->OrderVersionSearch = we_base_request::string('order', $this->OrderVersionSearch);
			$this->anzahlVersionSearch = we_base_request::int('anzahl', $this->anzahlVersionSearch);
			$this->searchstartVersionSearch = we_base_request::int('searchstart', $this->searchstartVersionSearch);
			$this->searchFieldsVersionSearch = we_base_request::stringList('searchFields');
			$this->height = count($this->searchFieldsVersionSearch);
			$this->locationVersionSearch = we_base_request::stringList('location', $this->locationVersionSearch);
			$this->searchVersionSearch = we_base_request::stringList('search', $this->searchVersionSearch);
		} else {
			if(isset($_REQUEST['we_cmd']['searchstart'])){
				$request = we_base_request::stringList('we_cmd');
				foreach($request as $k => $v){
					$key = (string) $k;
					if(stristr($key, 'searchFields[') && !stristr($key, 'hidden_')){
						$_REQUEST['we_cmd']['searchFields'][] = $v;
						continue;
					}
					if(stristr($key, 'location[')){
						$_REQUEST['we_cmd']['location'][] = $v;
						continue;
					}
					if(stristr($key, 'search[')){
						$_REQUEST['we_cmd']['search'][] = $v;
					}
				}

				$this->mode = we_base_request::int('we_cmd', $this->mode, 'mode');
				$this->OrderVersionSearch = we_base_request::string('we_cmd', $this->OrderVersionSearch, 'order');
				$this->searchstartVersionSearch = we_base_request::int('we_cmd', $this->searchstartVersionSearch, 'searchstart');
				$this->anzahlVersionSearch = we_base_request::int('we_cmd', $this->anzahlVersionSearch, 'anzahl');
				$this->searchFieldsVersionSearch = we_base_request::stringList('we_cmd', [], 'searchFields');
				$this->height = count($this->searchFieldsVersionSearch);
				$this->searchVersionSearch = array_map('trim', we_base_request::stringList('we_cmd', [], 'search'));
				$this->locationVersionSearch = we_base_request::stringList('we_cmd', [], 'location');
			}
		}

		$this->prepareModelForSearch();
	}

	public function prepareModelForSearch(): void{
		$this->currentSearchTables = $this->searchTablesVersionSearch;
		$this->currentSearchFields = $this->searchFieldsVersionSearch;
		$this->currentLocation = $this->locationVersionSearch;
		$this->currentSearch = $this->searchVersionSearch;
		$this->currentFolderID = $this->folderIDVersion;
		$this->currentOrder = $this->OrderVersionSearch;
		$this->currentSetView = $this->setViewVersionSearch;
		$this->currentSearchstart = $this->searchstartVersionSearch;
		$this->currentAnzahl = $this->anzahlVersionSearch;
	}

	public function getWhichSearch(): string{
		return $this->whichSearch;
	}

}
