<?php
declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: f4d8c0d614d0 $
 * $Author: Lukas Imhof $
 * $Date: 2020-06-02 16:29 +0200 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
class we_widget_shp extends we_widget_base{
	/** @var string */
	private $shopDashboard = '';

	/** @var string */
	private $newSCurrId = '';

	/** @var string */
	private $interval = '';

	/** @var array */
	private $jsConfig = [];

	public function __construct(string $curID = '', array $aProps = []){
		if(!$curID){//preview requested
			$aCols = we_base_request::string('we_cmd');
			$this->newSCurrId = we_base_request::string('we_cmd', '', 5);
		} else {
			$aCols = [];
			$this->newSCurrId = $curID;
		}
		if(!isset($aCols[5])){
			$aCols = explode(';', $aProps[3]);
		}

		$sKPIs = $aCols[0] ?: [];
		$bOrders = !empty($sKPIs[0]);
		$bCustomer = !empty($sKPIs[1]);
		$bAverageOrder = !empty($sKPIs[2]);
		$bTarget = !empty($sKPIs[3]);

		$iDate = intval($aCols[1]);
		$sRevenueTarget = intval($aCols[2]);
//note we use between, which means a between b and c is b<=a<=c, in case of date hour is always set to "00:00:00"
		switch($iDate){//FIXME: use cast & between to make this perform better
			default:
			case 0 : //heute
				$queryShopDateCondtion = '(o.DateOrder BETWEEN CURRENT_DATE() AND (CURRENT_DATE()+INTERVAL 1 DAY))';
				$timestampCustomer = '(MemberSince_Date >= CURDATE())';
				$this->interval = g_l('cockpit', '[today]');
				break;
			case 1 : //diese woche
				$queryShopDateCondtion = '(YEARWEEK(o.DateOrder,1) = YEARWEEK(CURDATE(),1))';
				$timestampCustomer = '(YEARWEEK(MemberSince_date,1) = YEARWEEK(CURDATE(),1))';
				$this->interval = g_l('cockpit', '[this_week]');
				break;
			case 2 : //letzte woche
				$queryShopDateCondtion = '(YEARWEEK(o.DateOrder,1) = YEARWEEK(CURDATE()-INTERVAL 1 WEEK,1))';
				$timestampCustomer = '(YEARWEEK(MemberSince_Date,1) = YEARWEEK(CURDATE()-INTERVAL 1 WEEK,1))';
				$this->interval = g_l('cockpit', '[last_week]');
				break;
			case 3 : //dieser monat
				$queryShopDateCondtion = '(o.DateOrder BETWEEN CONCAT(YEAR(CURDATE()),"-",MONTH(CURDATE()),"-01") AND (CONCAT(YEAR(CURDATE()),"-",MONTH(CURDATE()),"-01")+INTERVAL 1 MONTH))';
				$timestampCustomer = '(MemberSince_Date BETWEEN CONCAT(YEAR(CURDATE()),"-",MONTH(CURDATE()),"-01") AND (CONCAT(YEAR(CURDATE()),"-",MONTH(CURDATE()),"-01")+INTERVAL 1 MONTH))';
				$this->interval = g_l('cockpit', '[this_month]');
				break;
			case 4 : //letzter monat
				$queryShopDateCondtion = '(o.DateOrder BETWEEN (CONCAT(YEAR(CURDATE()),"-",MONTH(CURDATE()),"-01")-INTERVAL 1 MONTH) AND CONCAT(YEAR(CURDATE()),"-",MONTH(CURDATE()),"-01"))';
				$timestampCustomer = '(MemberSince_Date BETWEEN (CONCAT(YEAR(CURDATE()),"-",MONTH(CURDATE()),"-01")-INTERVAL 1 MONTH) AND CONCAT(YEAR(CURDATE()),"-",MONTH(CURDATE()),"-01"))';
				$this->interval = g_l('cockpit', '[last_month]');
				break;
			case 5 : //dieses jahr
				$queryShopDateCondtion = '(o.DateOrder BETWEEN MAKEDATE(YEAR(CURDATE()),1) AND MAKEDATE(YEAR(CURDATE())+1,1))';
				$timestampCustomer = '(MemberSince_Date BETWEEN MAKEDATE(YEAR(CURDATE()),1) AND MAKEDATE(YEAR(CURDATE())+1,1))';
				$this->interval = g_l('cockpit', '[this_year]');
				break;
			case 6 : //letztes jahr
				$queryShopDateCondtion = '(o.DateOrder BETWEEN MAKEDATE(YEAR(CURDATE())-1,1) AND MAKEDATE(YEAR(CURDATE()),1))';
				$timestampCustomer = '(MemberSince_Date BETWEEN MAKEDATE(YEAR(CURDATE())-1,1) AND MAKEDATE(YEAR(CURDATE()),1))';
				$this->interval = g_l('cockpit', '[last_year]');
				break;
		}

// get some preferences!
		$feldnamen = explode('|', fS('SELECT pref_value FROM ' . SETTINGS_TABLE . ' WHERE tool="shop" AND pref_name="shop_pref"'));
		$currency = htmlspecialchars($feldnamen[0]);
		$numberformat = $feldnamen[2];
		//$classid = (isset($feldnamen[3]) ? $feldnamen[3] : '');
		$defaultVat = ($feldnamen[1] ?: 0);

		$amountOrders = $amountArticles = $amountCanceledOrders = $canceled = 0;

		$total = $payed = $unpayed = 0;
		if(we_modules_info::isActive(we_modules_info::SHOP) && (we_base_permission::hasPerm([we_base_perm::SHOP_NEW_ARTICLE, we_base_perm::SHOP_DELETE_ARTICLE, we_base_perm::SHOP_EDIT_ORDER,
				we_base_perm::SHOP_DELETE_ORDER, we_base_perm::SHOP_EDIT_PREFS]))){
			if(fB('SELECT 1 FROM ' . SHOP_ORDER_TABLE . ' o JOIN ' . SHOP_ORDER_ITEM_TABLE . ' oi ON o.ID=oi.orderID WHERE ' . $queryShopDateCondtion)){

				$amountOrders = fI('SELECT COUNT(1) FROM ' . SHOP_ORDER_TABLE . ' o WHERE ' . $queryShopDateCondtion);
				$amountCanceledOrders = fI('SELECT COUNT(1) FROM ' . SHOP_ORDER_TABLE . ' o WHERE ' . $queryShopDateCondtion . ' AND o.DateCancellation IS NOT NULL');
				$amountArticles = fI('SELECT COUNT(1) FROM ' . SHOP_ORDER_TABLE . ' o JOIN ' . SHOP_ORDER_ITEM_TABLE . ' oi ON o.ID=oi.orderID WHERE ' . $queryShopDateCondtion);
				$query = 'SELECT
SUM((oi.Price*oi.quantity*IF(o.pricesNet && o.calcVat && COALESCE(oi.Vat,' . $defaultVat . '),(1+COALESCE(oi.Vat,' . $defaultVat . ')/100),1)))
FROM ' . SHOP_ORDER_TABLE . ' o JOIN ' . SHOP_ORDER_ITEM_TABLE . ' oi ON o.ID=oi.orderID WHERE ' . $queryShopDateCondtion;

				$payed = f($query . ' AND o.DatePayment IS NOT NULL AND o.DateCancellation IS NULL');
				$canceled = f($query . ' AND o.DatePayment IS NULL AND o.DateCancellation IS NOT NULL');
				$unpayed = f($query . ' AND o.DatePayment IS NULL AND o.DateCancellation IS NULL');
				$total = $payed + $canceled + $unpayed;
			}
		}

		$amountCustomers = (we_modules_info::isActive(we_modules_info::CUSTOMER) && we_base_permission::hasPerm([we_base_perm::CUSTOMER_EDIT]) ?
			fI('SELECT COUNT(1) FROM ' . CUSTOMER_TABLE . '	WHERE ' . $timestampCustomer) :
			0);

		$shopDashboardTable = new we_html_table(['class' => 'default'], 1, 2);
		$i = 0;
		if($bOrders){
			//1. row
			$shopDashboardTable
				->setCol($i, 0, we_html_element::b(g_l('cockpit', '[shop_dashboard][cnt_order]')), ['class' => 'middlefont'])
				->setCol($i, 1, we_html_element::b(strval($amountOrders > 0 ? $amountOrders : 0)), ['class' => 'middlefont', 'style' => "text-align:right"])

				//2. row
				->setCol(++$i, 0, g_l('cockpit', '[shop_dashboard][canceled_order]'), ['class' => 'middlefont', 'style' => "color:red;"])
				->setCol($i, 1, strval($amountCanceledOrders > 0 ? $amountCanceledOrders : 0), ['class' => 'middlefont', 'style' => "text-align:right;color:red;"])

				//3. row
				->setCol(++$i, 0, g_l('cockpit', '[shop_dashboard][cnt_articles]'), ['class' => 'middlefont'])
				->setCol($i, 1, strval($amountArticles > 0 ? $amountArticles : 0), ['class' => 'middlefont', 'style' => "text-align:right"])

				//4. row
				->setCol(++$i, 0, g_l('cockpit', '[shop_dashboard][articles_order]'), ['class' => 'middlefont'])
				->setCol($i, 1, we_base_util::formatNumber(($amountArticles > 0 ? ($amountArticles / $amountOrders) : 0), $numberformat), ['class' => 'middlefont', 'style' => 'text-align:right;padding-bottom:2ex;']);
		}

		if($bAverageOrder){
			//order volume
			$shopDashboardTable
				->setCol(++$i, 0, we_html_element::b(g_l('cockpit', '[shop_dashboard][order_volume]')), ['class' => 'middlefont'])
				->setCol($i, 1, we_html_element::b(we_base_util::formatNumber((float) $total, $numberformat) . '&nbsp;' . $currency), ['class' => 'middlefont', 'style' => "text-align:right"])

				//canceled volume
				->setCol(++$i, 0, g_l('cockpit', '[shop_dashboard][canceled]'), ['class' => 'middlefont', 'style' => 'color:red;'])
				->setCol($i, 1, we_base_util::formatNumber((float) $canceled, $numberformat) . '&nbsp;' . $currency, ['class' => 'middlefont', 'style' => 'text-align:right;color:red;'])

				//revenue
				->setCol(++$i, 0, we_html_element::b(g_l('cockpit', '[shop_dashboard][revenue]')), ['class' => 'middlefont'])
				->setCol($i, 1, we_html_element::b(we_base_util::formatNumber(($total - $canceled), $numberformat) . '&nbsp;' . $currency), ['class' => 'middlefont', 'style' => "text-align:right"])

				//payed volume
				->setCol(++$i, 0, g_l('cockpit', '[shop_dashboard][payed]'), ['class' => 'middlefont', 'style' => "color:green;"])
				->setCol($i, 1, we_base_util::formatNumber((float) $payed, $numberformat) . '&nbsp;' . $currency, ['class' => 'middlefont', 'style' => "text-align:right;color:green;"])

				//unpayed volume
				->setCol(++$i, 0, g_l('cockpit', '[shop_dashboard][unpayed]'), ['class' => 'middlefont', 'style' => "color:red;"])
				->setCol($i, 1, we_base_util::formatNumber((float) $unpayed, $numberformat) . '&nbsp;' . $currency, ['class' => 'middlefont', 'style' => "text-align:right;color:red;"])

				//volume per order
				->setCol(++$i, 0, g_l('cockpit', '[shop_dashboard][order_value_order]'), ['class' => 'middlefont'])
				->setCol($i, 1, we_base_util::formatNumber(($amountOrders > 0 ? ($total / $amountOrders) : 0), $numberformat) . '&nbsp;' . $currency, ['class' => 'middlefont',
					'style' => "text-align:right"])

				//need some space
				->setCol(++$i, 0, '&nbsp;', ['class' => 'middlefont'])
				->setCol($i, 1, '&nbsp;', ['class' => 'middlefont']);
		}

		if($bCustomer){
			//new customer
			$shopDashboardTable
				->setCol(++$i, 0, we_html_element::b(g_l('cockpit', '[shop_dashboard][cnt_new_customer]')), ['class' => 'middlefont'])
				->setCol($i, 1, we_html_element::b(strval($amountCustomers > 0 ? $amountCustomers : 0)), ['class' => 'middlefont', 'style' => "text-align:right"]);
		}

		$this->shopDashboard = '<div style="width:60%;float:left;">' .
			$shopDashboardTable->getHtml() .
			'</div>'
			. '<div style="width:40%;float:right;">' . ($bTarget ? '<b>' . g_l('cockpit', '[shop_dashboard][revenue_target]') . '&nbsp;' . we_base_util::formatNumber($sRevenueTarget, $numberformat) . '&nbsp;' . $currency . '</b><br/>' : '') .
			//note: canvas doesn't support CSS width/height....
			'<canvas id="' . $this->newSCurrId . '_chart_div" width="160" height="160"></canvas>' .
			'</div><br style="clear:both;"/>';
		$this->jsConfig = [
			'value' => we_base_util::formatNumber(($total - $canceled)),
			'label' => 'Ziel in ' . $currency,
			'unitsLabel' => ' ' . $currency,
			'min' => 0,
			'max' => ($sRevenueTarget * 2),
			'minorTicks' => 5, // small ticks inside each major tick
			'greenFrom' => ($sRevenueTarget * 1.1),
			'greenTo' => ($sRevenueTarget * 2),
			'yellowFrom' => ($sRevenueTarget * 0.9),
			'yellowTo' => ($sRevenueTarget * 1.1),
			'redFrom' => 0,
			'redTo' => ($sRevenueTarget * 0.9)
		];
	}

	/**
	 * @return array{string,array{string,string}}
	 */
	public function getInsertDiv(int $iCurrId, array $aProps, we_base_jsCmd $jsCmd): array{
		$cfg = self::getDefaultConfig();
		$oTblDiv = we_html_element::div(
				['id' => 'm_' . $iCurrId . '_inline',
					'style' => 'height:' . ($cfg['height'] - 25) . 'px;overflow:auto;'
				], we_html_element::div(['id' => 'shp_data'], $this->shopDashboard)
		);
		$aLang = [g_l('cockpit', '[shop_dashboard][headline]') . '&nbsp;' . $this->interval, ''];
		$jsCmd->addCmd('initGauge', $this->newSCurrId . '_chart_div', $this->jsConfig);
		return [$oTblDiv, $aLang];
	}

	/**
	 * @return array<string,mixed>
	 */
	public static function getDefaultConfig(): array{
		return [
			'width' => self::WIDTH_LARGE,
			'expanded' => 1,
			'height' => 212,
			'res' => 1,
			'cls' => 'lightCyan',
			'csv' => '1111;3;10000;',
			'dlgHeight' => 435,
			'isResizable' => false
		];
	}

	public static function showDialog(): void{
		list($jsFile, $oSelCls) = self::getDialogPrefs();

		list($sType, $iDate, $sRevenueTarget) = explode(';', we_base_request::string('we_cmd', '', 1));


// Typ block
		while(strlen($sType) < 4){
			$sType .= '0';
		}
		if($sType === "0000"){
			$sType = "1111";
		}

		$oChbxCustomer = (we_modules_info::isActive(we_modules_info::CUSTOMER) && we_base_permission::hasPerm([we_base_perm::CUSTOMER_EDIT]) ?
			we_html_forms::checkbox('0', boolval($sType[1]), "chbx_type", g_l('cockpit', '[shop_dashboard][cnt_new_customer]'), true, "defaultfont", "", !( we_base_permission::hasPerm([
					we_base_perm::CUSTOMER_EDIT])), "", 0, 0) :
			'');

		if(we_modules_info::isActive(we_modules_info::SHOP) && (we_base_permission::hasPerm([we_base_perm::SHOP_NEW_ARTICLE, we_base_perm::SHOP_DELETE_ARTICLE, we_base_perm::SHOP_EDIT_ORDER,
				we_base_perm::SHOP_DELETE_ORDER, we_base_perm::SHOP_EDIT_PREFS]))){
			$oChbxOrders = we_html_forms::checkbox('0', boolval($sType[0]), "chbx_type", g_l('cockpit', '[shop_dashboard][cnt_order]'), true, '', '', !( we_base_permission::hasPerm([
						we_base_perm::SHOP_EDIT_ORDER])));
			$oChbxAverageOrder = we_html_forms::checkbox('0', boolval($sType[2]), 'chbx_type', g_l('cockpit', '[shop_dashboard][revenue_order]'), true, '', '', !(we_base_permission::hasPerm([
						we_base_perm::SHOP_EDIT_ORDER])));
			$oChbxTarget = we_html_forms::checkbox('0', boolval($sType[3]), 'chbx_type', g_l('cockpit', '[shop_dashboard][revenue_target]'), true, '', '', !(we_base_permission::hasPerm([
						we_base_perm::SHOP_EDIT_ORDER])));

			//$revenueTarget = we_html_forms::textinput($value = "",$name = "input_revenueTarget", $text = "Umsatzziel", $uniqid = true, $class = "defaultfont",$onClick = "", $disabled = !( we_base_permission::hasPerm([we_base_perm::SHOP_EDIT_ORDER]), $description = "", $type = 0, $width = 255);
		} else {
			$oChbxOrders = $oChbxAverageOrder = $oChbxTarget = '';
		}

		$oDbTableType = $oChbxOrders . $oChbxCustomer . $oChbxAverageOrder . $oChbxTarget;
//$oDbTableType->setCol(0, 3, $revenueTarget);

		$oSctDate = new we_html_select(['name' => "sct_date", 'class' => 'defaultfont', 'onchange' => '']);
		$aLangDate = [
			g_l('cockpit', '[today]'),
			g_l('cockpit', '[this_week]'),
			g_l('cockpit', '[last_week]'),
			g_l('cockpit', '[this_month]'),
			g_l('cockpit', '[last_month]'),
			g_l('cockpit', '[this_year]'),
			g_l('cockpit', '[last_year]')
		];
		foreach($aLangDate as $k => $v){
			$oSctDate->insertOption($k, strval($k), $v);
		}
		$oSctDate->selectOption($iDate);

		$parts = (new we_html_multiIconBoxContent())
			->add((new we_html_multiIconBoxContentLine(g_l('cockpit', '[shop_dashboard][kpi]'), $oDbTableType))
				->space(we_html_multiIconBox::SPACE_MED)
			)
			->add((new we_html_multiIconBoxContentLine(g_l('cockpit', '[shop_dashboard][revenue_target]'), we_html_element::div(['style' => "display:block;"], we_html_tools::formElementTable('', we_html_tools::textInput("revenueTarget", $sRevenueTarget, 255, [
								], "text", '100px') . "&nbsp;&euro;"))))
				->space(we_html_multiIconBox::SPACE_MED)
			)
			->add((new we_html_multiIconBoxContentLine(g_l('cockpit', '[date]'), $oSctDate->getHTML()))
				->space(we_html_multiIconBox::SPACE_MED)
			)
			->add(new we_html_multiIconBoxContentLine(g_l('cockpit', '[display]'), $oSelCls->getHTML()))
		;

		$buttons = we_html_button::position_yes_no_cancel(we_html_button::create_button(we_html_button::SAVE, "javascript:save();"), we_html_button::create_button(we_html_button::PREVIEW, "javascript:preview();"), we_html_button::create_button(we_html_button::CLOSE, "javascript:exit_close();"));

		$sTblWidget = (new we_html_multiIconBox($parts, 'shpProps'))
			->marginLeft()
			->buttons($buttons)
			->headline("Shop")
			->get();

		echo we_html_tools::getHtmlTop(g_l('cockpit', '[shop_dashboard][headline]'), $jsFile .
			we_html_element::jsScript(JS_DIR . 'widgets/shop.js', '', ['id' => 'loadVarWidget', 'data-widget' => setDynamicVar([
					'sInitNum' => $sRevenueTarget,
			])]), we_html_element::body(
				['class' => 'weDialogBody', 'onload' => "init();"
				], we_html_element::form([], $sTblWidget)));
	}

	public function showPreview(): void{
		$jsCmd = new we_base_jsCmd();
		$jsCmd->addCmd('initPreview', [
			'id' => $this->newSCurrId,
			'type' => 'shp',
			'tb' => g_l('cockpit', '[shop_dashboard][headline]') . ':&nbsp;' . $this->interval
		]);
		$jsCmd->addCmd('initGauge', $this->newSCurrId . '_chart_div', $this->jsConfig);
		echo we_html_tools::getHtmlTop(g_l('cockpit', '[shop_dashboard][headline]') . '&nbsp;' . $this->interval, $jsCmd->getCmds(), we_html_element::body([
				'style' => 'margin:10px 15px;',
				], we_html_element::div(['id' => "shp"
					], we_html_element::div(['id' => 'shp_data'], $this->shopDashboard)
		)));
	}

}
