<?php
declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: 0250896a6f12 $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2020-04-15 18:45 +0200 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
class we_workflow_view extends we_modules_view{
	const PAGE_PROPERTIES = 0;
	const PAGE_OVERVIEW = 1;

	/** @var we_workflow_workflow */
	var $Model;

	/** @var we_workflow_document default document */
	var $documentDef;

	/** @var int what is current display 0-workflow(default);1-document */
	var $show = self::PAGE_PROPERTIES;

	/** @var array<string,string> */
	var $hiddens = [];

	/** @var string */
	private $uid;

	function __construct(){
		parent::__construct();
		$this->page = self::PAGE_PROPERTIES;
		$this->uid = 'wf_' . md5(uniqid(__FILE__, true));
		//default workflow
		$this->Model = new we_workflow_workflow();
		$this->documentDef = new we_workflow_document();
		$this->hiddens = array_merge($this->hiddens, ['ID', 'Status']);
	}

	function getHiddens(): string{
		return we_html_element::hiddens(['home' => '0',
				'wcmd' => 'new_workflow',
				'wid' => strval($this->Model->ID),
				'pnt' => 'edit',
				'wname' => strval($this->uid),
				'page' => strval($this->page)
		]);
	}

	function getHiddensFormPropertyPage(): string{
		$this->hiddens = array_merge($this->hiddens, ['Text', 'Type', 'Folders', 'ObjectFileFolders', 'Categories', 'ObjCategories', 'DocType', 'Objects', 'EmailPath',
			'LastStepAutoPublish']);

		return '';
	}

	function getHiddensFormOverviewPage(): string{
		//we need the following vars since fields expect this hidden fields & selectors don't generate a hidden field itself
		$this->hiddens = array_merge($this->hiddens, ['Folders', 'ObjectFileFolders', 'Categories', 'ObjCategories', 'Objects']);

		$out = '';

		$counter = 0;
		$counter1 = 0;
		foreach($this->Model->steps as $sv){
			$out .= we_html_element::hiddens([
					$this->uid . '_step' . $counter . '_sid' => (string) $sv->ID,
					$this->uid . '_step' . $counter . '_and' => (string) $sv->stepCondition,
					$this->uid . '_step' . $counter . '_Worktime' => (string) $sv->Worktime,
					$this->uid . '_step' . $counter . '_timeAction' => (string) $sv->timeAction
			]);
			$counter1 = 0;
			foreach($sv->tasks as $tv){
				$out .= we_html_element::hiddens([
						$this->uid . '_task' . $counter . $counter1 . '_tid' => (string) $tv->ID,
						$this->uid . '_task_' . $counter . '_' . $counter1 . '_userid' => (string) $tv->userID,
						$this->uid . '_task_' . $counter . '_' . $counter1 . '_Edit' => (string) $tv->Edit,
						$this->uid . '_task_' . $counter . '_' . $counter1 . '_Mail' => (string) $tv->Mail
				]);
				++$counter1;
			}
			++$counter;
		}
		$out .= we_html_element::hiddens([
				'wcat' => '0',
				'wocat' => '0',
				'wfolder' => '0',
				'woffolder' => '0',
				'wobject' => '0',
				'wsteps' => strval($counter),
				'wtasks' => strval($counter1)
		]);

		return $out;
	}

	function workflowHiddens(): string{
		$out = '';
		foreach($this->hiddens as $val){
			$dat = isset($this->Model->persistent_slots[$val]) ? $this->Model->$val : $this->{$val};
			$out .= we_html_element::hidden($this->uid . '_' . $val, (is_array($dat) ? implode(',', $dat) : $dat));
		}
		return $out;
	}

	function getProperties(we_base_jsCmd $jsCmd): string{
		$content = '<form name="' . we_base_constants::WE_FORM . '" onsubmit="return false">' .
			$this->getHiddens();
		if($this->show === self::PAGE_OVERVIEW){
			$content .= $this->getDocumentInfo();
		} else {
			switch($this->page){
				case self::PAGE_PROPERTIES:
					$parts = (new we_html_multiIconBoxContent())
						->add(new we_html_multiIconBoxContentLine(g_l('modules_workflow', '[name]'), we_html_tools::textInput($this->uid . '_Text', stripslashes($this->Model->Text), 0, [
									'id' => 'yuiAcInputPathName', 'onchange' => "top.content.setHot();",
									'onblur' => " parent.edheader.weTabs.setTitlePath(this.value);"], "text", '498px'), we_html_multiIconBox::PROP_PATH))
						->add(new we_html_multiIconBoxContentLine(g_l('modules_workflow', '[type]'), $this->getWorkflowTypeHTML($jsCmd), we_html_multiIconBox::PROP_META))
						->add(new we_html_multiIconBoxContentLine(g_l('modules_workflow', '[specials'), '<br/>' .
							we_html_forms::checkboxWithHidden($this->Model->EmailPath, $this->uid . '_EmailPath', g_l('modules_workflow', '[EmailPath]'), false, 'defaultfont', '', false) .
							'<br/>' .
							we_html_forms::checkboxWithHidden($this->Model->LastStepAutoPublish, $this->uid . '_LastStepAutoPublish', g_l('modules_workflow', '[LastStepAutoPublish]'), false, 'defaultfont', '', false), we_html_multiIconBox::PROP_ATTRIB));

					//	Workflow-Type
					$content .= $this->getHiddensFormOverviewPage() .
						(new we_html_multiIconBox($parts, 'workflowProperties'))->marginLeft()->get();
					break;
				case self::PAGE_OVERVIEW:
					$content .= $this->getHiddensFormPropertyPage() .
						we_html_tools::dialogLayout($this->getStepsHTML(), '');
			}
			$content .= $this->workflowHiddens();
		}
		$content .= '</form>';
		return we_html_element::body(['class' => 'weEditorBody', 'onload' => 'loaded=1;', 'onunload' => 'doUnload()'], $content);
	}

	function getWorkflowSelectHTML(): string{
		$vals = we_workflow_workflow::getAllWorkflowsInfo();
		return we_html_tools::select('wid', $vals, 4, $this->Model->ID, false, ['onclick' => "we_cmd(\"we_workflow_edit\")"], true, 200);
	}

	private function getWorkflowTypeHTML(we_base_jsCmd $jsCmd): string{
		return $this->getTypeTableHTML(we_html_forms::radiobuttonV(we_workflow_workflow::FOLDER, $this->Model->Type, $this->uid . '_Type', g_l('modules_workflow', '[type_dir]'), true, 'defaultfont', 'onclick=top.content.setHot();'), [
				$this->getFoldersHTML($jsCmd),
				], 25) .
			$this->getTypeTableHTML(we_html_forms::radiobuttonV(we_workflow_workflow::DOCTYPE_CATEGORY, $this->Model->Type, $this->uid . '_Type', g_l('modules_workflow', '[type_doctype]'), true, 'defaultfont', 'onclick=top.content.setHot();'), [
				$this->getDocTypeHTML(),
				(we_modules_info::isActive(we_modules_info::CATEGORY) ? $this->getCategoryHTML($jsCmd) : ''),
				], 25) .
			(we_modules_info::isActive(we_modules_info::OBJECT) ?
			$this->getTypeTableHTML(we_html_forms::radiobuttonV(we_workflow_workflow::OBJECT, $this->Model->Type, $this->uid . '_Type', g_l('modules_workflow', '[type_object]'), true, 'defaultfont', 'onclick=top.content.setHot();'), [
				$this->getObjectHTML($jsCmd),
				(we_modules_info::isActive(we_modules_info::CATEGORY) ? $this->getObjCategoryHTML($jsCmd) : ''),
				$this->getObjectFileFoldersHTML($jsCmd),
				], 25) :
			'');
	}

	private function getFoldersHTML(we_base_jsCmd $jsCmd): string{
		$delallbut = we_html_button::create_button(we_html_button::DELETE_ALL, "javascript:top.content.setHot();we_cmd('del_all_folders');");
		$addbut = we_html_button::create_button(we_html_button::ADD, "javascript:top.content.setHot();we_cmd('we_selector_directory',{table:WE().consts.tables.FILE_TABLE,jsCmd:'add_folder',multiple:true})");

		$dirs = new we_chooser_multiDir($this->Model->Folders, 'del_folder', $delallbut . $addbut, [], 'ContentType', FILE_TABLE, 'defaultfont', '', "top.content.setHot();");

		return we_html_tools::formElementTable(g_l('modules_workflow', '[dirs]'), $dirs->get($jsCmd));
	}

	private function getCategoryHTML(we_base_jsCmd $jsCmd): string{
		$delallbut = we_html_button::create_button(we_html_button::DELETE_ALL, "javascript:top.content.setHot();we_cmd('del_all_cats')", '', "", (!we_base_permission::hasPerm([
					we_base_perm::WORK_CATEGORY_EDIT])));
		$addbut = we_html_button::create_button(we_html_button::ADD, "javascript:top.content.setHot();we_cmd('we_selector_document',{id:0,table:WE().consts.tables.CATEGORY_TABLE,jsCmd:'add_cat'})", '', "", (!we_base_permission::hasPerm([
					we_base_perm::WORK_CATEGORY_EDIT])));

		$cats = new we_chooser_multiDir($this->Model->Categories, 'del_cat', $delallbut . $addbut, [], '"' . we_base_ContentTypes::CATEGORY . '"', CATEGORY_TABLE, 'defaultfont', '', "top.content.setHot();");

		return we_html_tools::formElementTable(g_l('modules_workflow', '[categories]'), $cats->get($jsCmd));
	}

	private function getObjCategoryHTML(we_base_jsCmd $jsCmd): string{
		$delallbut = we_html_button::create_button(we_html_button::DELETE_ALL, "javascript:top.content.setHot();we_cmd('del_all_objcats')", '', "", (!we_base_permission::hasPerm([
					we_base_perm::WORK_CATEGORY_EDIT])));
		$addbut = we_html_button::create_button(we_html_button::ADD, "javascript:top.content.setHot();we_cmd('we_selector_document',{id:0,table:WE().consts.tables.CATEGORY_TABLE,jsCmd:'add_objcat'})", '', "", (!we_base_permission::hasPerm([
					we_base_perm::WORK_CATEGORY_EDIT])));

		$cats = new we_chooser_multiDir($this->Model->ObjCategories, 'del_objcat', $delallbut . $addbut, [], '"' . we_base_ContentTypes::CATEGORY . '"', CATEGORY_TABLE, 'defaultfont', '', "top.content.setHot();");

		return we_html_tools::formElementTable(g_l('modules_workflow', '[categories]'), $cats->get($jsCmd));
	}

	private function getObjectHTML(we_base_jsCmd $jsCmd): string{
		$delallbut = we_html_button::create_button(we_html_button::DELETE_ALL, "javascript:top.content.setHot();we_cmd('del_all_objects')", '', "", (!we_base_permission::hasPerm([
					we_base_perm::WORK_CATEGORY_EDIT])));
		$addbut = we_html_button::create_button(we_html_button::ADD, "javascript:top.content.setHot();we_cmd('openObjselector','',WE().consts.tables.OBJECT_TABLE,'','','add_object')", '', "", (!we_base_permission::hasPerm([
					we_base_perm::WORK_CATEGORY_EDIT])));

		$cats = new we_chooser_multiDir($this->Model->Objects, 'del_object', $delallbut . $addbut, [], 'ContentType', OBJECT_TABLE, 'defaultfont', '', "top.content.setHot();");

		return we_html_tools::formElementTable(g_l('modules_workflow', '[classes]'), $cats->get($jsCmd));
	}

	private function getObjectFileFoldersHTML(we_base_jsCmd $jsCmd): string{
		$delallbut = we_html_button::create_button(we_html_button::DELETE_ALL, "javascript:top.content.setHot();we_cmd('del_all_object_file_folders');");
		$addbut = we_html_button::create_button(we_html_button::ADD, "javascript:top.content.setHot();we_cmd('we_selector_directory',{table:WE().consts.tables.OBJECT_FILES_TABLE,jsCmd:'add_object_file_folder',multiple:true});");

		$dirs = new we_chooser_multiDir($this->Model->ObjectFileFolders, 'del_object_file_folder', $delallbut . $addbut, [], 'ContentType', OBJECT_FILES_TABLE, 'defaultfont', '', "top.content.setHot();");

		return we_html_tools::formElementTable(g_l('modules_workflow', '[dirs]'), $dirs->get($jsCmd));
	}

	function getStatusHTML(): string{
		return we_html_forms::checkboxWithHidden(true, 'status_workflow', g_l('modules_workflow', '[active]'), false, 'defaultfont', 'top.content.setHot();');
	}

	function getStepsHTML(): string{
		$content = [];

		$ids = '';

		$headline = [g_l('modules_workflow', '[step]'),
			g_l('modules_workflow', '[and_or]'),
			g_l('modules_workflow', '[worktime]')
		];

		$counter = 0;
		$counter1 = 0;

		$weSuggest = we_gui_suggest::getInstance();

		/*		 * *** WORKFLOWSTEPS **** */
		foreach($this->Model->steps as $sv){
			$ids .= we_html_element::hidden($this->uid . '_step' . $counter . '_sid', $sv->ID);
			$content[$counter] = [
				(string) ($counter + 1),
				'<table><tr style="vertical-align:top"><td>' . we_html_forms::radiobutton('1', $sv->stepCondition ? true : false, $this->uid . '_step' . $counter . '_and', '', false, 'defaultfont', "top.content.setHot();") . '</td><td style="padding-left:5px;">' . we_html_forms::radiobutton('0', $sv->stepCondition ? true : false, $this->uid . '_step' . $counter . '_and', '', false, 'defaultfont', "top.content.setHot();") . '</td></tr></table>',
				'<table class="default" style="margin-top:5px;">
	<tr style="vertical-align:middle"><td class="middlefont" style="padding-bottom:1em;">' . we_html_tools::textInput($this->uid . '_step' . $counter . '_Worktime', (string) $sv->Worktime, 0, [
					'min' => '0', 'step' => '0.25', 'onchange' => "top.content.setHot();"], 'number') . '</td></tr>
	<tr style="vertical-align:top"><td class="middlefont">' . we_html_forms::checkboxWithHidden($sv->timeAction == 1, $this->uid . '_step' . $counter . '_timeAction', g_l('modules_workflow', '[go_next]'), false, 'middlefont', "top.content.setHot();") . '</td></tr>
</table>',
			];


			$counter1 = 0;
			foreach($sv->tasks as $tv){
				$ids .= we_html_element::hidden($this->uid . '_task' . $counter . '_' . $counter1 . '_tid', $tv->ID);
				$headline[$counter1 + 3] = g_l('modules_workflow', '[user]') . (string) ($counter1 + 1);

				$foo = fS('SELECT Path FROM ' . USER_TABLE . ' WHERE ID=' . intval($tv->userID));

				$weSuggest->setAcId('User_' . $counter . '_' . $counter1)
					->setContentType([we_users_user::TYPE_USER, we_users_user::TYPE_USER_GROUP])
					->setInput($this->uid . '_task_' . $counter . '_' . $counter1 . '_usertext', $foo, [], false, true)
					->setMaxResults(10)
					->setRequired(true)
					->setResult($this->uid . '_task_' . $counter . '_' . $counter1 . '_userid', strval($tv->userID))
					->setSelector(we_gui_suggest::DocSelector)
					->setTable(USER_TABLE)
					->setWidth(200)
					->setSelectButton(we_html_button::create_button(we_html_button::SELECT, "javascript:top.content.setHot();we_cmd('we_users_selector',{id:document.we_form." . $this->uid . '_task_' . $counter . '_' . $counter1 . "_userid.value, targetName:'" . $this->uid . "_task_" . $counter . "_" . $counter1 . "_usertext',targetID:'" . $this->uid . "_task_" . $counter . "_" . $counter1 . "_userid'});"));

				$content[$counter][$counter1 + 3] = ['dat' => '<table class="default" style="margin-top:1ex;">
	<tr style="vertical-align:middle"><td colspan="2">' . $weSuggest->getHTML() . '</td></tr>
						<tr style="vertical-align:top">
		<td class="middlefont" style="text-align:right">' . we_html_forms::checkboxWithHidden($tv->Mail, $this->uid . '_task_' . $counter . '_' . $counter1 . "_Mail", g_l('modules_workflow', '[send_mail]'), false, 'middlefont', "top.content.setHot();") . '</td>
		<td class="middlefont" style="padding-left:20px;">' . we_html_forms::checkboxWithHidden($tv->Edit, $this->uid . '_task_' . $counter . '_' . $counter1 . "_Edit", g_l('modules_workflow', '[edit]'), false, 'middlefont', "top.content.setHot();") . '</td>
						</tr></table>',
				];
				$counter1++;
			}
			++$counter;
		}
		return $ids . '
<table style="margin-right:30px;">
	<tr style="vertical-align:top">
		<td>' . we_html_tools::dialogBorder(400, $content, $headline) . '</td>
		<td>' . we_html_button::create_button(we_html_button::PLUS, "javascript:top.content.setHot();addTask()") . we_html_button::create_button(we_html_button::TRASH, "javascript:top.content.setHot();delTask()") . '</td>
	</tr>
	<tr style="vertical-align:top">
		<td colspan="2">' . we_html_button::create_button(we_html_button::PLUS, "javascript:top.content.setHot();addStep()") . we_html_button::create_button(we_html_button::TRASH, "javascript:top.content.setHot();delStep()") . '</td></tr>
</table>' .
			we_html_element::hiddens(['wsteps' => strval($counter),
				'wtasks' => strval($counter1)
		]);
	}

	function getTypeTableHTML(string $head, array $values, int $ident = 0, string $textalign = 'left', string $textclass = 'defaultfont'): string{
		$out = '<table class="default">' . ($head ? '<tr><td class="' . trim($textclass) . '" style="text-align:' . trim($textalign) . '" colspan="2">' . $head . '</td></tr>' : '');
		foreach($values as $val){
			$out .= '<tr><td class="' . trim($textclass) . '" style="padding-left:' . $ident . 'px;">' . $val . '</td></tr>';
		}
		$out .= '</table>';
		return $out;
	}

	private function getDocTypeHTML(): string{
		$dtq = we_doctype_docTypes::getDoctypeQuery();
		$vals = $this->db->getAllFirstq('SELECT dt.ID,dt.Text FROM ' . DOC_TYPES_TABLE . ' dt LEFT JOIN ' . FILE_TABLE . ' dtf ON dt.ParentFileID=dtf.ID ' . $dtq['join'] . ' WHERE ' . $dtq['where'], false);
		return we_html_tools::formElementTable(g_l('modules_workflow', '[doctype]'), we_html_tools::select($this->uid . '_DocType[]', $vals, 1, $this->Model->DocType, true, [
					'class' => 'searchSelect',
					'onchange' => "top.content.setHot();"], true, 498));
	}

	function getJSTop(): string{
		$modData = we_modules_info::getModuleField(we_modules_info::WORKFLOW, 'text');
		$title = ($modData ? 'webEdition ' . g_l('global', '[modules]') . ' - ' . $modData : '');
		return we_html_element::jsScript(WE_JS_MODULES_DIR . 'workflow/workflow_top.js', "parent.document.title='" . $title . "';");
	}

	function getJSProperty(array $jsVars = []): string{
		if($this->show === self::PAGE_PROPERTIES){
			$this->Model->loadDocuments();
		}
		return we_html_element::jsScript(WE_JS_MODULES_DIR . 'workflow/workflow_property.js', '', ['id' => 'loadVarWorkflowProp', 'data-workflow' => setDynamicVar(array_merge($jsVars, [
					'uid' => $this->uid,
					'noDocs' => ($this->show === self::PAGE_OVERVIEW ? 0 : count($this->Model->documents))
		]))]);
	}

	public function processCommands(we_base_jsCmd $jscmd): void{
		switch(we_base_request::string('wcmd', '')){
			case 'new_workflow':
				$this->Model = new we_workflow_workflow();
				$this->page = self::PAGE_PROPERTIES;

				$jscmd->addCmd('loadHeadFooter', ['page' => self::PAGE_PROPERTIES, 'txt' => '']);
				return;
			case 'add_cat':
				$arr = $this->Model->Categories;
				if(($ids = we_base_request::intList('wcat', []))){
					$this->Model->Categories = array_unique(array_filter(array_merge($arr, $ids)), SORT_NUMERIC);
				}
				return;
			case 'del_cat':
				$arr = $this->Model->Categories;
				if(($cat = we_base_request::int('wcat'))){
					if(($pos = array_search($cat, $arr, false)) === false){
						return;
					}
					unset($arr[$pos]);
					$this->Model->Categories = $arr;
				}
				return;
			case 'del_all_cats':
				$this->Model->Categories = [];
				return;
			case 'add_objcat':
				$arr = $this->Model->ObjCategories;
				if(($ids = we_base_request::intList('wocat', []))){
					$this->Model->ObjCategories = array_unique(array_filter(array_merge($arr, $ids)), SORT_NUMERIC);
				}
				return;
			case 'del_objcat':
				$arr = $this->Model->ObjCategories;
				if(($cat = we_base_request::int('wcat'))){
					if(($pos = array_search($cat, $arr, false)) === false){
						return;
					}
					unset($arr[$pos]);
					$this->Model->ObjCategories = $arr;
				}
				return;
			case 'del_all_objcats':
				$this->Model->ObjCategories = [];
				return;
			case 'add_folder':
				$arr = $this->Model->Folders;
				if(($ids = we_base_request::intList('wfolder', we_base_request::NOT_VALID_AI)) !== we_base_request::NOT_VALID_AI){
					$this->Model->Folders = array_unique(array_filter(array_merge($arr, $ids)), SORT_NUMERIC);
					;
				}
				return;
			case 'del_folder':
				$arr = $this->Model->Folders;
				if(($id = we_base_request::int('wfolder', -1)) > -1){
					if(($pos = array_search($id, $arr, false)) === false){
						return;
					}
					unset($arr[$pos]);
					$this->Model->Folders = $arr;
				}
				return;
			case 'del_all_folders':
				$this->Model->Folders = [];
				return;
			case 'add_object_file_folder':
				$arr = $this->Model->ObjectFileFolders;
				if(($ids = we_base_request::intList('woffolder', we_base_request::NOT_VALID_AI)) !== we_base_request::NOT_VALID_AI){
					$this->Model->ObjectFileFolders = array_unique(array_filter(array_merge($arr, $ids)), SORT_NUMERIC);
				}
				return;
			case 'del_object_file_folder':
				$arr = $this->Model->ObjectFileFolders;
				if(($id = we_base_request::int('woffolder', -1)) > -1){
					if(($pos = array_search($id, $arr, false)) === false){
						return;
					}
					unset($arr[$pos]);
					$this->Model->ObjectFileFolders = $arr;
				}
				return;
			case 'del_all_object_file_folders':
				$this->Model->ObjectFileFolders = [];
				return;
			case 'add_object':
				$arr = $this->Model->Objects;
				if(($id = we_base_request::int('wobject'))){
					$arr[] = $id;
					$this->Model->Objects = $arr;
				}
				return;
			case 'del_object':
				$arr = $this->Model->Objects;
				if(($id = we_base_request::int('wobject'))){
					if(($pos = array_search($id, $arr, false)) === false){
						break;
					}
					unset($arr[$pos]);
					$this->Model->Objects = $arr;
				}
				return;
			case 'del_all_objects':
				$this->Model->Objects = [];
				return;
			case 'reload':
				$jscmd->addCmd('loadHeadFooter', ['page' => $this->page, 'txt' => $this->Model->Text]);
				return;
			case 'we_workflow_edit':
				$this->show = self::PAGE_PROPERTIES;
				if(($id = we_base_request::int('wid'))){
					$this->Model = new we_workflow_workflow($id);
				}

				$jscmd->addCmd('loadHeadFooter', ['page' => $this->page, 'txt' => $this->Model->Text]);
				return;
			case 'switchPage':
				if(($page = we_base_request::int('page', -1)) > -1){
					$this->page = $page;
				}
				return;
			case 'save_workflow':
				if(we_base_request::int('wid', -1) > -1){
					$newone = (!$this->Model->ID);

					if(!we_base_permission::hasPerm([we_base_perm::WORKFLOW_EDIT, we_base_perm::WORKFLOW_NEW])){
						$jscmd->addMsg(g_l('modules_workflow', '[no_perms]'), we_base_util::WE_MESSAGE_ERROR);
						return;
					}
					if($newone && !we_base_permission::hasPerm([we_base_perm::WORKFLOW_NEW])){
						$jscmd->addMsg(g_l('modules_workflow', '[no_perms]'), we_base_util::WE_MESSAGE_ERROR);
						return;
					}
					//duplicate
					if(fB('SELECT 1 FROM ' . WORKFLOW_TABLE . ' WHERE Text="' . $this->db->escape($this->Model->Text) . '"' . ($newone ? '' : ' AND ID!=' . intval($this->Model->ID)) . ' LIMIT 1')){
						$jscmd->addMsg(g_l('modules_workflow', '[double_name]'), we_base_util::WE_MESSAGE_ERROR);
						return;
					}
					$this->Model->loadDocuments();
					foreach($this->Model->documents as $v){
						$jscmd->addCmd('deleteTreeEntry', [$v['ID'], 'file']);
					}
					if(($dts = we_base_request::intList($this->uid . '_DocType', we_base_request::NOT_VALID_AI)) !== we_base_request::NOT_VALID_AI){
						$this->Model->DocType = $dts;
					}

					$this->Model->save();
					if($newone){
						$jscmd->addCmd('makeTreeEntry', [
							'id' => $this->Model->ID,
							'parentid' => 0,
							'text' => $this->Model->Text,
							'open' => true,
							'contenttype' => we_base_ContentTypes::FOLDER_COMMON,
							'table' => "we_workflow_workflowDef",
						]);
					} else {
						$jscmd->addCmd('updateTreeEntry', [
							'id' => $this->Model->ID,
							'text' => $this->Model->Text,
						]);
					}
					$jscmd->addCmd('setHeadRow', we_html_element::b(g_l('modules_workflow', '[workflow]') . ': ' . htmlspecialchars($this->Model->Text)));
					$jscmd->addMsg(g_l('modules_workflow', '[save_ok]'), we_base_util::WE_MESSAGE_NOTICE);
				}
				return;
			case 'show_document':
				if(($id = we_base_request::int('wid'))){
					$this->show = self::PAGE_OVERVIEW;
					$this->page = self::PAGE_PROPERTIES;
					$this->documentDef->load($id);
					$jscmd->addCmd('loadHeadFooter', ['page' => self::PAGE_PROPERTIES, 'txt' => $this->documentDef->document->getText(), 'art' => 1]);
				}
				return;
			case 'delete_workflow':
				if(($id = we_base_request::int('wid'))){
					if(!we_base_permission::hasPerm([we_base_perm::WORKFLOW_DELETE])){
						$jscmd->addMsg(g_l('modules_workflow', '[no_perms]'), we_base_util::WE_MESSAGE_ERROR);
						return;
					}

					$this->Model = new we_workflow_workflow($id);
					if($this->Model->delete()){
						$this->Model = new we_workflow_workflow();
						$jscmd->addCmd('deleteTreeEntry', [$id, we_base_ContentTypes::FOLDER_COMMON]);
						$jscmd->addMsg(g_l('modules_workflow', '[delete_ok]'), we_base_util::WE_MESSAGE_NOTICE);
					} else {
						$jscmd->addMsg(g_l('modules_workflow', '[delete_nok]'), we_base_util::WE_MESSAGE_ERROR);
					}
				}
				return;
			case 'reload_table':
				$this->page = self::PAGE_OVERVIEW;
				return;
			case 'empty_log':
				$stamp = we_base_request::STRING('wopt');
				we_workflow_log::clearLog($stamp ? new DateTime($stamp) : null);
				$jscmd->addMsg(g_l('modules_workflow', '[empty_log_ok]'), we_base_util::WE_MESSAGE_NOTICE);
				return;
			default:
		}
	}

	public function processVariables(): void{
		$this->uid = we_base_request::string('wname', $this->uid);
		foreach($this->Model->persistent_slots as $val => $type){
			$varname = $this->uid . '_' . $val;
			if(($tmp = we_base_request::_($type, $varname, '_noval_')) !== '_noval_'){
				$this->Model->$val = $tmp;
			}
		}

		$wsteps = we_base_request::int('wsteps', 0);
		$wtasks = we_base_request::int('wtasks', 0);
		$this->page = we_base_request::int('page', self::PAGE_PROPERTIES);


		$this->Model->steps = [];
		if($wsteps == 0){
			$this->Model->addNewStep();
			$this->Model->addNewTask();
		}

		for($i = 0; $i < $wsteps; $i++){
			$this->Model->addNewStep();
		}
		for($j = 0; $j < $wtasks; $j++){
			$this->Model->addNewTask();
		}

		foreach($this->Model->steps as $skey => $step){
			assert($step instanceof we_workflow_step);
			$stepname = $this->uid . '_step' . $skey . '_';
			$step->update(
				$this->Model->ID,
		we_base_request::bool($stepname . 'and', $step->stepCondition),
												we_base_request::float($stepname . 'Worktime', $step->Worktime),
																	 we_base_request::bool($stepname . 'timeAction', $step->timeAction),
																				we_base_request::int($stepname . 'sid'));

			foreach($step->tasks as $tkey => $task){
				assert($task instanceof we_workflow_task);
				$taskName = $stepname . $tkey . '_';
				$task->update(
					we_base_request::int($taskName . 'userid', $task->userID),
													we_base_request::string($taskName . 'usertext', $task->username), //FIXME: this is a path to a user
																		 $step->ID,
																		 we_base_request::bool($taskName . 'Edit', $task->Edit),
																				 we_base_request::bool($taskName . 'Mail', $task->Mail),
																					 we_base_request::int($taskName . 'tid')
				);
			}
		}
	}

	private function getDocumentInfo(): string{
		if($this->documentDef->workflow->Type === we_workflow_workflow::OBJECT){
			return $this->getObjectInfo();
		}

		//	Part - file-information
		$parts = (new we_html_multiIconBoxContent())
			->add((new we_html_multiIconBoxContentLine(g_l('weEditorInfo', '[content_type]'), g_l('weEditorInfo', '[' . $this->documentDef->document->ContentType . ']')))->space(we_html_multiIconBox::SPACE_MED)->noline(((!in_array($this->documentDef->document->ContentType, we_base_ContentTypes::FOLDER_TYPES) && $this->documentDef->workflow->Type != we_workflow_workflow::OBJECT))));
		if(!in_array($this->documentDef->document->ContentType, we_base_ContentTypes::FOLDER_TYPES) && $this->documentDef->workflow->Type != we_workflow_workflow::OBJECT){
			//we_base_WE::setWeDoc($this->documentDef->document);
			$fs = $this->documentDef->document->getFilesize();
			$parts->add((new we_html_multiIconBoxContentLine(g_l('weEditorInfo', '[file_size]'), we_base_file::getHumanFileSize($fs) . '&nbsp;KB&nbsp;(' . we_base_file::getHumanFileSize($fs, we_base_file::SZ_BYTE) . ')'))->space(we_html_multiIconBox::SPACE_MED));
		}

		//	Part - publish-information
		$parts->add((new we_html_multiIconBoxContentLine(g_l('weEditorInfo', '[creation_date]'), $this->documentDef->document->Creation_Date->format(g_l('weEditorInfo', '[date_format]'))))->space(we_html_multiIconBox::SPACE_MED)->noline());

		if($this->documentDef->document->CreatorID){
			$creator = fS('SELECT CONCAT(First," ",Second," (",username,")") FROM ' . USER_TABLE . ' WHERE ID=' . $this->documentDef->document->CreatorID);
			if($creator){
				$parts->add((new we_html_multiIconBoxContentLine(g_l('modules_users', '[created_by]'), $creator))->space(we_html_multiIconBox::SPACE_MED)->noline());
			}
		}

		$parts->add((new we_html_multiIconBoxContentLine(g_l('weEditorInfo', '[changed_date]'), $this->documentDef->document->Mod_Date->format(g_l('weEditorInfo', '[date_format]'))))->space(we_html_multiIconBox::SPACE_MED)->noline());

		if($this->documentDef->document->ModifierID){
			$modifier = fS('SELECT CONCAT(First," ",Second," (",username,")") FROM ' . USER_TABLE . ' WHERE ID=' . $this->documentDef->document->ModifierID);
			if($modifier){
				$parts->add((new we_html_multiIconBoxContentLine(g_l('modules_users', '[changed_by]'), $modifier))->space(we_html_multiIconBox::SPACE_MED)->noline());
			}
		}

		if($this->documentDef->document->ContentType == we_base_ContentTypes::HTML || $this->documentDef->document->ContentType == we_base_ContentTypes::WEDOCUMENT){
			$parts->add((new we_html_multiIconBoxContentLine(g_l('weEditorInfo', '[lastLive]'), ($this->documentDef->document->Publish_Date ? $this->documentDef->document->Publish_Date->format(g_l('weEditorInfo', '[date_format]')) : '-')))->space(we_html_multiIconBox::SPACE_MED));
		}

		//	Part - Path-information

		if($this->documentDef->document->Table !== TEMPLATES_TABLE && $this->documentDef->workflow->Type !== we_workflow_workflow::OBJECT){
			$rp = realpath($this->documentDef->document->getRealPath());
			$http = $this->documentDef->document->getHttpPath();

			switch($this->documentDef->document->ContentType){
				default:
					$showlink = false;
					break;
				case we_base_ContentTypes::HTML:
				case we_base_ContentTypes::WEDOCUMENT:
				case we_base_ContentTypes::IMAGE:
				case we_base_ContentTypes::FLASH:
				case we_base_ContentTypes::VIDEO:
				case we_base_ContentTypes::AUDIO:
					$showlink = true;
			}

			$parts
				->add((new we_html_multiIconBoxContentLine(g_l('weEditorInfo', '[local_path]'), '<a href="#" style="text-decoration:none;cursor:text" class="defaultfont" title="' . $rp . '" onclick="WE().layout.openToEdit(\'' . $this->documentDef->document->Table . '\',\'' . $this->documentDef->document->ID . '\',\'' . $this->documentDef->document->ContentType . '\')" >' . ($rp ? we_base_util::shortenPath($rp, 74) : '') . '</a>'))->space(we_html_multiIconBox::SPACE_MED)->noline())
				->add((new we_html_multiIconBoxContentLine(g_l('weEditorInfo', '[http_path]'), ($showlink ? '<a href="' . $http . '" target="_blank" title="' . $http . '">' : '') . we_base_util::shortenPath($http, 74) . ($showlink ? '</a>' : '')))->space(we_html_multiIconBox::SPACE_MED))
				->add((new we_html_multiIconBoxContentLine('', '<a href="#" onclick="WE().layout.openToEdit(\'' . $this->documentDef->document->Table . '\',\'' . $this->documentDef->document->ID . '\',\'' . $this->documentDef->document->ContentType . '\')" >' . g_l('weEditorInfo', '[openDocument]') . '</a>'))->space(we_html_multiIconBox::SPACE_MED));
		}

		//	Logbook
		$parts->add(new we_html_multiIconBoxContentLine('', self::getDocumentStatus($this->documentDef->ID)));

		return we_html_element::jsScript(JS_DIR . 'tooltip.js') .
			(new we_html_multiIconBox($parts))->marginLeft()->get();
	}

	private function getObjectInfo(): string{
		//	Dokument properties
		$parts = (new we_html_multiIconBoxContent())
			->add((new we_html_multiIconBoxContentLine('ID', (string) $this->documentDef->document->ID))->space(we_html_multiIconBox::SPACE_MED2)->noline())
			->add((new we_html_multiIconBoxContentLine(g_l('weEditorInfo', '[content_type]'), g_l('weEditorInfo', '[' . $this->documentDef->document->ContentType . ']')))->space(we_html_multiIconBox::SPACE_MED2))
			// publish information
			->add((new we_html_multiIconBoxContentLine(g_l('weEditorInfo', '[creation_date]'), $this->documentDef->document->Creation_Date->format(g_l('weEditorInfo', '[date_format]'))))->space(we_html_multiIconBox::SPACE_MED2)->noline())
			->add(($user = fS('SELECT CONCAT(First," ",Second," (",username,")") FROM ' . USER_TABLE . ' WHERE ID=' . $this->documentDef->document->CreatorID)) ?
				(new we_html_multiIconBoxContentLine(g_l('modules_users', '[created_by]'), $user))
				->space(we_html_multiIconBox::SPACE_MED2)
				->noline() : null)
			->add((new we_html_multiIconBoxContentLine(g_l('weEditorInfo', '[changed_date]'), $this->documentDef->document->Mod_Date->format(g_l('weEditorInfo', '[date_format]'))))->space(we_html_multiIconBox::SPACE_MED2)->noline())
			->add(($user = fS('SELECT CONCAT(First," ",Second," (",username,")") FROM ' . USER_TABLE . ' WHERE ID=' . $this->documentDef->document->ModifierID)) ?
				(new we_html_multiIconBoxContentLine(g_l('modules_users', '[changed_by]'), $user))
				->space(we_html_multiIconBox::SPACE_MED2)
				->noline() : null)
			->add((new we_html_multiIconBoxContentLine(g_l('weEditorInfo', '[lastLive]'), ($this->documentDef->document->Publish_Date ? $this->documentDef->document->Publish_Date->format(g_l('weEditorInfo', '[date_format]')) : '-')))->space(we_html_multiIconBox::SPACE_MED2))
			->add((new we_html_multiIconBoxContentLine('', '<a href="#" onclick="WE().layout.openToEdit(\'' . $this->documentDef->document->Table . '\',\'' . $this->documentDef->document->ID . '\',\'' . $this->documentDef->document->ContentType . '\')" >' . g_l('weEditorInfo', '[openDocument]') . '</a>'))->space(we_html_multiIconBox::SPACE_MED2))
			->add((new we_html_multiIconBoxContentLine('', self::getDocumentStatus($this->documentDef->ID))));

		return we_editor_script::get($this->documentDef->document) .
			we_html_element::jsScript(JS_DIR . 'global.js', 'initWE();') .
			'</head>
		<body class="weEditorBody" onunload="doUnload()">
				<form name="' . we_base_constants::WE_FORM . '">' . we_contents_base::hiddenTrans() . '<table>' .
			(new we_html_multiIconBox($parts))->marginLeft()->get() .
			'</form></body></html>';
	}

	public static function getDocumentStatus(int $workflowDocID): string{
		$headline = [g_l('modules_workflow', '[step]')];

		$workflowDocument = new we_workflow_document($workflowDocID);

		$counter = 0;
		$content = [];
		try{
			$current = $workflowDocument->findLastActiveStep();
		} catch (we_exception_base $e){
			return g_l('modules_workflow', '[cannot_find_active_step]');
		}
		foreach($workflowDocument->steps as $sk => $sv){

			$workflowStep = new we_workflow_step($sv->workflowStepID);
			$timeStatus = $sv->getTimeStatus();

			if($timeStatus['remain']->format('%a%h') < 0){
				if($sk > $current){
					$finished_font = 'middlefont';
					$notfinished_font = 'middlefont lowContrast';
				} else {
					$finished_font = 'middlefont highlightElementChanged';
					$notfinished_font = 'middlefont highlightElementChanged';
				}
			} else {
				$finished_font = 'middlefont';
				$notfinished_font = 'middlefont lowContrast';
			}


			$content[$counter] = [
				($sv->Status == we_workflow_documentStep::STATUS_NEW ? '<div class="' . $notfinished_font . '">' : '<div class="' . $finished_font . '">') . ($counter + 1) . "</div>",
			];

			$counter1 = 0;
			foreach($sv->tasks as $tv){

				$headline[++$counter1] = g_l('modules_workflow', '[user]') . (string) ($counter1);

				$workflowTask = new we_workflow_task($tv->workflowTaskID);

				$foo = fS('SELECT username FROM ' . USER_TABLE . ' WHERE ID=' . intval($workflowTask->userID));

				if($sk == $current){
					$out = ($tv->Status == we_workflow_documentTask::STATUS_NEW ? '<div class="' . $notfinished_font . '">' : '<div class="' . $finished_font . '">') . $foo . "</div>";
				} else if($sk < $current){
					$out = '<div class="' . $finished_font . '">' . $foo . '</div>';
				} else {
					$out = '<div class="' . $notfinished_font . '">' . $foo . '</div>';
				}

				$content[$counter][$counter1] = $out;
			}


			$headline[++$counter1] = g_l('modules_workflow', '[worktime]');

			$content[$counter][$counter1] = ($sv->Status === we_workflow_documentStep::STATUS_NEW ? '<div class="' . $notfinished_font . '">' : '<div class="' . $finished_font . '">') . $workflowStep->Worktime . '</div>';

			if($sk <= $current){
				$headline[++$counter1] = g_l('modules_workflow', '[time_elapsed]');


				$content[$counter][$counter1] = ($sv->Status === we_workflow_documentStep::STATUS_NEW ? '<div class="' . $notfinished_font . '">' : '<div class="' . $finished_font . '">') . $timeStatus['elapsed']->format('%d %H:%I:%S') . "</div>";

				$headline[++$counter1] = g_l('modules_workflow', '[time_remained]');

				$content[$counter][$counter1] = ($sv->Status === we_workflow_documentStep::STATUS_NEW ? '<div class="' . $notfinished_font . '">' : '<div class="' . $finished_font . '">') . $timeStatus['remain']->format('%d %H:%I:%S') . "</div>";

				$headline[++$counter1] = g_l('modules_workflow', '[step_plan]');

				$content[$counter][$counter1] = ($sv->Status === we_workflow_documentStep::STATUS_NEW ? '<div class="' . $notfinished_font . '">' : '<div class="' . $finished_font . '">') . $timeStatus['end']->format(g_l('weEditorInfo', '[mysql_date_format]')) . "</div>";
			}
			++$counter;
		}

		$wfType = fS('SELECT ' . WORKFLOW_TABLE . '.Type FROM ' . WORKFLOW_TABLE . ',' . WORKFLOW_DOC_TABLE . ' WHERE ' . WORKFLOW_DOC_TABLE . '.workflowID=' . WORKFLOW_TABLE . '.ID AND ' . WORKFLOW_DOC_TABLE . '.ID=' . intval($workflowDocument->ID));
		return '<table class="default" style="margin-right:15px;">
		<tr><td>' . we_html_tools::dialogBorder(730, $content, $headline) . '</td></tr>
		<tr><td style="padding-top:10px;">' . we_html_button::create_button(we_html_button::LOG_SHOW, "javascript:(new (WE().util.jsOverlay)(window,'top', '" . WEBEDITION_DIR . 'we_cmd.php?we_cmd[0]=we_showMod&mod=wrokflow&pnt=log&art=' . $workflowDocument->document->ID . "&type=" . $wfType . "','workflow_history',WE().consts.size.dialog.medium,WE().consts.size.dialog.small)).open();") . '</td></tr>		</table>';
	}

	static function getLogForDocument(int $docID, string $type = we_workflow_workflow::DOCTYPE_CATEGORY): string{//type is an string-array
		$content = [];

		$headlines = [
			g_l('modules_workflow', '[action]'),
			g_l('modules_workflow', '[description]'),
			g_l('modules_workflow', '[time]'),
			g_l('modules_workflow', '[user]'),
		];

		$logs = we_workflow_log::getLogForDocument($docID, 'DESC', $type);
		$offset = we_base_request::int('offset');
		$art = we_base_request::int('art');
		$numRows = we_workflow_log::NUMBER_LOGS;
		$anz = we_workflow_log::$count;

		foreach($logs as $v){
			$content[] = [
				'<div class="middlefont">' . $v['Event'] . '</div>',
				'<div class="middlefont">' . $v['Description'] . '</div>',
				'<div class="middlefont">' . $v['Datum'] . '</div>',
				'<div class="middlefont">' . ((!empty($foo['First'])) ? $foo['First'] : '-') . ' ' . ((!empty($foo['Second'])) ? $foo['Second'] : '-') . ((!empty($foo['username'])) ? ' (' . $foo['username'] . ')' : '') . '</div>',
			];
		}

		$nextprev = '<table class="default"><tr><td>' .
			($offset ?
			we_html_button::create_button(we_html_button::BACK, WEBEDITION_DIR . 'we_cmd.php?we_cmd[0]=we_showMod&mod=workflow&pnt=log&art=' . $art . '&type=' . $type . '&offset=' . ($offset - $numRows)) :
			we_html_button::create_button(we_html_button::BACK, '', '', '', true)
			) .
			'</td><td class="defaultfont" style="padding: 0 10px 0 10px;"><b>' . (($anz) ? $offset + 1 : 0) . '-' .
			(($anz - $offset) < $numRows ? $anz : $offset + $numRows) . ' ' . g_l('global', '[from]') . ' ' . $anz . '</b></td><td>' .
			((($offset + $numRows) < $anz) ?
			we_html_button::create_button(we_html_button::NEXT, WEBEDITION_DIR . 'we_cmd.php?we_cmd[0]=we_showMod&mod=workflow&pnt=log&art=' . $art . '&type=' . $type . '&offset=' . ($offset + $numRows)/* . "&order=$order" */) :
			we_html_button::create_button(we_html_button::NEXT, '', '', '', true)
			) .
			'</td><td></tr></table>';

		$buttonsTable = '<table style="width:100%" class="default"><tr><td>' . $nextprev . '</td><td style="text-align:right">' . we_html_button::create_button(we_html_button::CLOSE, "javascript:WE().util.dialogTop(window).doClose();") . '</td></tr></table>';


		return ($logs ?
			we_html_tools::dialogLayout(we_html_tools::dialogBorder(580, $content, $headlines), '', $buttonsTable) :
			we_html_tools::dialogLayout('<div style="width:500px;text-align:center" class="middlefont">-- ' . g_l('modules_workflow', '[log_is_empty]') . ' --</div>', '', we_html_button::create_button(we_html_button::CLOSE, "javascript:WE().util.dialogTop(window).doClose();")));
	}

	function getLogQuestion(): string{
		$vals = ['<table class="default" style="margin-left:22px;"><tr><td>' .
			we_html_tools::getNewDateInput('log_time[%s]', new DateTime((date('Y-m-d H:i:s', (time() - (336 * 3600))))), we_html_tools::FORMAT_DATE) . '</td></tr></table>'];

		return we_html_tools::dialogLayout(
				we_html_element::hidden('clear_opt', 1) .
				'<form name="' . we_base_constants::WE_FORM . '">' .
				'<table class="default">' .
				'<tr><td class="defaultfont" style="padding-bottom:10px;">' . g_l('modules_workflow', '[log_question_text]') . '</td></tr>' .
				'<tr><td>' . $this->getTypeTableHTML(we_html_forms::radiobutton('1', true, 'clear_time', g_l('modules_workflow', '[log_question_time]'), true, 'defaultfont', "javascript:document.we_form.clear_opt.value=1;"), $vals) . '</td></tr>' .
				'<tr><td style="padding-top:1em;">' . we_html_forms::radiobutton('0', false, 'clear_time', g_l('modules_workflow', '[log_question_all]'), true, 'defaultfont', "javascript:document.we_form.clear_opt.value=0;") . '</td></tr>' .
				'</table>'
				, g_l('modules_workflow', '[empty_log]'), we_html_button::position_yes_no_cancel(we_html_button::create_button(we_html_button::OK, 'javascript:self.clearLog();'), '', we_html_button::create_button(we_html_button::CANCEL, 'javascript:WE().util.dialogTop(window).doClose();')
				)
			) . '</form>';
	}

	public static function getFooter(string $mode, we_contents_root $we_doc, bool $showPubl): string{
		switch($mode){
			default:
			case we_base_constants::MODE_NORMAL:
				return self::getFooterForNormalMode($we_doc, $showPubl);
			case we_base_constants::MODE_SEE:
				return self::getFooterForSEEMMode($we_doc, $showPubl);
		}
	}

	private static function getFooterForNormalMode(we_contents_root $we_doc, bool $showPubl): string{
		$col = 0;

		$table = new we_html_table(['class' => 'default'], 1, 0);

		$publishbutton = '';
		//	decline
		$table->addCol(2)
			->setCol(0, $col++, we_html_button::create_button(we_html_button::DECLINE, "javascript:decline_workflow();"));

		if(we_workflow_utility::isWorkflowFinished($we_doc->ID, $we_doc->Table) || ((1 + we_workflow_utility::findLastActiveStep($we_doc->ID, $we_doc->Table)) == we_workflow_utility::getNumberOfSteps($we_doc->ID, $we_doc->Table) && we_base_permission::hasPerm([
				we_base_perm::WORK_CAN_PUBLISH]))){
			$publishbutton = we_html_button::create_button(we_html_button::FOOTER_PUBLISH, "javascript:workflow_finish();");
		} else {
			$table->addCol(2)
				->setCol(0, $col++, we_html_button::create_button(we_html_button::FORWARD, "javascript:pass_workflow();"));
		}

		if(we_workflow_utility::canUserEditDoc($we_doc->ID, $we_doc->Table, we_users_user::getID()) && $we_doc->userCanSave()){
			$table->addCol(2)
				->setCol(0, $col++, we_html_button::create_button(we_html_button::SAVE, "javascript:_EditorFrame.setEditorPublishWhenSave(false);we_save_document();"));
		}

		if($publishbutton){
			$table->addCol(2)
				->setCol(0, $col++, $publishbutton);
		} elseif(we_workflow_utility::canUserEditDoc($we_doc->ID, $we_doc->Table, we_users_user::getID()) && $we_doc->userCanSave()){
			if($showPubl && !($we_doc instanceof we_folder_objectFile && $we_doc->isClassFolder())){
				$table->addCol(2)
					->setCol(0, $col++, we_html_button::create_button(we_html_button::FOOTER_PUBLISH, "javascript:_EditorFrame.setEditorPublishWhenSave(true);we_save_document();"));
			}
		}

		return $table->getHtml();
	}

	private static function getFooterForSEEMMode(we_contents_root $we_doc, bool $showPubl): string{
		$col = 0;
		$table = new we_html_table(['class' => 'default'], 1, 0);

		switch($we_doc->EditPageNr){
			case we_base_constants::WE_EDITPAGE_PREVIEW:
				//	Edit-Button
				$table->addCol(2)
					->setCol(0, $col++, we_html_button::create_button(we_html_button::OPEN_TO_EDIT, "javascript:parent.editHeader.we_cmd('switch_edit_page'," . we_base_constants::WE_EDITPAGE_CONTENT . ",'" . we_base_WE::$transaction . "');"))
					//	Decline Workflow
					->addCol(2)
					->setCol(0, $col++, we_html_button::create_button(we_html_button::DECLINE, "javascript:decline_workflow();"));

				$table->addCol(2);
				if(we_workflow_utility::isWorkflowFinished($we_doc->ID, $we_doc->Table) || ((1 + we_workflow_utility::findLastActiveStep($we_doc->ID, $we_doc->Table)) == we_workflow_utility::getNumberOfSteps($we_doc->ID, $we_doc->Table) && we_base_permission::hasPerm([
						we_base_perm::WORK_CAN_PUBLISH]))){
					$table->setCol(0, $col++, we_html_button::create_button(we_html_button::FOOTER_PUBLISH, "javascript:workflow_finish();"));
				} else {
					$table->setCol(0, $col++, we_html_button::create_button(we_html_button::FORWARD, "javascript:pass_workflow();"));
				}
				break;
			case we_base_constants::WE_EDITPAGE_CONTENT:

				//	Preview Button
				$table->addCol(2)
					->setCol(0, $col++, we_html_button::create_button(we_html_button::PREVIEW, "javascript:parent.editHeader.we_cmd('switch_edit_page'," . we_base_constants::WE_EDITPAGE_PREVIEW . ",'" . we_base_WE::$transaction . "');"))
					//	Propertie-button
					->addCol(2)
					->setCol(0, $col++, we_html_button::create_button('properties', "javascript:parent.editHeader.we_cmd('switch_edit_page'," . we_base_constants::WE_EDITPAGE_PROPERTIES . ",'" . we_base_WE::$transaction . "');"))
					//	Decline Workflow
					->addCol(2)
					->setCol(0, $col++, we_html_button::create_button(we_html_button::DECLINE, "javascript:decline_workflow();"))
					->addCol(2)
					->setCol(0, $col++, (we_workflow_utility::isWorkflowFinished($we_doc->ID, $we_doc->Table) || ((1 + we_workflow_utility::findLastActiveStep($we_doc->ID, $we_doc->Table)) == we_workflow_utility::getNumberOfSteps($we_doc->ID, $we_doc->Table) && we_base_permission::hasPerm([
							we_base_perm::WORK_CAN_PUBLISH])) ?
							we_html_button::create_button(we_html_button::FOOTER_PUBLISH, "javascript:workflow_finish();") :
							we_html_button::create_button(we_html_button::FORWARD, "javascript:pass_workflow();"))
				);

				if(we_workflow_utility::canUserEditDoc($we_doc->ID, $we_doc->Table, we_users_user::getID()) && $we_doc->userCanSave()){
					$table->addCol(2)
						->setCol(0, $col++, we_html_button::create_button(we_html_button::SAVE, "javascript:_EditorFrame.setEditorPublishWhenSave(false);we_save_document();"));
					if($showPubl && (!($we_doc instanceof we_folder_objectFile) || !$we_doc->isClassFolder())){
						$table->addCol(2)
							->setCol(0, $col++, we_html_button::create_button(we_html_button::FOOTER_PUBLISH, "javascript:_EditorFrame.setEditorPublishWhenSave(true);we_save_document();"));
					}
				}
				break;

			case we_base_constants::WE_EDITPAGE_PROPERTIES:
				$table->addCol(2)
					->setCol(0, $col++, we_html_button::create_button(we_html_button::PREVIEW, "javascript:parent.editHeader.we_cmd('switch_edit_page'," . we_base_constants::WE_EDITPAGE_PREVIEW . ",'" . we_base_WE::$transaction . "');"))
					->addCol(2)
					->setCol(0, $col++, we_html_button::create_button(we_html_button::PREVIEW, "javascript:parent.editHeader.we_cmd('switch_edit_page'," . we_base_constants::WE_EDITPAGE_PREVIEW . ",'" . we_base_WE::$transaction . "');"))
					->addCol(2)
					->setCol(0, $col++, we_html_button::create_button(we_html_button::DECLINE, "javascript:decline_workflow();"))
					->addCol(2)
					->setCol(0, $col++, (we_workflow_utility::isWorkflowFinished($we_doc->ID, $we_doc->Table) || ((1 + we_workflow_utility::findLastActiveStep($we_doc->ID, $we_doc->Table)) == we_workflow_utility::getNumberOfSteps($we_doc->ID, $we_doc->Table) && we_base_permission::hasPerm([
							we_base_perm::WORK_CAN_PUBLISH])) ?
							we_html_button::create_button(we_html_button::FOOTER_PUBLISH, "javascript:workflow_finish();") :
							we_html_button::create_button(we_html_button::FORWARD, "javascript:pass_workflow();"))
				);

				if(we_workflow_utility::canUserEditDoc($we_doc->ID, $we_doc->Table, we_users_user::getID()) && $we_doc->userCanSave()){
					$table->addCol(2)
						->setCol(0, $col++, we_html_button::create_button(we_html_button::SAVE, "javascript:_EditorFrame.setEditorPublishWhenSave(false);we_save_document();"));

					if($showPubl && (!($we_doc instanceof we_folder_objectFile) || !$we_doc->isClassFolder())){
						$table->addCol(2)
							->setCol(0, $col++, we_html_button::create_button(we_html_button::FOOTER_PUBLISH, "javascript:_EditorFrame.setEditorPublishWhenSave(true);we_save_document();"));
					}
				}
		}
		return $table->getHtml();
	}

	public function getHomeScreen(): string{
		$content = we_html_button::create_button('fat:new_workflow,fa-lg fa-gears', "javascript:we_cmd('new_workflow');", '', "", !we_base_permission::hasPerm([we_base_perm::WORKFLOW_NEW]));

		return parent::getActualHomeScreen(we_modules_info::WORKFLOW, $content, '<form name="' . we_base_constants::WE_FORM . '">' . $this->getHiddens() . '</form>');
	}

}
