<?php
declare(strict_types=1);

/*
 * webEdition CMS
 *
 * $Rev: 44db4d7c3342 $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2020-06-24 18:25 +0200 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */

/**
 *
 * @return bool true if inside WE
 */
function isWE(): bool{
	return !empty($_SESSION['user']['isWeSession']);
}

/* ----------- PATH / ID functions */

function in_parentID(int $id, int $pid, string $table = FILE_TABLE): bool{
	if($pid !== 0 && $id === 0){
		return false;
	}
	if($pid === 0 || $id === $pid){
		return true;
	}

	$found = [];
	$p = $id;
	do{
		if($p == $pid){
			return true;
		}
		if(in_array($p, $found)){
			return false;
		}
		$found[] = $p;
	} while(($p = fI('SELECT ParentID FROM ' . DB_WE::$inst->escape($table) . ' WHERE ID=' . $p)));
	return false;
}

function path_to_id(string $path, string $table = FILE_TABLE): int{
	if(empty($path)){
		return 0;
	}
	$db = DB_WE::$inst;
	return fI('SELECT ID FROM ' . $db->escape($table) . ' WHERE Path="' . $db->escape($path) . '"');
}

function path_to_idA(array $path, string $table = FILE_TABLE): array{
	if(empty($path)){
		return [];
	}
	$db = DB_WE::$inst;
	return (in_array('/', $path) ? [0] : []) + $db->getAllq('SELECT ID FROM ' . $db->escape($table) . ' WHERE Path IN ("' . implode('","', array_map('escape_sql_query', array_map('trim', $path))) . '")', true);
}

/**
 * @param int $id
 * @param string $table
 * @param bool $endslash
 * @param bool $isPublished
 * @return string
 */
function id2path(int $id, string $table = FILE_TABLE, bool $endslash = false, bool $isPublished = false): string{
	return implode(',', id2pathA([$id], $table, $endslash, $isPublished));
}

/**
 * @param int[] $IDs
 * @param string $table
 * @param bool $endslash
 * @param bool $isPublished
 * @return array<int,string>
 */
function id2pathA(array $IDs, string $table = FILE_TABLE, bool $endslash = false, bool $isPublished = false): array{
	if(!$IDs){
		return [0 => '/'];
	}

	$db = DB_WE::$inst;
	switch($table){
		case CUSTOMER_TABLE :
			$select = 'Username';
			break;
		default:
			$select = ($endslash ?
				'IF(IsFolder=1,CONCAT(Path,"/"),Path)' :
				'Path');
	}

	return (in_array(0, $IDs) ? [0 => '/'] : []) +
		$db->getAllFirstq('SELECT ID,' . $select . ' FROM ' . $db->escape($table) . ' WHERE ID IN(' . implode(',', array_map('intval', $IDs)) . ')' . ($isPublished ? ' AND (IsPublished=1 OR IsFolder=1)' : ''), false)
	;
}

/**
 * here we get all folders which are in the list of given workspaces
 * @param int[] $workspaces
 * @param string $table
 * @return int[]
 */
function getAllFoldersRecursive(array $workspaces, string $table = FILE_TABLE): array{
	/* this works only if ORDER BY in derived is working, which can only be done by adding LIMIT to it; this works in mariadb - what about mysql?
	  return array_merge($workspaces, DB_WE::$inst->getAllq('SELECT ID
	  FROM
	  (SELECT ID,ParentID FROM ' . $table . ' WHERE IsFolder=1 ORDER BY ParentID,ID LIMIT 18446744073709551615) f,
	  (SELECT @pv:="' . implode(',', $workspaces) . '") init
	  WHERE
	  FIND_IN_SET(ParentID, @pv)>0 AND
	  @pv := concat(@pv, ",", ID)', true)); */
	$paths = id2pathA($workspaces, $table);
	return array_merge($workspaces, DB_WE::$inst->getAllq('SELECT ID
FROM ' . $table . '
WHERE IsFolder=1 AND (Path LIKE "' . implode('%" OR Path LIKE "', $paths) . '%")', true));
}

/* --------------- DATABASE functions -------------- */

/**
 * Strips off the table prefix - this function is save of calling multiple times
 * @param string $table
 * @return string stripped tablename
 */
function stripTblPrefix(string $table): string{
	return TBL_PREFIX != '' && (strpos($table, TBL_PREFIX) === 0) ? (substr($table, strlen(TBL_PREFIX)) ?: '') : $table;
}

/**
 *
 * @param string $table
 * @return string
 */
function addTblPrefix(string $table): string{
	return TBL_PREFIX . $table;
}

/**
  @param $query: SQL query; an empty query resets the cache
 */
function getHash(string $query = '', we_database_base $DB_WE = null, int $resultType = MYSQLI_ASSOC): array{
	return ($DB_WE ?: DB_WE::$inst)->getHash($query, $resultType);
}

function f(string $query, string $field = '', we_database_base $DB_WE = null, /* mixed */ $emptyValue = '')/* :mixed */{
	$h = ($DB_WE ?: DB_WE::$inst)->getHash($query, MYSQLI_ASSOC);
	return
		($field ?
		($h && isset($h[$field]) ? $h[$field] : $emptyValue) :
		($h ? current($h) : $emptyValue)
		);
}

function fS(string $query, string $emptyValue = ''): string{
	return (string) f($query, '', null, $emptyValue);
}

function fI(string $query, int $emptyValue = 0): int{
	return (int) f($query, '', null, $emptyValue);
}

function fB(string $query, bool $emptyValue = false): bool{
	return (bool) f($query, '', null, $emptyValue);
}

/**
 * @param array|string|bool|int|float $inp
 * @return string
 */
function escape_sql_query(/* mixed */$inp): string{
	return DB_WE::$inst->escape($inp); //@phan-suppress-current-line PhanPartialTypeMismatchArgument
}

/**
 *
 * @staticvar int $data
 * @param array|string $name
 * @return bool|array
 */
function sql_function($name){
	static $data = 0;
	if(!$data){
		$data = md5(uniqid(__FUNCTION__, true));
	}
	return (is_array($name) ? isset($name['sqlFunction']) && $name['sqlFunction'] === $data :
		['sqlFunction' => $data, 'val' => $name]);
}

/* -------- MAIL functions ---------- */

function we_mail(string $recipient, string $subject, string $txt, string $from = '', string $replyTo = ''): bool{
	if(runAtWin() && $txt){
		$txt = str_replace("\n", "\r\n", $txt);
	}
	$to = we_mail_address::parse($recipient);
	if(!$to){
		throw new we_exception_mail('no recipient given');
	}
	$from = $from ?: WE_DEFAULT_EMAIL;
	$phpmail = new we_mail_mail([$to], $subject, we_mail_address::parse($from), we_mail_address::parse($replyTo));
	$txtMail = strip_tags($txt);
	if($txt != $txtMail){
		$phpmail->setTextPartOutOfHTML($txt);
		$phpmail->addHTMLPart($txt);
	} else {
		$phpmail->addTextPart(trim($txt));
	}
	$phpmail->buildMessage();
	return $phpmail->send();
}

/**
 * Validates email address
 * @param $email
 * @return bool
 */
function we_check_email(string $email): bool{
	if(($pos = (int) strpos($email, '<'))){//check format is "xxx xx" <test@test.de> because php validates only the pure address
		++$pos;
		$email = substr($email, $pos, ((int) strrpos($email, '>')) - $pos) ?: '';
	}
	list($name, $host) = explode('@', $email);
	if(!$host){
		return false;
	}

	$host = (function_exists('idn_to_ascii') && defined('IDNA_DEFAULT') && defined('INTL_IDNA_VARIANT_UTS46') ? idn_to_ascii($host, IDNA_DEFAULT, INTL_IDNA_VARIANT_UTS46) : $host);
	return (filter_var(trim($name . '@' . $host), FILTER_VALIDATE_EMAIL) !== false);
}

function runAtWin(): bool{
	return stripos(PHP_OS, 'win') !== false && (stripos(PHP_OS, 'darwin') === false);
}

/* ----------- DEBUG functions ----------------- */

/**
 * This function is equivalent to print_r, except that it adds addtional "pre"-headers
 * @param mixed $val the variable to print
 * @param bool $html (default: true) whether to apply oldHtmlspecialchars
 * @param bool $useTA (default: false) whether output is formated as textarea
 */
function p_r(/* mixed */$val, bool $html = true, bool $useTA = false): void{
	$val = print_r($val, true);
	echo ($useTA ? '<textarea style="width:100%" rows="20">' : '<pre>') .
	($html ? htmlspecialchars($val) : $val) .
	($useTA ? '</textarea>' : '</pre>');
}

/**
 * This function triggers an error, which is logged to systemlog, and if enabled to we-log. This function can take any number of variables!
 * @param mixed $type (optional) define the type of the log; possible values are: warning (default), error, notice, deprecated
 * Note: type error causes we to stop execution, cause this is considered a major bug; but value is still logged.
 */
function t_e(/* mixed */$type = 'warning'): void{
	$inc = false;
	$data = [];
	$values = func_get_args();
	switch(is_string($type) ? strtolower($type) : -1){
		case 'error':
			$inc = true;
			$type = E_USER_ERROR;
			break;
		case 'notice':
			$inc = true;
			$type = E_USER_NOTICE;
			break;
		case 'deprecated':
			//$inc = true;
			$values[0] = 'Deprecated';
			$type = E_USER_NOTICE; //E_USER_DEPRECATED - seems not to work anymore
			break;
		case 'warning':
			$inc = true;
			$type = E_USER_WARNING;
			break;
		default:
			$type = E_USER_WARNING;
	}
	if($inc){
		array_shift($values);
	}
	foreach($values as $value){
		$data[] = strtr((is_scalar($value) ?
			(is_bool($value) ? var_export($value, true) : (string) $value) :
			print_r($value, true))
			, [chr(0) => '\0']);
	}

	if($data){
		trigger_error(implode("\n---------------------------------------------------\n", $data), $type);
	}
}

function weMemDebug(): void{
	echo 'Mem usage ' . round(((memory_get_usage() / 1024) / 1024), 3) . " MiB\n" .
	(microtime(true) - floatval($_SERVER['REQUEST_TIME_FLOAT'])) . ' ';
}

/* ---------- DOCUMENT functions ---------------- */

function we_getDocumentByID(int $id, string $includepath = ''): string{
// look what document it is and get the className
	$ct = fS('SELECT ContentType FROM ' . FILE_TABLE . ' WHERE ID=' . $id);

	if(!$ct){
		t_e('Document with ID' . $id . ' missing, or ContentType not set.', $includepath);
		t_e('error', 'Classname/ID missing');
	}
	$doc = we_base_ContentTypes::getObject($ct);
	assert($doc instanceof we_document_base);
	$doc->initByID($id);
	$content = $doc->getDocumentData($includepath);

	return $content;
}

/* ---------- LANGUAGE functions ------------- */

function register_g_l_dir(string $dir): void{
	if(empty($_SESSION['weS']['gl'])){
		$_SESSION['weS']['gl'] = [];
	}
	if(strpos($dir, WE_DOCUMENT_ROOT) !== 0){
		$dir = WE_DOCUMENT_ROOT . $dir;
	}
	$_SESSION['weS']['gl'][] = $dir;
}

/**
 * getLanguage property
 *  Note: underscores in name are used as directories - modules_workflow is searched in subdir modules
 * usage example: echo g_l('modules_workflow','[test][new]');
 *
 * @param $name string name of the variable, without this name is also used for inclusion
 * @param $specific string the array element to access
 * @param $omitErrors boolean don't throw an error on non-existent entry
 * @throws we_exception_base
 */
function g_l(string $name, string $specific, bool $omitErrors = false, bool $forHtml = false): string{
	return we_base_country::g_l($name, $specific, $omitErrors, $forHtml);
}

function show_SeoLinks(): bool{
	return (
		!(SEOINSIDE_HIDEINWEBEDITION && we_base_WE::$mainDoc['doc']->InWebEdition) &&
		!(SEOINSIDE_HIDEINEDITMODE && (we_base_WE::$editmode || we_base_WE::$mainDoc['editmode']))
		);
}

function seoIndexHide(string $basename): bool{
	return show_SeoLinks() && NAVIGATION_DIRECTORYINDEX_NAMES && in_array($basename, array_map('trim', explode(',', NAVIGATION_DIRECTORYINDEX_NAMES)), true);
}

/* ------------------ STRING functions ---------------------- */

/**
 *
 * @param string $in
 * @return string
 */
function correctUml(string $in): string{
	return we_base_util::convertDisallowedChars($in);
}

/**
 *
 * @staticvar Services_JSON $json
 * @param string $string
 * @param mixed $default
 * @param bool $quiet
 * @return array
 */
function we_unserialize(string $string, $default = [], bool $quiet = false){
	//compressed?
	if($string && $string[0] === 'x'){
		$try = @gzuncompress($string);
		$string = $try ?: $string;
	}
	//no content, return default
	if($string === ''){
		return $default;
	}
	//std-serialized data by php
	if(we_base_util::isJSONSerialize($string)){
		//maybe we should just do it & check if it failed
		if(mb_check_encoding($string, 'UTF-8')){
			return json_decode($string, true); //@phan-suppress-current-line PhanPartialTypeMismatchReturn
		}
		//FIXME: remove in cardada
		//non UTF-8 json decode
		static $json = null;
		$json = $json ?: new Services_JSON(16/* SERVICES_JSON_LOOSE_TYPE */ | Services_JSON::SERVICES_JSON_USE_NO_CHARSET_CONVERSION);
		return (array) $json->decode(str_replace("\n", '\n', $string));
	}
	if(we_base_util::isIgBinarySerialize($string)){
		return igbinary_unserialize($string);
	}
	//std-serialized data by php
	if(we_base_util::isPHPSerialize($string)){
		$ret = @unserialize($string);
		if($ret === false){
			$ret = @unserialize(utf8_decode($string));
		}
		if($ret === false){
			$ret = @unserialize(utf8_encode($string));
		}

		//unserialize failed, we try to eliminate \r which seems to be a cause for this
		if($ret === false && strlen($string) > 6){
			$string = str_replace("\r", '', $string);
			$ret = @unserialize($string);
			if($ret === false){
				$ret = @unserialize(utf8_decode($string));
			}
			if($ret === false){
				$ret = @unserialize(utf8_encode($string));
			}
		}
		if(!$quiet && $ret === false){
			t_e('unable to decode', $string);
		}
		return ($ret === false ? $default : $ret);
	}
	//json data
	if(preg_match('|^,?\d+(,\d+)*,?$|', $string)){
		return explode(',', $string);
	}
	if(!$quiet){
		t_e('unable to decode', $string);
	}
	return $default;
}

/**
 * serializes an array
 * @param array $array array to operate on
 * @param string $target can be of two types "serialize" or "json" - NO default jet (!)
 * @param bool $numeric forces data to be treated as numeric (not assotiative) array - no position data will be used
 * @param bool $ksort sort the array by key (useful when numeric is used)
 * @return string serialized data
 */
function we_serialize(array $array, string $target = SERIALIZE_JSON, bool $numeric = false, int $compression = 0, bool $ksort = false): string{
	if(!$array){
		return '';
	}
	if($ksort){
		ksort($array, SORT_NUMERIC);
	}
	$array = ($numeric ? array_values($array) : $array);

	switch($target){
		case SERIALIZE_JSON:
			//we don't encode objects as json!
			$ret = json_encode($array, JSON_UNESCAPED_UNICODE) ?: '';
			break;
		default:
		case SERIALIZE_PHP:
			$ret = function_exists('igbinary_serialize') ? (string) igbinary_serialize($array) : (serialize($array) ?: '');
			break;
		case SERIALIZE_LIST:
			$ret = (string) implode(',', $array);
			break;
	}
	return $ret === false ? '' : ($compression ? (string) (gzcompress($ret, $compression) ?? '' ) : $ret);
}

function setDynamicVar(/* mixed */$data): string{
	$ret = json_encode($data);
	if($ret){
		return base64_encode($ret);
	}
	//t_e(json_last_error_msg(), $data);
	$json = new Services_JSON(Services_JSON::SERVICES_JSON_USE_NO_CHARSET_CONVERSION);
	return base64_encode($json->encode($data, false));
}

/* ------------ SHORT CUT functions (compability) -------------- */

function we_tag(string $name, array $attribs = [], string $content = '', bool $internal = false){
	return we_weTag_util::tag($name, $attribs, $content, $internal);
}

function getContentTypeFromFile(string $filename): string{
	return we_base_ContentTypes::getFromFilename($filename);
}

/* DEPRECATED */

/**
 *
 * @param string $server
 * @param string $url
 * @param int $port
 * @param string $username
 * @param string $password
 * @param int $timeout
 * @return string
 * @deprecated since version 8.1
 */
function getHTTP(string $server, string $url, int $port = 0, string $username = '', string $password = '', int $timeout = 0): string{
	return we_http_util::getHTTP($server, $url, $port, $username, $password, $timeout);
}

/**
 * @deprecated since version 8.1
 * @param string $string
 * @param int $flags
 * @param string $encoding
 * @param bool $double_encode
 * @return string
 */
function oldHtmlspecialchars(string $string, int $flags = -1, string $encoding = 'UTF-8', bool $double_encode = true): string{//FIXME: remove in cardada
	$flags = ($flags === -1 ? ENT_COMPAT | (defined('ENT_HTML401') ? ENT_HTML401 : 0) : $flags);
	t_e('deprecated', 'calling this function is deprecated, it will be removed soon');
	return htmlspecialchars($string, $flags, $encoding, $double_encode);
}

/**
 * function of we:include
 * @param int $we_unique
 * @deprecated since version 8.1
 */
function we_resetBackVar(int $we_unique = -1): void{//FIXME:remove in cardada
	t_e('deprecated', 'calling this function is deprecated, it will be removed soon');
	we_tag_include::resetBackVar($we_unique);
}

/**
 * function of we:include
 * @deprecated since version 8.1
 */
function we_newBackVarID(): int{//FIXME:remove in cardada
	t_e('deprecated', 'calling this function is deprecated, it will be removed soon');
	return we_tag_include::newBackVarID();
}

/**
 * @deprecated since version 8.1
 */
function we_templateInit(): void{//FIXME:remove in cardada
	we_contents_template::frontEndInit(true);
}

/**
 * @deprecated since version 8.1
 */
function we_templateHead(bool $fullHeader = false): void{//FIXME:remove in cardada
	we_contents_template::frontEndHead($fullHeader);
}

/**
 * @deprecated since version 8.1
 */
function we_templatePreContent(bool $force = false): void{//FIXME:remove in cardada
	we_contents_template::frontEndPreContent($force);
}

/**
 * @deprecated since version 8.1
 */
function we_templatePostContent(bool $force = false, bool $fullPoster = false): void{//FIXME:remove in cardada
	we_contents_template::frontEndPostContent($force, $fullPoster);
}

/**
 * @deprecated since version 8.1
 */
function we_templatePost(): void{//FIXME:remove in cardada
	we_contents_template::frontEndPost();
}

/**
 * @deprecated since version 8.1
 */
function we_TemplateExit($param = 0): void{//FIXME:remove in cardada
	we_contents_template::frontEndExit($param);
}

/** This function should be used ONLY in generating code for the FRONTEND
 * @return string the html element
 * @param	string $element the elements name
 * @param	array $attribs array of attribs
 * @param	string $content content of the tag, if applicable
 * @param	boolean $forceEndTag if it should be closed, even there is no content
 * @param boolean $onlyStartTag print
 * @deprecated since version 8.1 use we_weTag_tag::getHtmlTag
 * @desc	returns the html element with the given attribs.attr[pass_*] is replaced by "*" to loop some
 *          attribs through the tagParser.
 */
function getHtmlTag(string $element, array $attribs = [], string $content = '', bool $forceEndTag = false, bool $onlyStartTag = false): string{
	t_e('deprecated', 'old call, please update your tags!');
	return we_weTag_tag::htmlTag($element, $attribs, $content, $forceEndTag, $onlyStartTag);
}

/**
 * @return array
 * @param array<string,mixed> $attribs
 * @param string[] $remove
 * @deprecated since version 8.1
 * @desc removes all entries of $attribs, where the key from attribs is in values of $remove
 */
function removeAttribs(array $attribs, array $remove = []): array{
	t_e('deprecated', 'old call, please update your tags!');
	return we_weTag_tag::removeAttribs($attribs, $remove);
}

/**
 * @deprecated since version barrhorn
 * @param $ids
 * @param string $table
 */
function id_to_path($ids, $table = FILE_TABLE){
	if(is_array($ids)){
		return id2pathA($ids, (string) $table);
	}

	return id2path((int) $ids, (string) $table);
}

/**
 * get the full name of an Attribute with applied postTagName if set
 * @param string $var
 * @return string
 */
function we_tag_getPostName(string $var): string{//fixme: remove
	t_e('deprecated', 'old call, please update your tags!');
	return we_weTag_tag::getPostName($var);
}

//tag utility functions

function getArrayValue($var, string $name, string $arrayIndex, bool $isset = false)/* :mixed */{
	t_e('deprecated', 'old call, please update your tags!');
	return we_weTag_tag::getArrayValue($var, $name, $arrayIndex, $isset);
}

/**
 * get an attribute from $attribs, and return its value according to default
 * @param string $name attributes name
 * @param array $attribs array containg the attributes
 * @param mixed $default default value
 * @return mixed returns the attributes value or default if not set
 */
function weTag_getParserAttribute(string $name, array $attribs, /* mixed */ $default = '', string $type = we_base_request::RAW, bool $isDeprecated = false)/* :mixed */{
	t_e('deprecated', 'old call, please update your tags!');
	return we_weTag_tag::getAttribute($name, $attribs, $default, $type, $isDeprecated, false);
}

/**
 * get an attribute from $attribs, and return its value according to default
 * @param string $name attributes name
 * @param array $attribs array containg the attributes
 * @param mixed $default default value
 * @param bool $useGlobal check if attribute value is a php-variable and is found in $GLOBALS
 * @return mixed returns the attributes value or default if not set
 */
function weTag_getAttribute(string $name, array $attribs, /* mixed */ $default = '', string $type = we_base_request::RAW, bool $isDeprecated = false, bool $useGlobal = true)/* :mixed */{
	t_e('deprecated', 'old call, please update your tags!');
	return we_weTag_tag::getAttribute($name, $attribs, $default, $type, $isDeprecated, $useGlobal);
}

/**
 * @deprecated since version 8.0.4
 */
function we_getDocForTag(string $docAttr, bool $maindefault = false): we_base_mainmodel{
	t_e('deprecated', 'old call, please update your tags!');
	return we_weTag_tag::getDocForTag($docAttr, $maindefault);
}

/**
 * @deprecated since version 8.0.4
 */
function modulFehltError(string $modul, string $tag): string{
	t_e('deprecated', 'old call, please update your tags!');
	return we_weTag_tag::modulMissingError($modul, $tag);
}

/**
 * @deprecated since version 8.0.4
 */
function parseError(string $text, string $extra = ''): string{
	t_e('deprecated', 'old call, please update your tags!');
	return we_weTag_tag::parseError($text, $extra);
}

/**
 * @deprecated since version 8.0.4
 */
function attributFehltError(array $attribs, $attrs, string $tag, bool $canBeEmpty = false): string{//@phan-suppress-current-line PhanUnusedGlobalFunctionParameter
	t_e('deprecated', 'old call, please update your tags!');
	try{
		we_weTag_tag::attributeMissing($attribs, [$tag => $canBeEmpty], $tag);
	} catch (we_exception_parser $e){
		return $e->getMessage();
	}
	return '';
}

/**
 * @return array
 * @param array $atts
 * @desc only uses the attribs given in the array use
 */
function useAttribs(array $atts, array $use): array{
	t_e('deprecated', 'old call, please update your tags!');
	return array_intersect_key($atts, array_flip($use));
}

function we_getInputRadioField(string $name, $value, $itsValue, $atts): string{
	t_e('deprecated', 'old call, please update your tags!');
	return we_weTag_tag::inputRadioField($name, $value, $itsValue, $atts);
}

function we_getTextareaField(string $name, $value, array $atts): string{
	t_e('deprecated', 'old call, please update your tags!');
	return we_weTag_tag::textareaField($name, $value, $atts);
}

function we_getInputTextInputField(string $name, $value, array $atts): string{
	t_e('deprecated', 'old call, please update your tags!');
	return we_weTag_tag::inputTextInputField($name, $value, $atts);
}

function we_getInputCheckboxField(string $name, $value, array $attr): string{
	t_e('deprecated', 'old call, please update your tags!');
	return we_weTag_tag::inputCheckboxField($name, $value, $attr);
}

function we_getSelectField(string $name, $value, array $values, array $attribs = [], bool $addMissing = true): string{
	t_e('deprecated', 'old call, please update your tags!');
	return we_weTag_tag::selectField($name, $value, $values, $attribs, $addMissing);
}

/**
 * @deprecated since version 8.0.4
 */
function we_pre_tag_listview(we_listview_base $newLV): void{
	t_e('deprecated', 'old call, please update your tags!');
	we_tag_listview::pre($newLV);
}

/**
 * @deprecated since version 8.0.4
 */
function we_post_tag_listview(): void{
	t_e('deprecated', 'old call, please update your tags!');
	we_tag_listview::post();
}

/**
 * @deprecated since version 8.0.4
 */
function getFieldOutLang(array $attribs, bool $fallBackBackend = false): array{
	t_e('deprecated', 'old call, please update your tags!');
	return we_weTag_tag::getOutLang($attribs, $fallBackBackend);
}

/**
 *
 * @param mixed $paths
 * @param string $table
 * @deprecated since version Barrhorn
 */
function makeIDsFromPathCVS($paths, $table = FILE_TABLE){
	if(!$paths || !$table){
		return '';
	}

	return is_array($paths) ? path_to_idA($paths, $table) : path_to_id($paths, $table);
}
