/*
 * webEdition CMS
 *
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */

/* global WE, top */
'use strict';

var
	moduleData = WE().util.getDynamicVar(document, 'loadVarShowMod', 'data-moduleData'),
	weTabs = new (WE().layout.we_tabs)(document, window),
	makeNewEntryCheck = 0,
	publishWhenSave = 0,
	weModuleWindow = true,
	current = "",
	current = moduleData.mod;

function we_cmd() {
	/*jshint validthis:true */
	var caller = (this && this.window === this ? this : window),
		args = WE().util.getWe_cmdArgsArray(Array.prototype.slice.call(arguments)),
		url = WE().util.getWe_cmdArgsUrl(args),
		doc;

	switch (args[0]) {
		case "we_selector_image":
		case "we_selector_document":
		case "we_selector_directory":
			(new (WE().util.jsOverlay)(caller, 'top',url, "we_fileselector", WE().consts.size.dialog.big, WE().consts.size.dialog.medium)).setResizeable().open();
			break;
		case 'browse_server':
			(new (WE().util.jsOverlay)(caller, 'top',url, 'browse_server', WE().consts.size.dialog.medium, WE().consts.size.dialog.medium)).setResizeable().open();
			break;
		case 'processConfirmHot':
			args.shift();
			//process direct command
			top.content.we_cmd(args.shift());
			//do commands intended
			if (args.length) {
				top.content.we_cmd.apply(caller, args);
			}
			break;
		case 'setHot':
			doc = top.content.document.getElementById("mark") ? top.content :
				(top.content.editor.edheader.document.getElementById("mark") ? top.content.editor.edheader : null);
			if (doc) {
				doc.document.getElementById("mark").style.display = "inline";
			}
			top.content.hot = true;
			break;
		case 'unsetHot':
			doc = top.content.document.getElementById("mark") ? top.content :
				(top.content.editor.edheader.document.getElementById("mark") ? top.content.editor.edheader : null);
			if (doc) {
				doc.document.getElementById("mark").style.display = "none";
			}
			top.content.hot = false;
			break;
		case 'loadTree':
			var
				clear = args[1].clear,
				items = args[1].items,
				sorted = args[1].sorted;
			if (clear) {
				top.content.treeData.clear();
				top.content.treeData.add(top.content.Node.prototype.rootEntry(0, 'root', 'root'));
			}

			for (var i = 0; i < items.length; i++) {
				if (clear || top.content.treeData.indexOfEntry(items[i].id) < 0) {
					if (sorted) {
						top.content.treeData.addSort(new top.content.Node(items[i]));
					} else {
						top.content.treeData.add(new top.content.Node(items[i]));
					}
				}
			}

			top.content.drawTree();
			break;
		case 'drawTree':
			top.content.drawTree();
			break;
		case 'makeTreeEntry':
			top.content.treeData.makeNewEntry(args[1]);
			break;
		case 'updateTreeEntry':
			top.content.treeData.updateEntry(args[1]);
			break;
		case 'deleteTreeEntry':
			top.content.treeData.deleteEntry(args[1]);
			break;
		case "exit_doc_question":
			var yes = args, no = args;
			yes[0] = 'exit_doc_question_yes';
			no[0] = 'exit_doc_question_no';
			WE().util.showConfirm(caller, "", WE().consts.g_l.alert.exit_doc_question.tools, yes, no);
			break;
		case "updateTitle":
			top.content.editor.edheader.document.getElementById("titlePath").innerText = args[1];
			break;
		case "setTool":
			current = args[1];
			break;
		case "loadHome":
			top.content.editor.location = WE().consts.dirs.WEBEDITION_DIR + "we_cmd.php?we_cmd[0]=we_showMod&mod=" + current + "&home=1&pnt=editor";
			break;
		case "revertTab":
			weTabs.setActiveTab(current);
			break;
		case "setHotTab":
			top.content.hot = false;
			setTab(args[1]);
			break;
		case "setIconOfDocClass":
		case "we_customer_selector":
		case "we_suggest_postprocessSelection":
		case "loadIfActive"://fileupload to reload tree
		case "multiedit_delAll"://e.g. from customerfilters in navigation
		case "multiedit_addItem"://e.g. from customerfilters in navigation
			top.opener.top.we_cmd.apply(caller, Array.prototype.slice.call(arguments));
			break;
		case 'close_dialog':
			WE().util.dialogTop(caller).doClose();
			break;
		default:
			WE().t_e("non explicit module command to main frame", args);
			top.opener.top.we_cmd.apply(caller, Array.prototype.slice.call(arguments));
	}
}

function setTab(module) {
	if (top.content.hot) {
		WE().util.showConfirm(window, "", WE().consts.g_l.alert.discard_changed_data, ["setHotTab", module], ["revertTab"]);
		return;
	}
	if (typeof "top.content.unsetHot" === "function") {
		top.content.unsetHot();
	}
	current = module;
	top.content.location.replace(WE().consts.dirs.WEBEDITION_DIR + "we_cmd.php?we_cmd[0]=we_showMod&mod=" + module);
}

function getFrameset() {//FIXME: we use this function temporary until frames in modules are obsolete
	return WE().consts.dirs.WEBEDITION_DIR + "we_cmd.php?we_cmd[0]=we_showMod&mod=" + current;
}
