/* global tinymce */
/**
 * webEdition CMS
 *
 * $Author: Lukas Imhof $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
'use strict';

(function () {
	var weutil = (function () {
		var global = tinymce.util.Tools.resolve('tinymce.PluginManager');

		global.add('weutil', function () {
			return {
				init: function (ed) {
					var node;
					this.editor = ed;

					ed.on('MouseUp', (function (e){
						if(ed.selection.isCollapsed() && (node = ed.dom.getParent(e.target, 'A')) && node.nodeName.toLowerCase() === 'a'){
							ed.selection.select(node);
						}
					}));

					ed.on('TextColorChange', (function (e){
						var style = ed.getParam('weWin').document.documentElement.style;
						var color, stroke, fill, height, y, rgb;

						if(e.name === 'forecolor'){
							if(e.color){
								color = e.color;
								stroke = 'none';
								fill = e.color;
								height = 3;
								y = 24;
							}else{
								color = 'black';
								stroke = 'gray';
								fill = 'none';
								height = 5;
								y = 22;
							}
							ed.settings.weColorIcon.textColor = {
								color: color,
								stroke: stroke,
								fill: fill,
								height: height,
								y: y
							};
							style.setProperty('--tinyTextColor', color);
							style.setProperty('--tinyTextColorStroke', stroke);
							style.setProperty('--tinyTextColorFill', fill);
							style.setProperty('--tinyTextColorLineheight', height);
							style.setProperty('--tinyTextColorLineY', y);
						} else if(e.name === 'backcolor'){
							if(e.color){
								rgb = new tinymce.util.Color(e.color).toRgb();
								if((Math.sqrt(0.299*Math.pow(rgb.r,2) + 0.587*Math.pow(rgb.g,2) + 0.114*Math.pow(rgb.b,2))) < 128){
									color = 'white';
								} else {
									color = 'black';
								}
								fill = e.color;
								stroke = e.color;
							}else{
								color = 'black';
								stroke = 'black';
								fill = 'white';
							}
							ed.settings.weColorIcon.backColor = {
								char: color,
								fill: fill,
								stroke: 'gray'
							};
							style.setProperty('--tinyBackColorStroke', stroke);
							style.setProperty('--tinyBackColorFill', fill);
							style.setProperty('--tinyBackColorChar', color);
						}
					}));
					ed.on('activate', (function (e){
						var style = ed.getParam('weWin').document.documentElement.style,
							textColor = ed.settings.weColorIcon.textColor,
							backColor = ed.settings.weColorIcon.backColor;

						style.setProperty('--tinyTextColor', textColor.color);
						style.setProperty('--tinyTextColorStroke', textColor.stroke);
						style.setProperty('--tinyTextColorFill', textColor.fill);
						style.setProperty('--tinyTextColorLineheight', textColor.height);
						style.setProperty('--tinyTextColorLineY', textColor.y);
						style.setProperty('--tinyBackColorChar', backColor.char);
						style.setProperty('--tinyBackColorStroke', backColor.stroke);
						style.setProperty('--tinyBackColorFill', backColor.fill);
					}));



					ed.on('NodeChange', (function (e) {
						var mapTable = {
							strong: 'weadaptbold',
							b: 'weadaptbold',
							em: 'weadaptitalic',
							i: 'weadaptitalic',
							acronym: 'weacronym',
							abbr: 'weabbr',
							img: 'weimage',
							cite: 'cite',
							del: 'del',
							ins: 'ins',
							//table: 'table'
						};

						var cm = ed.weControlManager;
						var co = ed.selection.isCollapsed();
						var n = e.element;
						n = ed.dom.getParent(n, 'STRONG,B,EM,I,ACRONYM,ABBR,IMG,SPAN,INS,DEL,CITE,TABLE');

						//list = dom.getParent(sel.getNode(), 'ol,ul');

						//FIXME: most of these are obsolete
						cm.setDisabled('weacronym', co);
						cm.setDisabled('weabbr', co);
						cm.setDisabled('welang', co);
						cm.setDisabled('cite', co);
						cm.setDisabled('del', co);
						cm.setDisabled('ins', co);
						cm.setDisabled('welink', co);
						cm.setDisabled('weunlink', true);
						cm.setDisabled('wetabledelete', true);

						cm.setActive('weacronym', false);
						cm.setActive('weabbr', false);
						cm.setActive('welang', false);
						cm.setActive('cite', false);
						cm.setActive('del', false);
						cm.setActive('ins', false);
						cm.setActive('welink', false);
						cm.setActive('weunlink', false);
						cm.setActive('weimage', false);
						cm.setActive('wetabledelete', false);
						cm.setActive('wetableinsert', false);

						if (n && n.nodeName) {
							do {
								switch (n.nodeName.toLowerCase()) {
									case 'table':
										cm.setDisabled('wetabledelete', false);
										cm.setActive('wetableinsert', true);
										break;
									case 'span':
										if (n.getAttribute('lang') && n.getAttribute('lang') !== '') {
											cm.setActive('welang', true);
											cm.setDisabled('welang', false);
										}
										/* falls through */
									default:
										if (mapTable[n.nodeName.toLowerCase()] !== undefined) {
											cm.setActive(mapTable[n.nodeName.toLowerCase()], true);
											cm.setDisabled(mapTable[n.nodeName.toLowerCase()], false);
										}
								}
							} while ((n = n.parentNode));
						}
					}));
				}
			};
		});
		function Plugin() {}

		return Plugin;
	}());
})();